import pygame, random, threading, os, sys, subprocess, socket, time, psutil, math, json
from piece import Piece, get_piece_at, load_chess_pieces
from board import Board
from chess_engine import ChessEngine
from utils import pos_to_alg, pos_from_alg
from network import network_listener, cleanup_sockets
from config import *
import style
import settings

pygame.display.set_caption("SZACHY PRZEZ SIEĆ LOKALNĄ - Motorola Science Cup 2025")

class ChessGame:
    def __init__(self, mode=None, sock=None, opponent_socket=None, nerd_mode=False, allow_undo=False,
                hints_enabled=False, allow_unlimited_undo=False):
        pygame.init()
        pygame.font.init()
        self.white_time = 300
        self.black_time = 300
        self.draw_request_pending = False
        self.draw_requester = None
        
        # Initial window dimensions
        self.min_window_size = (800, 600)
        self.window_size = self.min_window_size
        self.canvas = pygame.display.set_mode(self.window_size, pygame.RESIZABLE)
        
        # Proportions and dynamic scaling
        self.base_square_size = 80
        self.sidebar_width_ratio = 0.3
        self.nerd_panel_width_ratio = 0.25
        
        # Initialize board first with default values
        self.square_size = self.base_square_size  # Temporary value for board initialization
        self.sidebar_width = int(self.window_size[0] * self.sidebar_width_ratio)  # Temporary value
        self.dark_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]
        self.board = Board(square_size=self.square_size, sidebar_width=self.sidebar_width)
        
        # Now update dimensions
        self.update_dimensions()
        
        # Fonts (scalable)
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.4))
        self.small_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.3))
        self.button_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.2))
        self.nerd_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.25))
        self.regular_small_font = pygame.font.Font(style.MONTSERRAT_REGULAR, int(self.square_size * 0.3))
        
        self.chess_pieces = load_chess_pieces(self.square_size)
        self.load_piece_images()
        
        self.pieces = []
        self.piece_id_counter = 0
        self.turn = "white"
        self.selected_piece = None
        self.en_passant_info = None
        self.move_history = []
        self.position_counts = {}
        self.game_over = False
        self.game_over_message = ""
        self.clock = pygame.time.Clock()
        self.animating = False
        self.animating_piece = None
        self.anim_start_pos = None
        self.anim_end_pos = None
        self.anim_start_time = 0
        self.anim_duration = 500
        self.return_to_menu = False
        self.promotion_pending = False
        self.promoting_pawn = None
        self.mode = mode
        self.sock = sock
        self.opponent_socket = opponent_socket
        self.connected = bool(mode)
        self.connection_lost = False
        self.player_color = None
        self.opponent_color = None
        self.nerd_mode = nerd_mode
        self.allow_undo = allow_undo
        self.hints_enabled = hints_enabled
        self.hint_displayed = False
        self.undo_request_pending = False
        self.undo_requester = None
        self.undo_response = None
        self.ping = 0
        self.last_response_time = None
        self.local_ip = self.get_local_ip()
        if self.mode == "host" and self.opponent_socket:
            self.remote_ip = self.opponent_socket.getpeername()[0]
        elif self.mode == "client" and self.sock:
            self.remote_ip = self.sock.getpeername()[0]
        else:
            self.remote_ip = "N/A"
        self.running = True
        self.engine = ChessEngine()
        self.game_start_time = pygame.time.get_ticks()
        self.fps = 0
        self.ram_usage = 0
        self.allow_unlimited_undo = allow_unlimited_undo
        self.hint_start_pos = None
        self.hint_end_pos = None
        self.hint_piece = None
        self.last_hint_turn = None
        self.white_reaction_time = 0
        self.black_reaction_time = 0
        self.turn_start_time = pygame.time.get_ticks()
        self.last_draw_request_time = 0
        self.last_undo_request_time = 0
        self.request_cooldown = 5000
        self.BLACK = (0, 0, 0)
        self.WHITE = (255, 255, 255)
        self.HOVER_COLOR_WHITE = (220, 220, 220)
        self.HOVER_COLOR_BLACK = (50, 50, 50)
        self.SHADOW_COLOR = (150, 150, 150)
        self.setup_pieces()
        self.initialize_network()

    def update_dimensions(self):
        self.window_size = self.canvas.get_size()
        self.square_size = min(self.window_size[0] * (1 - self.sidebar_width_ratio - self.nerd_panel_width_ratio) // 8,
                               self.window_size[1] // 8)
        self.sidebar_width = int(self.window_size[0] * self.sidebar_width_ratio)
        self.nerd_panel_width = int(self.window_size[0] * self.nerd_panel_width_ratio)
        self.board.square_size = self.square_size
        self.board.sidebar_width = self.sidebar_width
        self.board.board_width = self.square_size * 8
        self.board.board_height = self.square_size * 8
        self.board.total_size = (self.board.board_width + self.sidebar_width, self.board.board_height)
        self.total_size = (self.board.total_size[0] + self.nerd_panel_width, self.board.total_size[1])
        
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.4))
        self.small_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.3))
        self.button_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.2))
        self.nerd_font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size * 0.25))
        self.regular_small_font = pygame.font.Font(style.MONTSERRAT_REGULAR, int(self.square_size * 0.3))
        
        self.chess_pieces = load_chess_pieces(self.square_size)
        self.load_piece_images()

    def load_piece_images(self):
        self.pionekBialy = self.chess_pieces["pionekBialy"]
        self.pionekCzarny = self.chess_pieces["pionekCzarny"]
        self.wiezaBiala = self.chess_pieces["wiezaBiala"]
        self.wiezaCzarna = self.chess_pieces["wiezaCzarna"]
        self.konBialy = self.chess_pieces["konBialy"]
        self.konCzarny = self.chess_pieces["konCzarny"]
        self.goniecBialy = self.chess_pieces["goniecBialy"]
        self.goniecCzarny = self.chess_pieces["goniecCzarny"]
        self.hetmanBialy = self.chess_pieces["hetmanBialy"]
        self.hetmanCzarny = self.chess_pieces["hetmanCzarny"]
        self.krolBialy = self.chess_pieces["krolBialy"]
        self.krolCzarny = self.chess_pieces["krolCzarny"]

    def cleanup_sockets(self):
        self.running = False
        time.sleep(0.2)
        if self.sock:
            try:
                self.sock.close()
            except:
                pass
            self.sock = None
        if self.opponent_socket:
            try:
                self.opponent_socket.close()
            except:
                pass
            self.opponent_socket = None
        self.connected = False

    def get_local_ip(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(("8.8.8.8", 80))
            ip = s.getsockname()[0]
        except Exception:
            ip = "N/A"
        finally:
            s.close()
        return ip

    def add_piece(self, piece_type, color, col, row, image_white, image_black):
        image = image_white if color == "white" else image_black
        self.pieces.append(Piece(self.piece_id_counter, piece_type, color, (col, row), image))
        self.piece_id_counter += 1

    def setup_pieces(self):
        self.pieces.clear()
        self.piece_id_counter = 0
        self.add_piece("rook", "white", 0, 7, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "white", 1, 7, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "white", 2, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "white", 3, 7, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "white", 4, 7, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "white", 5, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "white", 6, 7, self.konBialy, self.konCzarny)
        self.add_piece("rook", "white", 7, 7, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "white", col, 6, self.pionekBialy, self.pionekCzarny)
        self.add_piece("rook", "black", 0, 0, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "black", 1, 0, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "black", 2, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "black", 3, 0, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "black", 4, 0, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "black", 5, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "black", 6, 0, self.konBialy, self.konCzarny)
        self.add_piece("rook", "black", 7, 0, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "black", col, 1, self.pionekBialy, self.pionekCzarny)

    def initialize_network(self):
        if self.connected:
            if self.mode == "host":
                self.player_color = "white"
                self.opponent_color = "black"
                try:
                    self.opponent_socket.settimeout(30)
                    data_to_send = json.dumps({
                        "color": self.player_color,
                        "nerd_mode": int(self.nerd_mode),
                        "hints_enabled": int(self.hints_enabled),
                        "allow_undo": int(self.allow_undo)
                    }).encode()
                    self.opponent_socket.send(data_to_send)
                    response = self.opponent_socket.recv(1024).decode()
                    if response != "ACK":
                        raise Exception("Client did not acknowledge color assignment")
                    threading.Thread(target=network_listener, args=(self,), daemon=True).start()
                except Exception as e:
                    self.connection_lost = True
                    self.game_over = True
                    self.game_over_message = "Połączenie utracone! Przeciwnik rozłączony."
                    self.cleanup_sockets()
            elif self.mode == "client":
                try:
                    self.sock.settimeout(30)
                    data = self.sock.recv(1024).decode()
                    try:
                        received_data = json.loads(data)
                        self.opponent_color = received_data["color"]
                        self.player_color = "black" if self.opponent_color == "white" else "white"
                        self.nerd_mode = bool(received_data["nerd_mode"])
                        self.hints_enabled = bool(received_data["hints_enabled"])
                        self.allow_undo = bool(received_data["allow_undo"])
                        self.sock.send("ACK".encode())
                    except json.JSONDecodeError as e:
                        raise Exception(f"Invalid data received: {data}")
                    threading.Thread(target=network_listener, args=(self,), daemon=True).start()
                except Exception as e:
                    self.connection_lost = True
                    self.game_over = True
                    self.game_over_message = "Połączenie utracone! Przeciwnik rozłączony."
                    self.cleanup_sockets()
        else:
            self.player_color = "white"
            self.opponent_color = "black"
            self.board.player_color = self.player_color

    def get_position_key(self):
        board_state = []
        for piece in sorted(self.pieces, key=lambda p: (p.pos[0], p.pos[1])):
            board_state.append(f"{piece.type}{piece.color}{piece.pos}")
        return "|".join(board_state) + f"|{self.turn}|{self.en_passant_info}"

    def draw_pieces(self):
        for piece in self.pieces:
            if piece == self.animating_piece and self.animating:
                anim_col, anim_row = self.get_anim_pos()
                if self.player_color == "black":
                    anim_col = 7 - anim_col
                    anim_row = 7 - anim_row
                x = self.nerd_panel_width + anim_col * self.square_size
                y = anim_row * self.square_size
                image_rect = piece.image.get_rect()
                offset_x = (self.square_size - image_rect.width) // 2
                offset_y = (self.square_size - image_rect.height) // 2
                self.canvas.blit(piece.image, (x + offset_x, y + offset_y))
            else:
                col, row = piece.pos
                if self.player_color == "black":
                    col = 7 - col
                    row = 7 - row
                x = self.nerd_panel_width + col * self.square_size
                y = row * self.square_size
                image_rect = piece.image.get_rect()
                offset_x = (self.square_size - image_rect.width) // 2
                offset_y = (self.square_size - image_rect.height) // 2
                self.canvas.blit(piece.image, (x + offset_x, y + offset_y))

    def get_anim_pos(self):
        elapsed = pygame.time.get_ticks() - self.anim_start_time
        if elapsed >= self.anim_duration:
            self.animating = False
            return self.anim_end_pos
        progress = elapsed / self.anim_duration
        start_col, start_row = self.anim_start_pos
        end_col, end_row = self.anim_end_pos
        current_col = start_col + (end_col - start_col) * progress
        current_row = start_row + (end_row - start_row) * progress
        return (current_col, current_row)

    def surrender(self):
        self.game_over = True
        if self.player_color == "white":
            self.game_over_message = "Poddałeś się! Czarne wygrywają!"
        else:
            self.game_over_message = "Poddałeś się! Białe wygrywają!"
        if self.connected:
            try:
                socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                socket_to_use.send("SURRENDER".encode())
            except Exception as e:
                self.connection_lost = True
                self.game_over_message = "Połączenie utracone! Przeciwnik rozłączony."
                self.cleanup_sockets()

    def propose_draw(self):
        current_time = pygame.time.get_ticks()
        if self.connected:
            if not self.draw_request_pending and self.turn == self.player_color:
                if current_time - self.last_draw_request_time >= self.request_cooldown:
                    self.draw_request_pending = True
                    self.draw_requester = self.mode
                    try:
                        socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                        socket_to_use.send("DRAW_REQUEST".encode())
                        self.last_draw_request_time = current_time
                    except Exception as e:
                        self.connection_lost = True
                        self.game_over_message = "Połączenie utracone! Przeciwnik rozłączony."
                        self.cleanup_sockets()
        else:
            if self.turn == self.player_color:
                self.game_over = True
                self.game_over_message = "Remis Zaakceptowany!"

    def undo_move(self):
        if self.move_history:
            self.pieces, self.en_passant_info, self.turn, move_notation = self.move_history.pop()
            current_position = self.get_position_key()
            self.position_counts[current_position] = self.position_counts.get(current_position, 0) - 1
            if self.position_counts[current_position] <= 0:
                del self.position_counts[current_position]
            self.selected_piece = None
            self.animating = False
            self.game_over = False
            self.turn_start_time = pygame.time.get_ticks()
            self.hint_displayed = False

    def check_insufficient_material(self):
        if len(self.pieces) > 4:
            return False
        white_pieces = [p for p in self.pieces if p.color == "white"]
        black_pieces = [p for p in self.pieces if p.color == "black"]
        if len(white_pieces) == 1 and len(black_pieces) == 1:
            return True
        if len(white_pieces) == 1 and len(black_pieces) == 2:
            other_piece = next(p for p in black_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(black_pieces) == 1 and len(white_pieces) == 2:
            other_piece = next(p for p in white_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(white_pieces) == 2 and len(black_pieces) == 2:
            white_non_king = next(p for p in white_pieces if p.type != "king")
            black_non_king = next(p for p in black_pieces if p.type != "king")
            if white_non_king.type == "bishop" and black_non_king.type == "bishop":
                white_bishop_pos = white_non_king.pos
                black_bishop_pos = black_non_king.pos
                white_color = (white_bishop_pos[0] + white_bishop_pos[1]) % 2
                black_color = (black_bishop_pos[0] + black_bishop_pos[1]) % 2
                return white_color == black_color
        return False

    def handle_promotion(self, pawn):
        if self.connected and self.turn != self.player_color:
            return
        self.promotion_pending = True
        self.promoting_pawn = pawn
        promotion_options = ["queen", "rook", "bishop", "knight"]
        images = {
            "queen": self.hetmanBialy if pawn.color == "white" else self.hetmanCzarny,
            "rook": self.wiezaBiala if pawn.color == "white" else self.wiezaCzarna,
            "bishop": self.goniecBialy if pawn.color == "white" else self.goniecCzarny,
            "knight": self.konBialy if pawn.color == "white" else self.konCzarny
        }
        chosen = None
        if self.connected and self.turn == self.player_color:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(200)
            overlay.fill((100, 100, 100))
            self.canvas.blit(overlay, (0, 0))
            promotion_size = self.square_size
            total_width = promotion_size * 4
            x0 = (self.total_size[0] - total_width) // 2
            y0 = (self.total_size[1] - promotion_size) // 2
            for i, option in enumerate(promotion_options):
                rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
                pygame.draw.rect(self.canvas, (255, 255, 255), rect)
                pygame.draw.rect(self.canvas, (0, 0, 0), rect, 2)
                img = images[option]
                if img:
                    img_rect = img.get_rect()
                    offset_x = (promotion_size - img_rect.width) // 2
                    offset_y = (promotion_size - img_rect.height) // 2
                    self.canvas.blit(img, (rect.x + offset_x, rect.y + offset_y))
            pygame.display.update()
            while chosen is None:
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        self.cleanup_sockets()
                        pygame.quit()
                        sys.exit()
                    elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                        pos = event.pos
                        for i, option in enumerate(promotion_options):
                            rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
                            if rect.collidepoint(pos):
                                chosen = option
                                break
        else:
            chosen = "queen"
        pawn.type = chosen
        pawn.image = images[chosen]
        self.promotion_pending = False
        self.promoting_pawn = None

    def process_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.cleanup_sockets()
                pygame.quit()
                sys.exit()
            elif event.type == pygame.VIDEORESIZE:
                new_size = (max(event.w, self.min_window_size[0]), max(event.h, self.min_window_size[1]))
                self.canvas = pygame.display.set_mode(new_size, pygame.RESIZABLE)
                self.update_dimensions()
                self.setup_pieces()
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_pos = pygame.mouse.get_pos()
                if self.game_over:
                    if self.button_rect_back.collidepoint(mouse_pos):
                        self.cleanup_sockets()
                        pygame.quit()
                        sys.exit()
                if self.animating:
                    return
                if self.button_rect_surrender.collidepoint(mouse_pos):
                    self.surrender()
                    return
                if self.button_rect_draw.collidepoint(mouse_pos):
                    self.propose_draw()
                    return
                if self.button_rect_undo.collidepoint(mouse_pos):
                    current_time = pygame.time.get_ticks()
                    if self.connected and not self.undo_request_pending and self.allow_undo:
                        if current_time - self.last_undo_request_time >= self.request_cooldown:
                            self.undo_request_pending = True
                            self.undo_requester = self.mode
                            socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                            socket_to_use.send("UNDO_REQUEST\n".encode())
                            self.last_undo_request_time = current_time
                    elif not self.connected and self.move_history and self.allow_undo:
                        self.undo_move()
                    return
                if self.hints_enabled and self.button_rect_hint.collidepoint(
                        mouse_pos) and self.turn == self.player_color and not self.hint_displayed:
                    self.show_hint()
                    return
                if self.draw_request_pending and self.draw_requester != self.mode:
                    if hasattr(self, 'accept_button_rect') and self.accept_button_rect.collidepoint(mouse_pos):
                        self.game_over = True
                        self.game_over_message = "Remis Zaakceptowany!"
                        if self.connected:
                            socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                            socket_to_use.send("DRAW_ACCEPT\n".encode())
                        self.draw_request_pending = False
                        self.draw_requester = None
                    elif hasattr(self, 'reject_button_rect') and self.reject_button_rect.collidepoint(mouse_pos):
                        if self.connected:
                            socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                            socket_to_use.send("DRAW_REJECT\n".encode())
                        self.draw_request_pending = False
                        self.draw_requester = None
                if self.undo_request_pending and self.undo_requester != self.mode and self.allow_undo:
                    if hasattr(self, 'undo_accept_button_rect') and self.undo_accept_button_rect.collidepoint(
                            mouse_pos):
                        if self.move_history:
                            self.undo_move()
                            socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                            socket_to_use.send("UNDO_ACCEPT\n".encode())
                        self.undo_request_pending = False
                        self.undo_requester = None
                        self.undo_response = "accept"
                    elif hasattr(self, 'undo_reject_button_rect') and self.undo_reject_button_rect.collidepoint(
                            mouse_pos):
                        socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                        socket_to_use.send("UNDO_REJECT\n".encode())
                        self.undo_request_pending = False
                        self.undo_requester = None
                        self.undo_response = "reject"
                if self.undo_request_pending:
                    return
                if mouse_pos[0] < self.board.board_width + self.nerd_panel_width:
                    col = (7 - ((mouse_pos[
                                     0] - self.nerd_panel_width) // self.square_size)) if self.player_color == "black" else (
                            (mouse_pos[0] - self.nerd_panel_width) // self.square_size)
                    row = (7 - (mouse_pos[1] // self.square_size)) if self.player_color == "black" else (
                            mouse_pos[1] // self.square_size)
                    clicked_pos = (col, row)
                    if self.connected and self.player_color is None:
                        return
                    if self.turn != self.player_color:
                        return
                    if not self.animating:
                        if self.selected_piece is None:
                            piece = get_piece_at(clicked_pos, self.pieces)
                            if piece and piece.color == self.turn:
                                self.selected_piece = piece
                        else:
                            target_piece = get_piece_at(clicked_pos, self.pieces)
                            if target_piece and target_piece.color == self.turn:
                                self.selected_piece = target_piece
                            else:
                                valid_moves = self.engine.get_valid_moves(self.selected_piece, self.pieces,
                                                                          self.en_passant_info)
                                if clicked_pos in valid_moves:
                                    reaction_time = pygame.time.get_ticks() - self.turn_start_time
                                    if self.turn == "white":
                                        self.white_reaction_time = reaction_time
                                    else:
                                        self.black_reaction_time = reaction_time
                                    orig_pos = self.selected_piece.pos
                                    move_notation = pos_to_alg(orig_pos) + "-" + pos_to_alg(clicked_pos)
                                    self.move_history.append(([p.clone() for p in self.pieces], self.en_passant_info,
                                                              self.turn, move_notation))
                                    self.animating = True
                                    self.animating_piece = self.selected_piece
                                    self.anim_start_pos = orig_pos
                                    self.anim_end_pos = clicked_pos
                                    self.anim_start_time = pygame.time.get_ticks()
                                    is_en_passant = False
                                    captured_pos = None
                                    if self.selected_piece.type == "pawn" and self.en_passant_info and clicked_pos == \
                                            self.en_passant_info["target"]:
                                        is_en_passant = True
                                        captured_pos = (clicked_pos[0], orig_pos[1])
                                    self.pieces, self.en_passant_info = self.engine.simulate_move(self.selected_piece,
                                                                                                  clicked_pos,
                                                                                                  self.pieces,
                                                                                                  self.en_passant_info)
                                    moved_piece = get_piece_at(clicked_pos, self.pieces)
                                    promotion_type = None
                                    if moved_piece and moved_piece.type == "pawn":
                                        if (moved_piece.color == "white" and clicked_pos[1] == 0) or (
                                                moved_piece.color == "black" and clicked_pos[1] == 7):
                                            self.handle_promotion(moved_piece)
                                            promotion_type = moved_piece.type
                                    if self.connected:
                                        try:
                                            socket_to_use = self.opponent_socket if self.mode == "host" else self.sock
                                            move_data = {
                                                "move": move_notation,
                                                "en_passant": self.en_passant_info,
                                                "promotion": promotion_type if promotion_type else "none",
                                                "piece_id": self.selected_piece.id,
                                                "is_en_passant": is_en_passant,
                                                "captured_pos": captured_pos
                                            }
                                            socket_to_use.send(json.dumps(move_data).encode())
                                        except Exception:
                                            pass
                                    self.turn = "black" if self.turn == "white" else "white"
                                    self.turn_start_time = pygame.time.get_ticks()
                                    self.update_position_count()
                                    self.hint_displayed = False
                                    self.selected_piece = None
                                else:
                                    self.selected_piece = None

    def show_hint(self):
        self.update_hint()
        self.hint_displayed = True

    def update_position_count(self):
        current_position = self.get_position_key()
        self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1

    def update(self):
        if self.animating:
            elapsed = pygame.time.get_ticks() - self.anim_start_time
            if elapsed >= self.anim_duration:
                self.animating = False
                self.animating_piece.pos = self.anim_end_pos
        if not self.engine.has_any_valid_move(self.turn, self.pieces, self.en_passant_info):
            if self.engine.is_in_check(self.turn, self.pieces):
                self.game_over = True
                self.game_over_message = "Mat! " + ("Czarne" if self.turn == "white" else "Białe") + " wygrywają!"
            else:
                self.game_over = True
                self.game_over_message = "Pat!"
        elif self.check_insufficient_material():
            self.game_over = True
            self.game_over_message = "Remis z powodu braku materiału!"
        elif any(count >= 3 for count in self.position_counts.values()):
            self.game_over = True
            self.game_over_message = "Remis z powodu trzykrotnego powtórzenia pozycji!"

    def update_hint(self):
        if not self.hints_enabled or self.game_over or self.turn != self.player_color:
            self.hint_piece = None
            self.hint_start_pos = None
            self.hint_end_pos = None
            return

        best_piece = None
        best_move = None
        best_score = -float('inf')

        for piece in self.pieces:
            if piece.color == self.turn:
                move = self.engine.get_hint(piece, self.pieces, self.en_passant_info)
                if move:
                    sim_pieces, _ = self.engine.simulate_move(piece, move, self.pieces, self.en_passant_info)
                    score = 0
                    target = get_piece_at(move, self.pieces)
                    if target and target.color != piece.color:
                        piece_values = {"pawn": 1, "knight": 3, "bishop": 3, "rook": 5, "queen": 9}
                        score += piece_values.get(target.type, 0) * 10
                    if self.engine.is_in_check(self.turn, sim_pieces):
                        score -= 50
                    elif self.engine.is_in_check("black" if self.turn == "white" else "white", sim_pieces):
                        score += 50
                    if piece.type == "pawn" and abs(move[1] - piece.pos[1]) == 2:
                        score += 10
                    if not piece.has_moved and piece.type in ["knight", "bishop", "rook"]:
                        score += 15
                    if score > best_score:
                        best_score = score
                        best_piece = piece
                        best_move = move

        if best_piece and best_move:
            self.hint_piece = best_piece
            self.hint_start_pos = best_piece.pos
            self.hint_end_pos = best_move
            self.last_hint_turn = self.turn
        else:
            self.hint_piece = None
            self.hint_start_pos = None
            self.hint_end_pos = None

    def render(self):
        self.canvas.fill(self.dark_color)
        self.board.draw(self.canvas, self.player_color if self.player_color else "white",
                        offset_x=self.nerd_panel_width)
        self.draw_pieces()
        self.fps = self.clock.get_fps()
        process = psutil.Process()
        self.ram_usage = process.memory_info().rss / 1024 / 1024
        
        if self.nerd_mode:
            nerd_rect = pygame.Rect(0, 0, self.nerd_panel_width, self.total_size[1])
            pygame.draw.rect(self.canvas, self.dark_color, nerd_rect)
            y_offset = int(self.square_size * 0.25)
            nerd_stats = [
                f"FPS: {self.fps:.1f}",
                f"Ping: {self.ping if self.ping >= 0 else 'N/A'} ms",
                f"Lokalny IP: {self.local_ip or 'N/A'}",
                f"Zdalny IP: {self.remote_ip or 'N/A'}",
                f"Czas gry: {int((pygame.time.get_ticks() - self.game_start_time) / 1000)} s",
                f"RAM: {self.ram_usage:.2f} MB",
                f"Czas reakcji Białych: {self.white_reaction_time / 1000:.2f} s",
                f"Czas reakcji Czarnych: {self.black_reaction_time / 1000:.2f} s",
                f"Liczba ruchów: {len(self.move_history)}",
                f"Liczba figur: {len(self.pieces)}"
            ]
            for stat in nerd_stats:
                stat_text = self.nerd_font.render(stat, True, self.WHITE)
                self.canvas.blit(stat_text, (int(self.square_size * 0.125), y_offset))
                y_offset += int(self.square_size * 0.375)
        
        if self.selected_piece:
            col, row = self.selected_piece.pos
            if self.player_color == "black":
                col = 7 - col
                row = 7 - row
            rect = pygame.Rect(self.nerd_panel_width + col * self.square_size, row * self.square_size,
                               self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (0, 255, 0), rect, 3)
            valid_moves = self.engine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
            for move in valid_moves:
                m_col, m_row = move
                if self.player_color == "black":
                    m_col = 7 - m_col
                    m_row = 7 - m_row
                s = pygame.Surface((self.square_size, self.square_size))
                s.set_alpha(100)
                s.fill((255, 255, 0))
                self.canvas.blit(s, (self.nerd_panel_width + m_col * self.square_size, m_row * self.square_size))
        
        current_king = next((p for p in self.pieces if p.type == "king" and p.color == self.turn), None)
        if current_king and self.engine.is_in_check(self.turn, self.pieces):
            col, row = current_king.pos
            if self.player_color == "black":
                col = 7 - col
                row = 7 - row
            rect = pygame.Rect(self.nerd_panel_width + col * self.square_size, row * self.square_size,
                               self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (153, 0, 0), rect, 3)
            check_text = self.font.render("Szach!", True, (153, 0, 0))
            self.canvas.blit(check_text, (self.nerd_panel_width + int(self.square_size * 0.125),
                                          int(self.square_size * 0.125)))
        
        if self.hints_enabled and self.hint_displayed and self.hint_start_pos and self.hint_end_pos and self.turn == self.player_color:
            start_col, start_row = self.hint_start_pos
            end_col, end_row = self.hint_end_pos
            if self.player_color == "black":
                start_col = 7 - start_col
                start_row = 7 - start_row
                end_col = 7 - end_col
                end_row = 7 - end_row
            s_start = pygame.Surface((self.square_size, self.square_size))
            s_start.set_alpha(150)
            s_start.fill((0, 255, 0))
            self.canvas.blit(s_start,
                             (self.nerd_panel_width + start_col * self.square_size, start_row * self.square_size))
            s_end = pygame.Surface((self.square_size, self.square_size))
            s_end.set_alpha(150)
            s_end.fill((255, 255, 0))
            self.canvas.blit(s_end, (self.nerd_panel_width + end_col * self.square_size, end_row * self.square_size))
            start_x = self.nerd_panel_width + start_col * self.square_size + self.square_size // 2
            start_y = start_row * self.square_size + self.square_size // 2
            end_x = self.nerd_panel_width + end_col * self.square_size + self.square_size // 2
            end_y = end_row * self.square_size + self.square_size // 2
            pygame.draw.line(self.canvas, (255, 165, 0), (start_x, start_y), (end_x, end_y), 5)
            arrow_size = int(self.square_size * 0.25)
            angle = math.atan2(end_y - start_y, end_x - start_x)
            arrow_p1 = (
                end_x - arrow_size * math.cos(angle - math.pi / 6), end_y - arrow_size * math.sin(angle - math.pi / 6))
            arrow_p2 = (
                end_x - arrow_size * math.cos(angle + math.pi / 6), end_y - arrow_size * math.sin(angle + math.pi / 6))
            pygame.draw.polygon(self.canvas, (255, 165, 0), [(end_x, end_y), arrow_p1, arrow_p2])
        
        sidebar_rect = pygame.Rect(self.board.board_width + self.nerd_panel_width, 0, self.sidebar_width,
                                   self.board.board_height)
        pygame.draw.rect(self.canvas, self.dark_color, sidebar_rect)
        white_minutes = int(self.white_time // 60)
        white_seconds = int(self.white_time % 60)
        black_minutes = int(self.black_time // 60)
        black_seconds = int(self.black_time % 60)
        white_clock_text = self.small_font.render(f"Białe: {white_minutes:02d}:{white_seconds:02d}", True, self.WHITE)
        black_clock_text = self.small_font.render(f"Czarne: {black_minutes:02d}:{black_seconds:02d}", True, self.WHITE)
        turn_text = self.small_font.render("Tura: " + self.turn.capitalize(), True, self.WHITE)
        self.canvas.blit(white_clock_text, (self.board.board_width + self.nerd_panel_width + int(self.square_size * 0.125),
                                            int(self.square_size * 0.25)))
        self.canvas.blit(black_clock_text, (self.board.board_width + self.nerd_panel_width + int(self.square_size * 0.125),
                                            int(self.square_size * 0.625)))
        self.canvas.blit(turn_text, (self.board.board_width + self.nerd_panel_width + int(self.square_size * 0.125),
                                     int(self.square_size * 1.0)))
        moves_title = self.small_font.render("Historia Ruchów", True, self.WHITE)
        self.canvas.blit(moves_title, (self.board.board_width + self.nerd_panel_width + int(self.square_size * 0.125),
                                       int(self.square_size * 1.6)))
        
        button_width = int(self.sidebar_width * 0.9)
        button_height = int(self.square_size * 0.625)
        button_spacing = int(self.square_size * 0.125)
        total_buttons_height = (button_height + button_spacing) * (4 if self.hints_enabled else 3)
        start_y = self.board.board_height - total_buttons_height - int(self.square_size * 0.125)
        start_x = self.nerd_panel_width + self.board.board_width + int(self.sidebar_width * 0.05)
        self.button_rect_surrender = pygame.Rect(start_x, start_y, button_width, button_height)
        self.button_rect_draw = pygame.Rect(start_x, start_y + (button_height + button_spacing), button_width,
                                            button_height)
        self.button_rect_undo = pygame.Rect(start_x, start_y + 2 * (button_height + button_spacing), button_width,
                                            button_height)
        if self.hints_enabled:
            self.button_rect_hint = pygame.Rect(start_x, start_y + 3 * (button_height + button_spacing), button_width,
                                                button_height)
        mouse_pos = pygame.mouse.get_pos()
        default_color = self.WHITE if self.player_color == "white" else self.BLACK
        text_color = self.BLACK if self.player_color == "white" else self.WHITE
        hover_color = self.HOVER_COLOR_WHITE if self.player_color == "white" else self.HOVER_COLOR_BLACK
        
        surrender_hover = self.button_rect_surrender.collidepoint(mouse_pos)
        if surrender_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_surrender.move(3, 3), border_radius=12)
        pygame.draw.rect(self.canvas, hover_color if surrender_hover else default_color, self.button_rect_surrender,
                         border_radius=12)
        surrender_text = self.small_font.render("Poddaj się", True, text_color)
        self.canvas.blit(surrender_text, (
            self.button_rect_surrender.x + (self.button_rect_surrender.width - surrender_text.get_width()) // 2,
            self.button_rect_surrender.y + (self.button_rect_surrender.height - surrender_text.get_height()) // 2))
        
        draw_hover = self.button_rect_draw.collidepoint(mouse_pos)
        if draw_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_draw.move(3, 3), border_radius=12)
        pygame.draw.rect(self.canvas, hover_color if draw_hover else default_color, self.button_rect_draw,
                         border_radius=12)
        draw_text = self.small_font.render("Zaproponuj remis", True, text_color)
        self.canvas.blit(draw_text, (
            self.button_rect_draw.x + (self.button_rect_draw.width - draw_text.get_width()) // 2,
            self.button_rect_draw.y + (self.button_rect_draw.height - draw_text.get_height()) // 2))
        
        if self.allow_undo:
            undo_hover = self.button_rect_undo.collidepoint(mouse_pos)
            if undo_hover:
                pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_undo.move(3, 3), border_radius=12)
            pygame.draw.rect(self.canvas, hover_color if undo_hover else default_color, self.button_rect_undo,
                             border_radius=12)
            undo_text = self.small_font.render("Cofnij ruch", True, text_color)
            self.canvas.blit(undo_text, (
                self.button_rect_undo.x + (self.button_rect_undo.width - undo_text.get_width()) // 2,
                self.button_rect_undo.y + (self.button_rect_undo.height - undo_text.get_height()) // 2))
        
        if self.hints_enabled:
            hint_hover = self.button_rect_hint.collidepoint(mouse_pos)
            if hint_hover:
                pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_hint.move(3, 3), border_radius=12)
            pygame.draw.rect(self.canvas, hover_color if hint_hover else default_color, self.button_rect_hint,
                             border_radius=12)
            hint_text = self.small_font.render("Podpowiedź", True, text_color)
            self.canvas.blit(hint_text, (
                self.button_rect_hint.x + (self.button_rect_hint.width - hint_text.get_width()) // 2,
                self.button_rect_hint.y + (self.button_rect_hint.height - hint_text.get_height()) // 2))
        
        moves_y_offset = int(self.square_size * 1.75)
        for i, move in enumerate(self.move_history[-7:]):
            move_text = self.regular_small_font.render(move[3], True, self.WHITE)
            self.canvas.blit(move_text,
                             (self.board.board_width + self.nerd_panel_width + int(self.square_size * 0.125),
                              moves_y_offset + int(self.square_size * 0.3125) + i * int(self.square_size * 0.3125)))

        if self.undo_request_pending and self.undo_requester != self.mode and self.allow_undo:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(100)
            overlay.fill((150, 150, 150))
            self.canvas.blit(overlay, (0, 0))
            undo_prompt_rect = pygame.Rect(self.total_size[0] // 2 - int(self.square_size * 2.5),
                                          self.total_size[1] // 2 - int(self.square_size * 1.25),
                                          int(self.square_size * 5), int(self.square_size * 2.5))
            pygame.draw.rect(self.canvas, (255, 255, 255), undo_prompt_rect, border_radius=10)
            pygame.draw.rect(self.canvas, (0, 0, 0), undo_prompt_rect, 2, border_radius=10)
            prompt_text = self.small_font.render("Przeciwnik chce cofnąć ruch", True, (0, 0, 0))
            self.canvas.blit(prompt_text, (
                undo_prompt_rect.x + (undo_prompt_rect.width - prompt_text.get_width()) // 2,
                undo_prompt_rect.y + int(self.square_size * 0.25)))
            accept_button_rect = pygame.Rect(undo_prompt_rect.x + int(self.square_size * 0.625),
                                             undo_prompt_rect.y + int(self.square_size * 1.25),
                                             int(self.square_size * 1.25), int(self.square_size * 0.625))
            reject_button_rect = pygame.Rect(undo_prompt_rect.x + int(self.square_size * 3.125),
                                             undo_prompt_rect.y + int(self.square_size * 1.25),
                                             int(self.square_size * 1.25), int(self.square_size * 0.625))
            accept_hover = accept_button_rect.collidepoint(mouse_pos)
            reject_hover = reject_button_rect.collidepoint(mouse_pos)
            pygame.draw.rect(self.canvas, (0, 180, 0) if accept_hover else (0, 150, 0), accept_button_rect,
                             border_radius=10)
            pygame.draw.rect(self.canvas, (180, 0, 0) if reject_hover else (150, 0, 0), reject_button_rect,
                             border_radius=10)
            accept_text = self.button_font.render("Zaakceptuj", True, (255, 255, 255))
            reject_text = self.button_font.render("Odrzuć", True, (255, 255, 255))
            self.canvas.blit(accept_text, (
                accept_button_rect.x + (accept_button_rect.width - accept_text.get_width()) // 2,
                accept_button_rect.y + (accept_button_rect.height - accept_text.get_height()) // 2))
            self.canvas.blit(reject_text, (
                reject_button_rect.x + (reject_button_rect.width - reject_text.get_width()) // 2,
                reject_button_rect.y + (reject_button_rect.height - reject_text.get_height()) // 2))
            self.undo_accept_button_rect = accept_button_rect
            self.undo_reject_button_rect = reject_button_rect

        if self.draw_request_pending and self.draw_requester != self.mode:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(100)
            overlay.fill((150, 150, 150))
            self.canvas.blit(overlay, (0, 0))
            draw_prompt_rect = pygame.Rect(self.total_size[0] // 2 - int(self.square_size * 2.5),
                                           self.total_size[1] // 2 - int(self.square_size * 1.25),
                                           int(self.square_size * 5), int(self.square_size * 2.5))
            pygame.draw.rect(self.canvas, (255, 255, 255), draw_prompt_rect, border_radius=10)
            pygame.draw.rect(self.canvas, (0, 0, 0), draw_prompt_rect, 2, border_radius=10)
            prompt_text = self.small_font.render("Przeciwnik proponuje remis", True, (0, 0, 0))
            self.canvas.blit(prompt_text, (
                draw_prompt_rect.x + (draw_prompt_rect.width - prompt_text.get_width()) // 2,
                draw_prompt_rect.y + int(self.square_size * 0.25)))
            accept_button_rect = pygame.Rect(draw_prompt_rect.x + int(self.square_size * 0.625),
                                             draw_prompt_rect.y + int(self.square_size * 1.25),
                                             int(self.square_size * 1.25), int(self.square_size * 0.625))
            reject_button_rect = pygame.Rect(draw_prompt_rect.x + int(self.square_size * 3.125),
                                             draw_prompt_rect.y + int(self.square_size * 1.25),
                                             int(self.square_size * 1.25), int(self.square_size * 0.625))
            accept_hover = accept_button_rect.collidepoint(mouse_pos)
            reject_hover = reject_button_rect.collidepoint(mouse_pos)
            pygame.draw.rect(self.canvas, (0, 180, 0) if accept_hover else (0, 150, 0), accept_button_rect,
                             border_radius=10)
            pygame.draw.rect(self.canvas, (180, 0, 0) if reject_hover else (150, 0, 0), reject_button_rect,
                             border_radius=10)
            accept_text = self.button_font.render("Zaakceptuj", True, (255, 255, 255))
            reject_text = self.button_font.render("Odrzuć", True, (255, 255, 255))
            self.canvas.blit(accept_text, (
                accept_button_rect.x + (accept_button_rect.width - accept_text.get_width()) // 2,
                accept_button_rect.y + (accept_button_rect.height - accept_text.get_height()) // 2))
            self.canvas.blit(reject_text, (
                reject_button_rect.x + (reject_button_rect.width - reject_text.get_width()) // 2,
                reject_button_rect.y + (reject_button_rect.height - reject_text.get_height()) // 2))
            self.accept_button_rect = accept_button_rect
            self.reject_button_rect = reject_button_rect

        if self.game_over:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(200)
            overlay.fill((255, 255, 255))
            self.canvas.blit(overlay, (0, 0))
            text_surface = self.font.render(self.game_over_message, True, (0, 0, 0))
            text_rect = text_surface.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - int(self.square_size * 0.625)))
            self.canvas.blit(text_surface, text_rect)
            self.button_rect_back = pygame.Rect(self.total_size[0] // 2 - int(self.square_size * 1.25),
                                                self.total_size[1] // 2 + int(self.square_size * 0.25),
                                                int(self.square_size * 2.5), int(self.square_size * 0.5))
            back_hover = self.button_rect_back.collidepoint(mouse_pos)
            if back_hover:
                pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_back.move(3, 3), border_radius=12)
            back_color = self.HOVER_COLOR_BLACK if back_hover else self.BLACK
            pygame.draw.rect(self.canvas, back_color, self.button_rect_back, border_radius=12)
            back_text = self.small_font.render("Wyjdź", True, self.WHITE)
            self.canvas.blit(back_text, (
                self.button_rect_back.x + (self.button_rect_back.width - back_text.get_width()) // 2,
                self.button_rect_back.y + (self.button_rect_back.height - back_text.get_height()) // 2))
        pygame.display.update()

if __name__ == "__main__":
    game = ChessGame()
    while game.running:
        game.process_events()
        game.update()
        game.render()
        game.clock.tick(60)