import pygame, socket, threading, time, random, os, subprocess, sys

try:
    import style
except ImportError:
    class Style:
        WHITE = (255, 255, 255)
        MONTSERRAT_SEMI_BOLD = None
    style = Style()

try:
    from config import *
except ImportError:
    DARK_GRAY = (80, 80, 80)
    HOVER_COLOR = (50, 50, 50)
    WHITE = (255, 255, 255)
    BLACK = (0, 0, 0)

CHESS_SETS = [
    ((240, 217, 181), (181, 136, 99)),
    ((200, 200, 200), (70, 70, 70)),
    ((0, 255, 0), (100, 149, 237)),
    ((255, 182, 193), (139, 0, 139)),
    ((255, 215, 0), (255, 69, 0)),
    ((173, 216, 230), (70, 130, 180)),
    ((255, 250, 240), (105, 105, 105))
]
CURRENT_CHESS_SET = 0
CONFIG_FILE = os.path.join(os.path.expanduser("~"), ".azbest_chess")

class InitialWindow:
    def __init__(self):
        pygame.init()
        self.MIN_WIDTH, self.MIN_HEIGHT = 850, 500
        self.screen = pygame.display.set_mode((self.MIN_WIDTH, self.MIN_HEIGHT), pygame.RESIZABLE)
        pygame.display.set_caption("SZACHY PRZEZ SIEĆ LOKALNĄ - Motorola Science Cup 2025")
        self.base_font_size = 22
        self.font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, self.base_font_size)
        self.ip_text = ""
        self.input_text = ""
        self.active_ip = False
        self.active_code = False
        self.state = "choose_mode"
        self.mode = None
        self.code = None
        self.sock = None
        self.opponent_socket = None
        self.connected = False
        self.nerd_mode = False
        self.allow_undo = False
        self.hints_enabled = False
        self.allow_unlimited_undo = False
        self.running = True
        self.connection_timeout = 30
        self.host_ip = self.get_host_ip()
        self.load_settings()

    def load_settings(self):
        global CURRENT_CHESS_SET
        if os.path.exists(CONFIG_FILE):
            with open(CONFIG_FILE, "r", encoding="utf-8") as file:
                lines = file.readlines()
            if len(lines) >= 3:
                try:
                    CURRENT_CHESS_SET = int(lines[2].strip())
                except ValueError:
                    CURRENT_CHESS_SET = 0
        else:
            CURRENT_CHESS_SET = 0

    def get_host_ip(self):
        try:
            ip = socket.gethostbyname(socket.gethostname())
            if ip.startswith("127."):
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(("8.8.8.8", 80))
                ip = s.getsockname()[0]
                s.close()
            return ip
        except Exception as e:
            print(f"Nie można pobrać IP hosta: {e}")
            return "Nieznane IP"

    def draw_button(self, screen, text, x, y, w, h, color, hover_color, text_color=(255, 255, 255)):
        mouse = pygame.mouse.get_pos()
        click = pygame.mouse.get_pressed()
        rect = pygame.Rect(x, y, w, h)
        if rect.collidepoint(mouse):
            pygame.draw.rect(screen, hover_color, rect, border_radius=12)
            if click[0]:
                pygame.time.delay(150)
                return True
        else:
            pygame.draw.rect(screen, color, rect, border_radius=12)
        pygame.draw.rect(screen, WHITE, rect, 2, border_radius=12)
        text_surf = self.font.render(text, True, text_color)
        text_rect = text_surf.get_rect(center=(x + w // 2, y + h // 2))
        screen.blit(text_surf, text_rect)
        return False

    def draw_chessboard(self, screen, width, height):
        light_color, dark_color = CHESS_SETS[CURRENT_CHESS_SET]
        square_size = 50
        for y in range(0, height + square_size, square_size):
            for x in range(0, width + square_size, square_size):
                color = light_color if (x // square_size + y // square_size) % 2 == 0 else dark_color
                pygame.draw.rect(screen, color, (x, y, square_size, square_size))

    def draw(self):
        current_width, current_height = self.screen.get_size()
        scale_x = current_width / self.MIN_WIDTH
        scale_y = current_height / self.MIN_HEIGHT
        font_size = int(self.base_font_size * min(scale_x, scale_y))
        self.font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, font_size)
        
        self.draw_chessboard(self.screen, current_width, current_height)

        button_width = int(260 * scale_x)
        button_height = int(60 * scale_y)
        buttonb_width = int(260 * scale_x)
        buttonb_height = int(40 * scale_y)
        center_x = (current_width - button_width) // 2
        back_y = int(current_height - 50 * scale_y)

        if self.state == "choose_mode":
            if self.draw_button(self.screen, "Hostuj grę", center_x, int(100 * scale_y), button_width, button_height,
                                DARK_GRAY, HOVER_COLOR):
                self.start_hosting()
            if self.draw_button(self.screen, "Dołącz do gry", center_x, int(200 * scale_y), button_width, button_height,
                                DARK_GRAY, HOVER_COLOR):
                self.state = "joining"
                self.active_ip = True
            if self.draw_button(self.screen, "Wyjdź", center_x, back_y, buttonb_width, buttonb_height,
                                DARK_GRAY, HOVER_COLOR):
                pygame.quit()
                sys.exit()

        elif self.state == "hosting":
            box_rect = pygame.Rect(int(40 * scale_x), int(10 * scale_y), int(current_width - 80 * scale_x), int(120 * scale_y))
            pygame.draw.rect(self.screen, BLACK, box_rect, border_radius=10)
            pygame.draw.rect(self.screen, WHITE, box_rect, 2, border_radius=10)
            ip_text = self.font.render(f"Twój IP: {self.host_ip}", True, WHITE)
            code_text = self.font.render(f"Kod: {self.code}", True, WHITE)
            waiting_text = self.font.render("Oczekiwanie na przeciwnika...", True, WHITE)
            self.screen.blit(ip_text, (box_rect.x + int(10 * scale_x), box_rect.y + int(10 * scale_y)))
            self.screen.blit(code_text, (box_rect.x + int(10 * scale_x), box_rect.y + int(42 * scale_y)))
            self.screen.blit(waiting_text, (box_rect.x + int(10 * scale_x), box_rect.y + int(70 * scale_y)))

            nerd_y = int(150 * scale_y)
            undo_y = int(200 * scale_y)
            hints_y = int(250 * scale_y)
            if self.draw_button(self.screen, f"Tryb nerd: {'WŁ' if self.nerd_mode else 'WYŁ'}", center_x, nerd_y,
                                button_width, button_height, DARK_GRAY, HOVER_COLOR):
                self.nerd_mode = not self.nerd_mode
            if self.draw_button(self.screen, f"Cofnięcie: {'WŁ' if self.allow_undo else 'WYŁ'}", center_x, undo_y+30,
                                button_width, button_height, DARK_GRAY, HOVER_COLOR):
                self.allow_undo = not self.allow_undo
            if self.draw_button(self.screen, f"Podpowiedzi: {'WŁ' if self.hints_enabled else 'WYŁ'}", center_x, hints_y+60,
                                button_width, button_height, DARK_GRAY, HOVER_COLOR):
                self.hints_enabled = not self.hints_enabled
            if self.draw_button(self.screen, "Wyjdź", center_x, back_y, buttonb_width, buttonb_height,
                                DARK_GRAY, HOVER_COLOR):
                self.back_to_choose_mode()

        elif self.state == "joining":
            ip_label = self.font.render("Wprowadź IP hosta:", True, BLACK)
            code_label = self.font.render("Wprowadź 5-cyfrowy kod:", True, BLACK)
            ip_label_rect = ip_label.get_rect(center=(center_x + button_width // 2, int(50 * scale_y)))  # Wyśrodkowany label
            code_label_rect = code_label.get_rect(center=(center_x + button_width // 2, int(142 * scale_y)))  # Wyśrodkowany label
            self.screen.blit(ip_label, ip_label_rect)
            self.screen.blit(code_label, code_label_rect)

            # Wyśrodkowane pola inputu
            ip_box_scaled = pygame.Rect(center_x, int(80 * scale_y), int(250 * scale_x), int(40 * scale_y))
            input_box_scaled = pygame.Rect(center_x, int(170 * scale_y), int(250 * scale_x), int(40 * scale_y))
            pygame.draw.rect(self.screen, WHITE, ip_box_scaled, border_radius=5)
            pygame.draw.rect(self.screen, BLACK, ip_box_scaled, 2, border_radius=5)
            pygame.draw.rect(self.screen, WHITE, input_box_scaled, border_radius=5)
            pygame.draw.rect(self.screen, BLACK, input_box_scaled, 2, border_radius=5)
            ip_text_surf = self.font.render(self.ip_text, True, BLACK)
            code_text_surf = self.font.render(self.input_text, True, BLACK)
            self.screen.blit(ip_text_surf, (ip_box_scaled.x + int(5 * scale_x), ip_box_scaled.y + int(10 * scale_y)))
            self.screen.blit(code_text_surf, (input_box_scaled.x + int(5 * scale_x), input_box_scaled.y + int(10 * scale_y)))
            if self.active_ip:
                pygame.draw.rect(self.screen, HOVER_COLOR, ip_box_scaled, 3, border_radius=5)
            if self.active_code:
                pygame.draw.rect(self.screen, HOVER_COLOR, input_box_scaled, 3, border_radius=5)

            if self.draw_button(self.screen, "Potwierdź", center_x-5, int(230 * scale_y), button_width, button_height,
                                DARK_GRAY, HOVER_COLOR):
                self.start_joining()
            if self.draw_button(self.screen, "Wyjdź", center_x, back_y, buttonb_width, buttonb_height,
                                DARK_GRAY, HOVER_COLOR):
                self.back_to_choose_mode()

        pygame.display.flip()

    def start_hosting(self):
        self.state = "hosting"
        self.mode = "host"
        self.code = str(random.randint(10000, 99999))
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            self.sock.bind(("0.0.0.0", 12345))
            self.sock.listen(1)
            self.sock.settimeout(1)
            print(f"Hostowanie na porcie 12345 z kodem {self.code}")
            print("Serwer nasłuchuje na", self.sock.getsockname())
            threading.Thread(target=self.accept_connection, daemon=True).start()
        except Exception as e:
            print(f"Błąd hostowania: {e}")
            self.sock.close()
            self.sock = None
            self.running = False

    def accept_connection(self):
        while self.running:
            try:
                self.opponent_socket, addr = self.sock.accept()
                print(f"Klient podłączony: {addr}")
                self.opponent_socket.send(self.code.encode())
                self.opponent_socket.settimeout(5)
                response = self.opponent_socket.recv(1024).decode()
                if response == "CODE_OK":
                    self.connected = True
                    print("Połączenie nawiązane z klientem")
                    return
                else:
                    print("Nieprawidłowa odpowiedź kodu od klienta")
                    self.opponent_socket.close()
                    self.opponent_socket = None
            except socket.timeout:
                continue
            except Exception as e:
                print(f"Błąd przyjmowania połączenia: {e}")
                if self.opponent_socket:
                    self.opponent_socket.close()
                    self.opponent_socket = None
                break
        if not self.connected:
            print("Hostowanie przerwane.")
            if self.sock:
                self.sock.close()
                self.sock = None
            self.running = False

    def start_joining(self):
        if len(self.input_text) != 5 or not self.input_text.isdigit():
            print("Kod musi być 5-cyfrową liczbą!")
            return
        if not self.ip_text:
            print("Proszę wprowadzić IP hosta!")
            return
        self.state = "joining"
        self.mode = "client"
        self.code = self.input_text
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        start_time = time.time()
        print(f"Próba połączenia z {self.ip_text}:12345 z kodem {self.code}")
        while self.running and time.time() - start_time < self.connection_timeout:
            try:
                self.sock.connect((self.ip_text, 12345))
                print(f"Połączono z hostem na {self.ip_text}:12345")
                self.sock.settimeout(10)
                received_code = self.sock.recv(1024).decode()
                if received_code == self.code:
                    self.sock.send("CODE_OK".encode())
                    self.connected = True
                    print("Kod zweryfikowany, połączenie nawiązane")
                    return
                else:
                    print(f"Nieprawidłowy kod! Otrzymano: {received_code}, oczekiwano: {self.code}")
                    self.sock.close()
                    self.sock = None
                    break
            except Exception as e:
                print(f"Błąd połączenia: {e}")
                self.sock.close()
                self.sock = None
                break
        if not self.connected:
            print("Nie udało się połączyć z hostem w wyznaczonym czasie.")
            self.running = False

    def back_to_choose_mode(self):
        self.running = False
        if self.sock:
            try:
                self.sock.close()
            except:
                pass
            self.sock = None
        if self.opponent_socket:
            try:
                self.opponent_socket.close()
            except:
                pass
            self.opponent_socket = None
        self.state = "choose_mode"
        self.mode = None
        self.code = None
        self.connected = False
        self.ip_text = ""
        self.input_text = ""
        self.active_ip = False
        self.active_code = False
        self.running = True

    def run(self):
        while self.running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
                    if self.sock:
                        self.sock.close()
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    current_width, current_height = self.screen.get_size()
                    scale_x = current_width / self.MIN_WIDTH
                    scale_y = current_height / self.MIN_HEIGHT
                    button_width = int(250 * scale_x)  # Spójna szerokość z draw
                    center_x = (current_width - button_width) // 2
                    back_button_scaled = pygame.Rect(
                        center_x,
                        int(current_height - 50 * scale_y),
                        button_width,
                        int(40 * scale_y)
                    )
                    if self.state == "joining":
                        ip_box_scaled = pygame.Rect(center_x, int(80 * scale_y), int(250 * scale_x), int(40 * scale_y))
                        input_box_scaled = pygame.Rect(center_x, int(170 * scale_y), int(250 * scale_x), int(40 * scale_y))
                        if ip_box_scaled.collidepoint(event.pos):
                            self.active_ip = True
                            self.active_code = False
                        elif input_box_scaled.collidepoint(event.pos):
                            self.active_ip = False
                            self.active_code = True
                        elif back_button_scaled.collidepoint(event.pos):
                            self.back_to_choose_mode()
                        else:
                            self.active_ip = False
                            self.active_code = False
                    elif self.state == "hosting" and back_button_scaled.collidepoint(event.pos):
                        self.back_to_choose_mode()
                    elif self.state == "choose_mode" and back_button_scaled.collidepoint(event.pos):
                        pygame.quit()
                        sys.exit()
                elif event.type == pygame.KEYDOWN:
                    if self.state == "joining":
                        if self.active_ip:
                            if event.key == pygame.K_RETURN:
                                self.active_ip = False
                                self.active_code = True
                            elif event.key == pygame.K_BACKSPACE:
                                self.ip_text = self.ip_text[:-1]
                            else:
                                self.ip_text += event.unicode
                        elif self.active_code:
                            if event.key == pygame.K_RETURN:
                                self.start_joining()
                            elif event.key == pygame.K_BACKSPACE:
                                self.input_text = self.input_text[:-1]
                            elif event.unicode.isdigit() and len(self.input_text) < 5:
                                self.input_text += event.unicode
                elif event.type == pygame.VIDEORESIZE:
                    new_width = max(event.w, self.MIN_WIDTH)
                    new_height = max(event.h, self.MIN_HEIGHT)
                    self.screen = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
            self.draw()
            if self.connected:
                return self.mode, self.sock, self.opponent_socket, self.nerd_mode, self.allow_undo, self.hints_enabled, self.allow_unlimited_undo
            pygame.time.wait(10)
        return None, None, None, False, False, False, False

if __name__ == "__main__":
    window = InitialWindow()
    result = window.run()
    print(result)