import random
from utils import is_inside, pos_to_alg, alg_to_pos
from Piece import Piece

class ChessEngine:
    piece_values = {"pawn": 10, "knight": 30, "bishop": 30, "rook": 50, "queen": 90, "king": 900}
    center_squares = [(3, 3), (3, 4), (4, 3), (4, 4)]

    def __init__(self, pgn_games):
        self.pgn_games = pgn_games
        self.move_history = []
        self.en_passant_info = None

    @staticmethod
    def get_piece_at(pos, pieces_list):
        for piece in pieces_list:
            if piece.pos == pos:
                return piece
        return None

    @staticmethod
    def simulate_move(piece, move, pieces_list, en_passant_info, promotion_type=None):
        new_pieces = [p.clone() for p in pieces_list]
        moving_piece = None
        for p in new_pieces:
            if p.id == piece.id:
                moving_piece = p
                break
        if moving_piece is None:
            return new_pieces, None

        original_pos = moving_piece.pos

        if moving_piece.type == "king" and abs(move[0] - original_pos[0]) == 2:
            if move[0] > original_pos[0]:
                rook_start = (7, original_pos[1])
                rook_end = (original_pos[0] + 1, original_pos[1])
            else:
                rook_start = (0, original_pos[1])
                rook_end = (original_pos[0] - 1, original_pos[1])
            rook_found = False
            for p in new_pieces:
                if p.pos == rook_start and p.type == "rook" and p.color == moving_piece.color:
                    p.pos = rook_end
                    p.has_moved = True
                    rook_found = True
                    break
            if not rook_found:
                return pieces_list, en_passant_info
            moving_piece.pos = move
            moving_piece.has_moved = True
            new_en_passant_info = None
        else:
            is_en_passant = False
            if moving_piece.type == "pawn" and en_passant_info is not None:
                if move == en_passant_info["target"]:
                    is_en_passant = True

            if not is_en_passant:
                target = None
                for p in new_pieces:
                    if p.id != moving_piece.id and p.pos == move:
                        target = p
                        break
                if target:
                    new_pieces.remove(target)
            else:
                capture_pos = (move[0], original_pos[1])
                target = None
                for p in new_pieces:
                    if p.id != moving_piece.id and p.pos == capture_pos and p.type == "pawn":
                        target = p
                        break
                if target:
                    new_pieces.remove(target)

            moving_piece.pos = move
            new_en_passant_info = None
            if moving_piece.type == "pawn":
                if abs(move[1] - original_pos[1]) == 2:
                    passed_row = (original_pos[1] + move[1]) // 2
                    new_en_passant_info = {"target": (move[0], passed_row), "pawn_id": moving_piece.id}
                if (moving_piece.color == "white" and move[1] == 0) or (moving_piece.color == "black" and move[1] == 7):
                    moving_piece.type = promotion_type if promotion_type else "queen"
            moving_piece.has_moved = True

        return new_pieces, new_en_passant_info

    @staticmethod
    def get_valid_moves(piece, pieces_list, en_passant_info):
        candidate_moves = []
        col, row = piece.pos
        color = piece.color

        if piece.type == "pawn":
            direction = -1 if color == "white" else 1
            start_row = 6 if color == "white" else 1
            forward = (col, row + direction)
            if is_inside(*forward) and ChessEngine.get_piece_at(forward, pieces_list) is None:
                candidate_moves.append(forward)
                if row == start_row:
                    forward2 = (col, row + 2 * direction)
                    if is_inside(*forward2) and \
                       ChessEngine.get_piece_at(forward2, pieces_list) is None and \
                       ChessEngine.get_piece_at(forward, pieces_list) is None:
                        candidate_moves.append(forward2)
            for dc in [-1, 1]:
                diag = (col + dc, row + direction)
                if is_inside(*diag):
                    target = ChessEngine.get_piece_at(diag, pieces_list)
                    if target and target.color != color:
                        candidate_moves.append(diag)
            if en_passant_info is not None:
                target_square = en_passant_info["target"]
                if target_square[1] == row + direction and abs(target_square[0] - col) == 1:
                    candidate_moves.append(target_square)

        elif piece.type == "knight":
            knight_moves = [(2, 1), (2, -1), (-2, 1), (-2, -1), (1, 2), (1, -2), (-1, 2), (-1, -2)]
            for dx, dy in knight_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = ChessEngine.get_piece_at(new_pos, pieces_list)
                    if not target or target.color != color:
                        candidate_moves.append(new_pos)

        elif piece.type in ["bishop", "rook", "queen"]:
            directions = []
            if piece.type in ["rook", "queen"]:
                directions += [(1, 0), (-1, 0), (0, 1), (0, -1)]
            if piece.type in ["bishop", "queen"]:
                directions += [(1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in directions:
                new_col, new_row = col, row
                while True:
                    new_col += dx
                    new_row += dy
                    if not is_inside(new_col, new_row):
                        break
                    target = ChessEngine.get_piece_at((new_col, new_row), pieces_list)
                    if not target:
                        candidate_moves.append((new_col, new_row))
                    else:
                        if target.color != color:
                            candidate_moves.append((new_col, new_row))
                        break

        elif piece.type == "king":
            king_moves = [(1, 0), (-1, 0), (0, 1), (0, -1), (1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in king_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = ChessEngine.get_piece_at(new_pos, pieces_list)
                    if not target or target.color != color:
                        candidate_moves.append(new_pos)
            if not piece.has_moved:
                if (ChessEngine.get_piece_at((col + 1, row), pieces_list) is None and
                    ChessEngine.get_piece_at((col + 2, row), pieces_list) is None):
                    rook = ChessEngine.get_piece_at((7, row), pieces_list)
                    if rook and rook.type == "rook" and rook.color == color and not rook.has_moved:
                        safe = True
                        for intermediate in [(col, row), (col + 1, row), (col + 2, row)]:
                            sim_pieces, _ = ChessEngine.simulate_move(piece, intermediate, pieces_list, en_passant_info)
                            if ChessEngine.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((col + 2, row))
                if (ChessEngine.get_piece_at((col - 1, row), pieces_list) is None and
                    ChessEngine.get_piece_at((col - 2, row), pieces_list) is None and
                    ChessEngine.get_piece_at((col - 3, row), pieces_list) is None):
                    rook = ChessEngine.get_piece_at((0, row), pieces_list)
                    if rook and rook.type == "rook" and rook.color == color and not rook.has_moved:
                        safe = True
                        for intermediate in [(col, row), (col - 1, row), (col - 2, row)]:
                            sim_pieces, _ = ChessEngine.simulate_move(piece, intermediate, pieces_list, en_passant_info)
                            if ChessEngine.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((col - 2, row))

        valid_moves = []
        for move in candidate_moves:
            sim_pieces, sim_en_passant = ChessEngine.simulate_move(piece, move, pieces_list, en_passant_info)
            if not ChessEngine.is_in_check(color, sim_pieces):
                valid_moves.append(move)

        return valid_moves

    @staticmethod
    def is_in_check(color, pieces_list):
        king = next((p for p in pieces_list if p.type == "king" and p.color == color), None)
        if king is None:
            return True
        king_pos = king.pos
        for p in pieces_list:
            if p.color != color:
                if king_pos in p.get_attack_squares(pieces_list):
                    return True
        return False

    @staticmethod
    def has_any_valid_move(color, pieces_list, en_passant_info):
        for p in pieces_list:
            if p.color == color:
                if ChessEngine.get_valid_moves(p, pieces_list, en_passant_info):
                    return True
        return False

    @staticmethod
    def evaluate_king_safety(pieces, color):
        king = next((p for p in pieces if p.type == "king" and p.color == color), None)
        if not king:
            return 0

        safety_score = 0
        col, row = king.pos

        if color == "white" and row >= 6 and (col <= 2 or col >= 5):
            safety_score += 20
        elif color == "black" and row <= 1 and (col <= 2 or col >= 5):
            safety_score += 20

        pawn_shield = 0
        direction = -1 if color == "white" else 1
        for dc in [-1, 0, 1]:
            check_pos = (col + dc, row + direction)
            if is_inside(*check_pos):
                piece = ChessEngine.get_piece_at(check_pos, pieces)
                if piece and piece.type == "pawn" and piece.color == color:
                    pawn_shield += 5
        safety_score += pawn_shield

        return safety_score

    @staticmethod
    def evaluate_position(pieces, en_passant_info, turn, bot_color):
        if not ChessEngine.has_any_valid_move(turn, pieces, en_passant_info):
            if ChessEngine.is_in_check(turn, pieces):
                return -10000 if turn == bot_color else 10000
            else:
                return 0

        eval_val = 0

        for p in pieces:
            value = ChessEngine.piece_values.get(p.type, 0)
            if p.color == bot_color:
                eval_val += value
            else:
                eval_val -= value

        center_control = 0
        for pos in ChessEngine.center_squares:
            piece = ChessEngine.get_piece_at(pos, pieces)
            if piece:
                if piece.color == bot_color:
                    if piece.type == "pawn":
                        center_control += 10
                    else:
                        center_control += 5
                else:
                    if piece.type == "pawn":
                        center_control -= 10
                    else:
                        center_control -= 5
        eval_val += center_control

        pawn_progress = 0
        for p in pieces:
            if p.type == "pawn":
                row = p.pos[1]
                if p.color == bot_color:
                    if bot_color == "white":
                        pawn_progress += (7 - row) * 2
                    else:
                        pawn_progress += row * 2
                else:
                    if bot_color == "white":
                        pawn_progress -= row * 2
                    else:
                        pawn_progress -= (7 - row) * 2
        eval_val += pawn_progress

        bot_king_safety = ChessEngine.evaluate_king_safety(pieces, bot_color)
        opp_color = "white" if bot_color == "black" else "black"
        opp_king_safety = ChessEngine.evaluate_king_safety(pieces, opp_color)
        eval_val += bot_king_safety - opp_king_safety

        return eval_val

    @staticmethod
    def minimax(pieces, en_passant_info, turn, depth, alpha, beta, bot_color):
        if depth == 0 or not ChessEngine.has_any_valid_move(turn, pieces, en_passant_info):
            return ChessEngine.evaluate_position(pieces, en_passant_info, turn, bot_color)

        moves = []
        for p in pieces:
            if p.color == turn:
                valid_moves = ChessEngine.get_valid_moves(p, pieces, en_passant_info)
                for move in valid_moves:
                    moves.append((p, move))

        if turn == bot_color:
            max_eval = -float('inf')
            for piece, move in moves:
                new_pieces, new_en_passant = ChessEngine.simulate_move(piece, move, pieces, en_passant_info)
                new_turn = "white" if turn == "black" else "black"
                eval_val = ChessEngine.minimax(new_pieces, new_en_passant, new_turn, depth - 1, alpha, beta, bot_color)
                max_eval = max(max_eval, eval_val)
                alpha = max(alpha, eval_val)
                if beta <= alpha:
                    break
            return max_eval
        else:
            min_eval = float('inf')
            for piece, move in moves:
                new_pieces, new_en_passant = ChessEngine.simulate_move(piece, move, pieces, en_passant_info)
                new_turn = "white" if turn == "black" else "black"
                eval_val = ChessEngine.minimax(new_pieces, new_en_passant, new_turn, depth - 1, alpha, beta, bot_color)
                min_eval = min(min_eval, eval_val)
                beta = min(beta, eval_val)
                if beta <= alpha:
                    break
            return min_eval

    def parse_san_move(self, san_move, pieces, turn, en_passant_info):
        promotion = None
        if "=" in san_move:  # Obsługa promocji, np. "e8=Q"
            san_move, promotion = san_move.split("=")
        
        # Usuwamy dodatkowe znaki (np. "+", "#")
        san_move = san_move.replace("+", "").replace("#", "")
        
        # Rozpoznawanie typu figury i celu
        if san_move[0] in ["N", "B", "R", "Q", "K"]:  # Skoczek, goniec, wieża, hetman, król
            piece_type = {"N": "knight", "B": "bishop", "R": "rook", "Q": "queen", "K": "king"}[san_move[0]]
            from_hint = None
            target = san_move[1:]
            
            # Sprawdzamy, czy jest wskazówka o pozycji startowej (np. "Nbd7")
            if len(san_move) > 3 and san_move[1] in "abcdefgh":
                from_hint = san_move[1]  # np. "b" w "Nbd7"
                target = san_move[2:]
            elif len(san_move) > 4 and san_move[1] in "abcdefgh" and san_move[2] in "12345678":
                from_hint = san_move[1:3]  # np. "b6" w "Nb6d7"
                target = san_move[3:]
            
            to_pos = alg_to_pos(target)  # Konwersja np. "f6" na (5, 2)
            if to_pos is None:
                print(f"Invalid target position: {san_move}")
                return None, None, None, promotion
            
            # Znajdź figurę, która może wykonać ruch
            candidates = [p for p in pieces if p.type == piece_type and p.color == turn]
            for piece in candidates:
                valid_moves = self.get_valid_moves(piece, pieces, en_passant_info)
                if to_pos in valid_moves:
                    if from_hint:
                        from_col = ord(from_hint[0]) - ord("a") if len(from_hint) == 1 else alg_to_pos(from_hint)[0]
                        if piece.pos[0] != from_col and len(from_hint) == 1:
                            continue
                        if len(from_hint) == 2 and piece.pos != alg_to_pos(from_hint):
                            continue
                    return piece, piece.pos, to_pos, promotion
            print(f"No valid {piece_type} found for move: {san_move}")
            return None, None, None, promotion
        
        else:  # Ruch pionka, np. "e4", "exd5"
            piece_type = "pawn"
            if "x" in san_move:  # Bicie, np. "exd5"
                from_col = ord(san_move[0]) - ord("a")
                target = san_move[2:]
                to_pos = alg_to_pos(target)
                if to_pos is None:
                    print(f"Invalid target position: {san_move}")
                    return None, None, None, promotion
                candidates = [p for p in pieces if p.type == "pawn" and p.color == turn and p.pos[0] == from_col]
            else:  # Ruch bez bicia, np. "e4"
                target = san_move
                to_pos = alg_to_pos(target)
                if to_pos is None:
                    print(f"Invalid target position: {san_move}")
                    return None, None, None, promotion
                candidates = [p for p in pieces if p.type == "pawn" and p.color == turn]
            
            for piece in candidates:
                valid_moves = self.get_valid_moves(piece, pieces, en_passant_info)
                if to_pos in valid_moves:
                    return piece, piece.pos, to_pos, promotion
            print(f"No valid pawn found for move: {san_move}")
            return None, None, None, promotion

    def are_positions_equal(self, pieces1, pieces2):
        if len(pieces1) != len(pieces2):
            return False
        pos1 = sorted(f"{p.type}{p.color}{pos_to_alg(p.pos)}" for p in pieces1)
        pos2 = sorted(f"{p.type}{p.color}{pos_to_alg(p.pos)}" for p in pieces2)
        return pos1 == pos2

    def setup_initial_position(self):
        pieces = []
        piece_id = 0
        for col in range(8):
            pieces.append(Piece(piece_id, "pawn", "white", (col, 6), None))
            piece_id += 1
            pieces.append(Piece(piece_id, "pawn", "black", (col, 1), None))
            piece_id += 1
        for color, row in [("white", 7), ("black", 0)]:
            pieces.append(Piece(piece_id, "rook", color, (0, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "knight", color, (1, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "bishop", color, (2, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "queen", color, (3, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "king", color, (4, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "bishop", color, (5, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "knight", color, (6, row), None)); piece_id += 1
            pieces.append(Piece(piece_id, "rook", color, (7, row), None)); piece_id += 1
        return pieces

    def simulate_game(self, moves):
        pieces = self.setup_initial_position()
        turn = "white"
        en_passant_info = None
        for move in moves:
            piece, _, to_pos, promotion = self.parse_san_move(move, pieces, turn, en_passant_info)
            if piece and to_pos:
                pieces, en_passant_info = self.simulate_move(piece, to_pos, pieces, en_passant_info, promotion)
            turn = "black" if turn == "white" else "white"
        return pieces

    def get_pgn_move(self, pieces, turn, move_number):
        if not self.pgn_games or move_number > len(self.move_history) + 1:
            print(f"No PGN moves available: move_number={move_number}, history_len={len(self.move_history)}")
            return None, None

        current_position = "|".join(sorted(f"{p.type}{p.color}{pos_to_alg(p.pos)}" for p in pieces)) + f"|{turn}"
        print(f"DEBUG: Current position after {self.move_history}: {current_position}")

        for game_idx, game in enumerate(self.pgn_games):
            moves = game
            if len(moves) < move_number:
                print(f"DEBUG: Game {game_idx} too short: {len(moves)} < {move_number}")
                continue
            sim_pieces = self.simulate_game(moves[:move_number - 1])
            sim_position = "|".join(sorted(f"{p.type}{p.color}{pos_to_alg(p.pos)}" for p in sim_pieces)) + f"|{turn}"
            print(f"DEBUG: Game {game_idx} moves: {moves[:move_number]}")
            print(f"DEBUG: Simulated position: {sim_position}")
            print(f"DEBUG: Position match: {sim_position == current_position}")
            if sim_position == current_position:
                next_move = moves[move_number - 1]
                print(f"DEBUG: Next move from PGN: {next_move}")
                piece, from_pos, to_pos, promotion = self.parse_san_move(next_move, pieces, turn, self.en_passant_info)
                if piece and to_pos:
                    print(f"DEBUG: Parsed {next_move}: {piece.type}{piece.color} from {pos_to_alg(from_pos)} to {pos_to_alg(to_pos)}")
                    valid_moves = self.get_valid_moves(piece, pieces, self.en_passant_info)
                    print(f"DEBUG: Valid moves for {piece.type}{piece.color} at {pos_to_alg(from_pos)}: {[pos_to_alg(m) for m in valid_moves]}")
                    if to_pos in valid_moves:
                        print(f"DEBUG: Valid PGN move found: {next_move}")
                        if promotion:
                            pieces, _ = self.simulate_move(piece, to_pos, pieces, self.en_passant_info, promotion)
                        return piece, to_pos
                    else:
                        print(f"DEBUG: Move {next_move} not in valid moves")
                else:
                    print(f"DEBUG: Failed to parse move: {next_move}")
        print("DEBUG: No matching PGN move found")
        return None, None

    def get_next_pgn_moves(self, pieces, turn, move_number):
        next_moves = []
        for moves in self.pgn_games:
            sim_pieces = self.simulate_game(moves[:move_number - 1])
            if self.are_positions_equal(pieces, sim_pieces):
                if move_number - 1 < len(moves):
                    next_moves.append(moves[move_number - 1])
        return next_moves if next_moves else [self.get_pgn_move(pieces, turn, move_number)[1]]

    def get_bot_move(self, pieces, en_passant_info, turn, move_number):
            self.en_passant_info = en_passant_info
            
            # Jeśli bot gra białymi i jest to pierwszy ruch, wybierz losowy ruch z PGN
            if turn == "white" and move_number == 1 and self.pgn_games:
                # Wybierz losową grę z PGN
                random_game = random.choice(self.pgn_games)
                if random_game and len(random_game) > 0:
                    first_move = random_game[0]  # Pierwszy ruch z losowej gry
                    piece, from_pos, to_pos, promotion = self.parse_san_move(first_move, pieces, turn, en_passant_info)
                    if piece and to_pos and to_pos in self.get_valid_moves(piece, pieces, en_passant_info):
                        print(f"DEBUG: Bot wybrał losowy ruch otwierający z PGN: {first_move}")
                        return piece, to_pos
                    else:
                        print(f"DEBUG: Nie udało się sparsować lub zweryfikować losowego ruchu: {first_move}")

            # Standardowa logika dla pozostałych przypadków
            piece, move = self.get_pgn_move(pieces, turn, move_number)
            if piece and move:
                return piece, move

            # Jeśli nie ma ruchu z PGN, użyj minimax
            best_eval = -float('inf')
            best_move = None
            best_piece = None
            moves = []
            for p in pieces:
                if p.color == turn:
                    valid_moves = self.get_valid_moves(p, pieces, en_passant_info)
                    for m in valid_moves:
                        moves.append((p, m))
            for piece, move in moves:
                new_pieces, new_en_passant = self.simulate_move(piece, move, pieces, en_passant_info)
                new_turn = "white" if turn == "black" else "black"
                eval_val = self.minimax(new_pieces, new_en_passant, new_turn, 3 - 1, -float('inf'), float('inf'), turn)
                if eval_val > best_eval:
                    best_eval = eval_val
                    best_piece = piece
                    best_move = move
            return best_piece, best_move