import pygame
import sys
import os
import websocket
import json
import threading
import queue
import time
import socket
import style
import random

from utils import *
from engine import *
from Board import *
from Piece import *

# Dodajemy ścieżkę nadrzędną do sys.path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from chess_assets import load_chess_pieces, get_chess_style
import settings

# Inicjalizacja Pygame
pygame.init()
pygame.mixer.init()

pygame.display.set_caption("SZACHY ONLINE - Motorola Science Cup 2025")

# Ścieżka do pliku konfiguracyjnego
HOME_DIR = os.path.expanduser("~")
if len(sys.argv) > 1:
    CONFIG_FILE = sys.argv[1]
else:
    CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        base_path = sys._MEIPASS
    except AttributeError:
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

# Ładowanie ustawień
settings.load_settings()

WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)

class ChessGame:
    def __init__(self, screen, pgn_games=None):
        print(f"Initializing ChessGame with pgn_games type: {type(pgn_games)}")
        self.screen = screen
        self.background_surface = pygame.Surface((WIDTH, HEIGHT))
        
        pygame.init()
        pygame.font.init()
        self.base_font_size = 36
        self.update_fonts()
        self.board = Board()
        self.canvas = self.screen
        pygame.display.set_caption("Szachy vs Arcymistrz")

        self.background_surface.fill(self.board.dark_color)

        self.pgn_games = pgn_games if pgn_games is not None else load_pgn_files("games")
        if not isinstance(self.pgn_games, list):
            raise ValueError("load_pgn_files musi zwrócić listę gier PGN")
        if not self.pgn_games:
            print("Uwaga: Brak gier PGN w folderze 'games'. Bot użyje tylko minimax.")
        self.engine = ChessEngine(self.pgn_games)

        self.pieces = []
        self.piece_id_counter = 0
        self.turn = "white"
        self.en_passant_info = None
        self.selected_piece = None
        self.game_over = False
        self.game_over_message = ""
        self.clock = pygame.time.Clock()
        self.position_counts = {}
        self.move_states = []

        self.white_time = 600
        self.black_time = 600
        self.last_update_time = pygame.time.get_ticks()

        square_size = self.board.square_size
        self.chess_pieces = load_chess_pieces(square_size)

        self.piece_images = {
            "pawn": {"white": self.chess_pieces["pionekB"], "black": self.chess_pieces["pionekC"]},
            "rook": {"white": self.chess_pieces["wiezaB"], "black": self.chess_pieces["wiezaC"]},
            "knight": {"white": self.chess_pieces["konB"], "black": self.chess_pieces["konC"]},
            "bishop": {"white": self.chess_pieces["goniecB"], "black": self.chess_pieces["goniecC"]},
            "queen": {"white": self.chess_pieces["hetmanB"], "black": self.chess_pieces["hetmanC"]},
            "king": {"white": self.chess_pieces["krolB"], "black": self.chess_pieces["krolC"]}
        }

        self.bot_thinking = False
        self.bot_think_time = 0
        self.bot_move = None
        self.move_number = 1
        self.next_pgn_moves = []
        self.last_castling = None
        self.promotion_pending = False
        self.promotion_piece = None
        self.promotion_pos = None

        self.BLACK = (0, 0, 0)
        self.WHITE = (255, 255, 255)
        self.HOVER_COLOR_WHITE = (220, 220, 220)
        self.HOVER_COLOR_BLACK = (50, 50, 50)
        self.SHADOW_COLOR = (150, 150, 150)

        self.min_width = 800  # Added minimum width
        self.min_height = 700  # Added minimum height

        self.update_button_positions()

        # Ładowanie dźwięku ruchu
        try:
            self.move_sound = pygame.mixer.Sound(
                resource_path(os.path.join("assets", "audio", settings.CURRENT_SOUND))
            )
            self.move_sound.set_volume(settings.sfprev_value / 100)
        except FileNotFoundError:
            print(f"Błąd: Nie znaleziono pliku dźwiękowego {settings.CURRENT_SOUND}")
            self.move_sound = None
        except pygame.error as e:
            print(f"Błąd podczas ładowania dźwięku: {e}")
            self.move_sound = None

        self.choose_color()
        self.setup_pieces()
        self.update_position_count()
        self.save_state()

    def update_fonts(self):
        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.base_font_size * scale_factor))
        self.small_font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, int(24 * scale_factor))
        self.small_font2 = pygame.font.Font(style.MONTSERRAT_REGULAR, int(24 * scale_factor))
        self.small_button_font = pygame.font.Font(style.MONTSERRAT_MEDIUM, int(21 * scale_factor))

    def update_button_positions(self):
        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        sidebar_start = self.board.board_width
        base_button_width = 200
        base_button_height = 40
        base_margin = 50
        base_vertical_spacing = 50
        
        button_width = int(base_button_width * scale_factor)
        button_height = int(base_button_height * scale_factor)
        margin = int(base_margin * scale_factor)
        vertical_spacing = int(base_vertical_spacing * scale_factor)
        
        self.button_rect_surrender = pygame.Rect(sidebar_start + margin, 
                                               self.board.board_height - 2 * (button_height + vertical_spacing), 
                                               button_width, button_height)
        self.button_rect_undo = pygame.Rect(sidebar_start + margin, 
                                          self.board.board_height - (button_height + vertical_spacing), 
                                          button_width, button_height)

    def resize_background(self, new_width, new_height):
        """Update background surface and board when window is resized."""
        # Enforce minimum size
        new_width = max(new_width, self.min_width)
        new_height = max(new_height, self.min_height)
        
        self.background_surface = pygame.Surface((new_width, new_height))
        self.background_surface.fill(self.board.dark_color)
        self.board.resize(new_width, new_height)
        
        self.chess_pieces = load_chess_pieces(self.board.square_size)
        self.piece_images = {
            "pawn": {"white": self.chess_pieces["pionekB"], "black": self.chess_pieces["pionekC"]},
            "rook": {"white": self.chess_pieces["wiezaB"], "black": self.chess_pieces["wiezaC"]},
            "knight": {"white": self.chess_pieces["konB"], "black": self.chess_pieces["konC"]},
            "bishop": {"white": self.chess_pieces["goniecB"], "black": self.chess_pieces["goniecC"]},
            "queen": {"white": self.chess_pieces["hetmanB"], "black": self.chess_pieces["hetmanC"]},
            "king": {"white": self.chess_pieces["krolB"], "black": self.chess_pieces["krolC"]}
        }
        
        for piece in self.pieces:
            piece.image = self.piece_images[piece.type][piece.color]
        
        self.update_fonts()
        self.update_button_positions()

    def transform_pos(self, pos):
        if self.board.player_color == "black":
            return (7 - pos[0], 7 - pos[1])
        return pos

    def inverse_transform_pos(self, pos):
        if self.board.player_color == "black":
            return (7 - pos[0], 7 - pos[1])
        return pos

    def add_piece(self, piece_type, color, col, row):
        logical_pos = (col, row)
        image = self.piece_images[piece_type][color]
        piece = Piece(self.piece_id_counter, piece_type, color, logical_pos, image)
        self.pieces.append(piece)
        self.piece_id_counter += 1

    def setup_pieces(self):
        for col in range(8):
            self.add_piece("pawn", "white", col, 6)
            self.add_piece("pawn", "black", col, 1)
        for color, row in [("white", 7), ("black", 0)]:
            self.add_piece("rook", color, 0, row)
            self.add_piece("knight", color, 1, row)
            self.add_piece("bishop", color, 2, row)
            self.add_piece("queen", color, 3, row)
            self.add_piece("king", color, 4, row)
            self.add_piece("bishop", color, 5, row)
            self.add_piece("knight", color, 6, row)
            self.add_piece("rook", color, 7, row)

    def get_position_key(self):
        board_state = [f"{piece.type}{piece.color}{piece.pos}" for piece in sorted(self.pieces, key=lambda p: (p.pos[0], p.pos[1]))]
        return "|".join(board_state) + f"|{self.turn}|{self.en_passant_info}"

    def update_position_count(self):
        current_position = self.get_position_key()
        self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1

    def draw_pieces(self):
        for piece in self.pieces:
            display_pos = self.transform_pos(piece.pos)
            x = display_pos[0] * self.board.square_size
            y = display_pos[1] * self.board.square_size
            image_rect = piece.image.get_rect()
            offset_x = (self.board.square_size - image_rect.width) // 2
            offset_y = (self.board.square_size - image_rect.height) // 2
            self.canvas.blit(piece.image, (x + offset_x, y + offset_y))

    def get_piece_at(self, pos):
        return next((piece for piece in self.pieces if piece.pos == pos), None)

    def save_state(self):
        state = {
            'pieces': [piece.clone() for piece in self.pieces],
            'en_passant_info': self.en_passant_info,
            'last_castling': self.last_castling,
            'turn': self.turn,
            'move_number': self.move_number,
            'position_counts': self.position_counts.copy(),
            'white_time': self.white_time,
            'black_time': self.black_time
        }
        self.move_states.append(state)

    def surrender(self):
        self.game_over = True
        self.game_over_message = "Poddanie! " + ("Czarne" if self.turn == "white" else "Białe") + " wygrywają!"

    def process_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
                return "quit"
            elif event.type == pygame.VIDEORESIZE:
                new_width = max(event.w, self.min_width)
                new_height = max(event.h, self.min_height)
                self.resize_background(new_width, new_height)
                if event.w < self.min_width or event.h < self.min_height:
                    SCREEN = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_pos = pygame.mouse.get_pos()
                if self.game_over and hasattr(self, 'exit_button_rect') and self.exit_button_rect.collidepoint(mouse_pos):
                    pygame.quit()
                    sys.exit()
                    return "quit"
                elif not self.game_over:
                    if mouse_pos[0] >= self.board.board_width:
                        self.selected_piece = None
                        if self.promotion_pending:
                            self.handle_promotion_click(mouse_pos)
                        elif self.button_rect_surrender.collidepoint(mouse_pos) and not self.promotion_pending:
                            self.surrender()
                        elif self.button_rect_undo.collidepoint(mouse_pos) and not self.promotion_pending and self.turn == self.board.player_color:
                            self.undo_move()
                        continue
                    col, row = mouse_pos[0] // self.board.square_size, mouse_pos[1] // self.board.square_size
                    clicked_pos = self.inverse_transform_pos((col, row))

                    if self.turn == self.board.player_color and not self.promotion_pending:
                        if not self.selected_piece:
                            piece = self.get_piece_at(clicked_pos)
                            if piece and piece.color == self.turn:
                                self.selected_piece = piece
                        else:
                            target_piece = self.get_piece_at(clicked_pos)
                            if target_piece and target_piece.color == self.turn:
                                self.selected_piece = target_piece
                            else:
                                valid_moves = ChessEngine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
                                if clicked_pos in valid_moves:
                                    orig_pos = self.selected_piece.pos
                                    move_notation = self.handle_move(self.selected_piece, clicked_pos)
                                    self.selected_piece = None
                                    self.render()
                                    self.turn = "white" if self.board.player_color == "black" else "black"
                                    self.move_number += 1
                                    self.next_pgn_moves = self.engine.get_next_pgn_moves(self.pieces, self.turn, self.move_number)
                                    self.update_position_count()
                                    if self.move_sound:
                                        self.move_sound.play()
                                else:
                                    self.selected_piece = None
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_LEFT and not self.game_over and not self.promotion_pending:
                if self.engine.move_history and self.turn == self.board.player_color:
                    self.undo_move()
        return None

    def generate_san_move(self, piece, from_pos, to_pos, target_piece):
        if piece.type == "pawn":
            if target_piece:
                return f"{pos_to_alg(from_pos)[0]}x{pos_to_alg(to_pos)}"
            return pos_to_alg(to_pos)
        else:
            piece_symbol = {"knight": "N", "bishop": "B", "rook": "R", "queen": "Q", "king": "K"}[piece.type]
            disambiguate = ""
            others = [p for p in self.pieces if p.type == piece.type and p.color == piece.color and p != piece and to_pos in ChessEngine.get_valid_moves(p, self.pieces, self.en_passant_info)]
            if others:
                if all(p.pos[0] != from_pos[0] for p in others):
                    disambiguate = pos_to_alg(from_pos)[0]
                elif all(p.pos[1] != from_pos[1] for p in others):
                    disambiguate = pos_to_alg(from_pos)[1]
                else:
                    disambiguate = pos_to_alg(from_pos)
            return f"{piece_symbol}{disambiguate}{'x' if target_piece else ''}{pos_to_alg(to_pos)}"

    def handle_move(self, piece, dest_pos):
        orig_pos = piece.pos
        target_piece = self.get_piece_at(dest_pos)
        
        if piece.type == "king" and abs(dest_pos[0] - orig_pos[0]) == 2:
            move_notation = "O-O" if dest_pos[0] > orig_pos[0] else "O-O-O"
            self.last_castling = (orig_pos, dest_pos)
            self.pieces, self.en_passant_info = ChessEngine.simulate_move(piece, dest_pos, self.pieces, self.en_passant_info)
        elif piece.type == "pawn" and ((piece.color == "white" and dest_pos[1] == 0) or (piece.color == "black" and dest_pos[1] == 7)):
            self.promotion_pending = True
            self.promotion_piece = piece
            self.promotion_pos = dest_pos
            move_notation = pos_to_alg(orig_pos) + pos_to_alg(dest_pos)
            if target_piece and target_piece.color != piece.color:
                self.pieces = [p for p in self.pieces if p != target_piece]
        else:
            move_notation = self.generate_san_move(piece, orig_pos, dest_pos, target_piece)
            self.last_castling = None
            self.pieces, self.en_passant_info = ChessEngine.simulate_move(piece, dest_pos, self.pieces, self.en_passant_info)
        
        if move_notation not in self.engine.move_history[-1:]:
            self.engine.move_history.append(move_notation)
        self.save_state()
        return move_notation

    def handle_promotion_click(self, mouse_pos):
        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        base_spacing = 60
        spacing = int(base_spacing * scale_factor)
        
        promotion_options = [
            ("queen", self.board.board_width + 20, 200, self.piece_images["queen"][self.promotion_piece.color]),
            ("rook", self.board.board_width + 20 + spacing, 200, self.piece_images["rook"][self.promotion_piece.color]),
            ("bishop", self.board.board_width + 20 + 2 * spacing, 200, self.piece_images["bishop"][self.promotion_piece.color]),
            ("knight", self.board.board_width + 20 + 3 * spacing, 200, self.piece_images["knight"][self.promotion_piece.color])
        ]
        for option, x, y, image in promotion_options:
            rect = pygame.Rect(x, y, self.board.square_size, self.board.square_size)
            if rect.collidepoint(mouse_pos):
                orig_pos = self.promotion_piece.pos
                promotion_notation = {"queen": "Q", "rook": "R", "bishop": "B", "knight": "N"}[option]
                target_piece = self.get_piece_at(self.promotion_pos)
                move_notation = f"{pos_to_alg(orig_pos)}{'x' if target_piece else ''}{pos_to_alg(self.promotion_pos)}={promotion_notation}"
                
                self.pieces = [piece for piece in self.pieces if piece != self.promotion_piece]
                self.add_piece(option, self.promotion_piece.color, self.promotion_pos[0], self.promotion_pos[1])
                self.en_passant_info = None
                self.promotion_pending = False
                self.promotion_piece = None
                self.promotion_pos = None
                
                self.engine.move_history[-1] = move_notation
                self.turn = "white" if self.board.player_color == "black" else "black"
                self.move_number += 1
                self.next_pgn_moves = self.engine.get_next_pgn_moves(self.pieces, self.turn, self.move_number)
                self.update_position_count()
                self.save_state()
                break

    def undo_move(self):
        if len(self.move_states) < 3:
            return
        
        self.move_states.pop()
        self.move_states.pop()
        
        if len(self.engine.move_history) >= 2:
            self.engine.move_history.pop()
            self.engine.move_history.pop()
        
        last_state = self.move_states[-1]
        self.pieces = [piece.clone() for piece in last_state['pieces']]
        self.en_passant_info = last_state['en_passant_info']
        self.last_castling = last_state['last_castling']
        self.turn = last_state['turn']
        self.move_number = last_state['move_number']
        self.position_counts = last_state['position_counts'].copy()
        self.white_time = last_state['white_time']
        self.black_time = last_state['black_time']
        self.next_pgn_moves = self.engine.get_next_pgn_moves(self.pieces, self.turn, self.move_number)
        self.piece_id_counter = max([p.id for p in self.pieces], default=-1) + 1
        
        for piece in self.pieces:
            piece.image = self.piece_images[piece.type][piece.color]

    def check_insufficient_material(self):
        if len(self.pieces) > 4:
            return False
        white_pieces = [p for p in self.pieces if p.color == "white"]
        black_pieces = [p for p in self.pieces if p.color == "black"]
        if len(white_pieces) == 1 and len(black_pieces) == 1:
            return True
        if len(white_pieces) == 1 and len(black_pieces) == 2:
            other_piece = next(p for p in black_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(black_pieces) == 1 and len(white_pieces) == 2:
            other_piece = next(p for p in white_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(white_pieces) == 2 and len(black_pieces) == 2:
            white_non_king = next(p for p in white_pieces if p.type != "king")
            black_non_king = next(p for p in black_pieces if p.type != "king")
            if white_non_king.type == "bishop" and black_non_king.type == "bishop":
                white_color = (white_non_king.pos[0] + white_non_king.pos[1]) % 2
                black_color = (black_non_king.pos[0] + black_non_king.pos[1]) % 2
                return white_color == black_color
        return False

    def format_time(self, seconds):
        minutes = int(seconds // 60)
        seconds = int(seconds % 60)
        return f"{minutes:02d}:{seconds:02d}"

    def choose_best_promotion(self, pawn, promotion_pos):
        bot_color = pawn.color
        opponent_color = "white" if bot_color == "black" else "black"
        promotion_options = ["queen", "rook", "bishop", "knight"]
        best_option = "queen"
        best_score = -float('inf')

        temp_pieces = [piece.clone() for piece in self.pieces if piece != pawn]
        target_piece = self.get_piece_at(promotion_pos)
        if target_piece and target_piece.color != pawn.color:
            temp_pieces = [p for p in temp_pieces if p != target_piece]

        for option in promotion_options:
            new_piece = Piece(self.piece_id_counter, option, bot_color, promotion_pos, self.piece_images[option][bot_color])
            sim_pieces = temp_pieces + [new_piece]

            if not ChessEngine.has_any_valid_move(opponent_color, sim_pieces, self.en_passant_info):
                if ChessEngine.is_in_check(opponent_color, sim_pieces):
                    return option

            score = ChessEngine.minimax(sim_pieces, self.en_passant_info, opponent_color, 2, -float('inf'), float('inf'), bot_color)
            if option == "knight":
                valid_moves = ChessEngine.get_valid_moves(new_piece, sim_pieces, self.en_passant_info)
                for move in valid_moves:
                    if ChessEngine.get_piece_at(move, sim_pieces) and ChessEngine.get_piece_at(move, sim_pieces).type == "king":
                        score += 50
                    elif ChessEngine.get_piece_at(move, sim_pieces) and ChessEngine.get_piece_at(move, sim_pieces).type in ["queen", "rook"]:
                        score += 20

            if score > best_score:
                best_score = score
                best_option = option

        return best_option

    def update(self):
        player_color = self.board.player_color
        bot_color = "white" if player_color == "black" else "black"

        if not self.game_over and not self.promotion_pending and not self.bot_thinking:
            current_time = pygame.time.get_ticks()
            elapsed = (current_time - self.last_update_time) / 1000
            self.last_update_time = current_time
            if self.turn == "white":
                self.white_time = max(0, self.white_time - elapsed)
                if self.white_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Czarne wygrywają!"
            else:
                self.black_time = max(0, self.black_time - elapsed)
                if self.black_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Białe wygrywają!"
                    

        if self.turn == bot_color and not self.game_over and not self.bot_thinking and not self.promotion_pending:
            self.bot_thinking = True
            self.bot_think_time = 500
            self.bot_move = self.engine.get_bot_move(self.pieces, self.en_passant_info, self.turn, self.move_number)

        if self.bot_thinking and self.bot_think_time <= 0 and not self.game_over:
            bot_piece, bot_move_pos = self.bot_move
            if bot_piece and bot_move_pos:
                orig_pos = bot_piece.pos
                if bot_piece.type == "king" and abs(bot_move_pos[0] - orig_pos[0]) == 2:
                    move_notation = "O-O" if bot_move_pos[0] > orig_pos[0] else "O-O-O"
                    self.last_castling = (orig_pos, bot_move_pos)
                    self.pieces, self.en_passant_info = ChessEngine.simulate_move(bot_piece, bot_move_pos, self.pieces, self.en_passant_info)
                elif bot_piece.type == "pawn" and ((bot_piece.color == "white" and bot_move_pos[1] == 0) or 
                                                (bot_piece.color == "black" and bot_move_pos[1] == 7)):
                    promotion_choice = self.choose_best_promotion(bot_piece, bot_move_pos)
                    promotion_notation = {"queen": "Q", "rook": "R", "bishop": "B", "knight": "N"}[promotion_choice]
                    target_piece = self.get_piece_at(bot_move_pos)
                    move_notation = f"{pos_to_alg(orig_pos)}{'x' if target_piece else ''}{pos_to_alg(bot_move_pos)}={promotion_notation}"
                    if target_piece and target_piece.color != bot_piece.color:
                        self.pieces = [piece for piece in self.pieces if piece != target_piece]
                    self.pieces = [piece for piece in self.pieces if piece != bot_piece]
                    self.add_piece(promotion_choice, bot_piece.color, bot_move_pos[0], bot_move_pos[1])
                    self.last_castling = None
                else:
                    move_notation = self.generate_san_move(bot_piece, orig_pos, bot_move_pos, self.get_piece_at(bot_move_pos))
                    self.last_castling = None
                    self.pieces, self.en_passant_info = ChessEngine.simulate_move(bot_piece, bot_move_pos, self.pieces, self.en_passant_info)
                
                self.engine.move_history.append(move_notation)
                if self.move_sound:
                    try:
                        self.move_sound.play()
                    except pygame.error as e:
                        print(f"Błąd podczas odtwarzania dźwięku: {e}")
                self.turn = player_color
                self.move_number += 1
                self.next_pgn_moves = self.engine.get_next_pgn_moves(self.pieces, self.turn, self.move_number)
                self.update_position_count()
                self.save_state()
            self.bot_thinking = False
            self.bot_move = None
            
        if not ChessEngine.has_any_valid_move(self.turn, self.pieces, self.en_passant_info):
            self.game_over = True
            self.game_over_message = "Mat! Wygrywają " + ("białe" if self.turn == "black" else "czarni") if ChessEngine.is_in_check(self.turn, self.pieces) else "Pat!"
        elif self.check_insufficient_material():
            self.game_over = True
            self.game_over_message = "Remis przez brak wystarczającego materiału!"
        elif any(count >= 3 for count in self.position_counts.values()):
            self.game_over = True
            self.game_over_message = "Remis przez trzykrotne powtórzenie pozycji!"

    def render(self):
        current_size = self.screen.get_size()
        if current_size != self.background_surface.get_size():
            self.resize_background(*current_size)
        
        self.screen.blit(self.background_surface, (0, 0))
        self.board.draw(self.canvas, self.board.player_color)
        self.draw_pieces()

        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        base_margin = 20
        margin = int(base_margin * scale_factor)
        base_move_spacing = 25
        move_spacing = int(base_move_spacing * scale_factor)

        if self.selected_piece and self.turn == self.board.player_color and not self.promotion_pending:
            display_pos = self.transform_pos(self.selected_piece.pos)
            col, row = display_pos
            pygame.draw.rect(self.canvas, (0, 255, 0), (col * self.board.square_size, row * self.board.square_size, self.board.square_size, self.board.square_size), 3)
            for move in ChessEngine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info):
                m_display_pos = self.transform_pos(move)
                m_col, m_row = m_display_pos
                s = pygame.Surface((self.board.square_size, self.board.square_size))
                s.set_alpha(128)
                s.fill((255, 255, 0))
                self.canvas.blit(s, (m_col * self.board.square_size, m_row * self.board.square_size))
                pygame.draw.circle(self.canvas, (255, 255, 0), (m_col * self.board.square_size + self.board.square_size // 2, m_row * self.board.square_size + self.board.square_size // 2), int(10 * scale_factor))

        if self.last_castling:
            king_start, king_end = self.last_castling
            display_king_start = self.transform_pos(king_start)
            display_king_end = self.transform_pos(king_end)
            rook_start = (7 if king_end[0] > king_start[0] else 0, king_start[1])
            rook_end = (king_start[0] + 1 if king_end[0] > king_start[0] else king_start[0] - 1, king_start[1])
            display_rook_start = self.transform_pos(rook_start)
            display_rook_end = self.transform_pos(rook_end)
            for pos in [display_king_start, display_king_end, display_rook_start, display_rook_end]:
                pygame.draw.rect(self.canvas, (0, 255, 255), (pos[0] * self.board.square_size, pos[1] * self.board.square_size, self.board.square_size, self.board.square_size), 2)

        current_king = next((p for p in self.pieces if p.type == "king" and p.color == self.turn), None)
        if current_king and ChessEngine.is_in_check(self.turn, self.pieces):
            king_display_pos = self.transform_pos(current_king.pos)
            king_col, king_row = king_display_pos
            pygame.draw.rect(self.canvas, (255, 0, 0), (king_col * self.board.square_size, king_row * self.board.square_size, self.board.square_size, self.board.square_size), 4)
            check_text = self.font.render("Szach!", True, (255, 0, 0))
            self.canvas.blit(check_text, (self.board.board_width // 2 - check_text.get_width() // 2, int(10 * scale_factor)))

        sidebar_rect = pygame.Rect(self.board.board_width, 0, self.board.sidebar_width, self.board.board_height)
        pygame.draw.rect(self.canvas, self.board.dark_color, sidebar_rect)

        turn_text = self.small_font.render(f"Tura: {'Białe' if self.turn == 'white' else 'Czarne'}", True, self.WHITE)
        self.canvas.blit(turn_text, (self.board.board_width + margin, margin))

        moves_title = self.small_font.render("Historia ruchów:", True, self.WHITE)
        self.canvas.blit(moves_title, (self.board.board_width + margin, int(50 * scale_factor)))
        
        history = self.engine.move_history[-20:]
        move_y = int(80 * scale_factor)
        for i in range(0, len(history), 2):
            white_move = history[i]
            black_move = history[i+1] if i+1 < len(history) else ""
            white_text = self.small_font2.render(white_move, True, self.WHITE)
            self.canvas.blit(white_text, (self.board.board_width + int(60 * scale_factor), move_y))
            if black_move:
                black_text = self.small_font2.render(black_move, True, self.WHITE)
                self.canvas.blit(black_text, (self.board.board_width + int(140 * scale_factor), move_y))
            move_y += move_spacing

        if self.next_pgn_moves and self.turn == self.board.player_color:
            pgn_title = self.small_font.render("Następne ruchy z PGN:", True, self.WHITE)
            self.canvas.blit(pgn_title, (self.board.board_width + margin, int(330 * scale_factor)))
            for i, move in enumerate(self.next_pgn_moves[:5]):
                pgn_text = self.small_font.render(f"{i+1}. {move}", True, self.WHITE)
                self.canvas.blit(pgn_text, (self.board.board_width + margin, int(360 * scale_factor) + i * move_spacing))

        mouse_pos = pygame.mouse.get_pos()
        surrender_hover = self.button_rect_surrender.collidepoint(mouse_pos) and not self.game_over and not self.promotion_pending
        undo_hover = self.button_rect_undo.collidepoint(mouse_pos) and not self.game_over and not self.promotion_pending and self.turn == self.board.player_color

        player_default_color = self.WHITE if self.board.player_color == "white" else self.BLACK
        player_text_color = self.BLACK if self.board.player_color == "white" else self.WHITE
        player_hover_color = self.HOVER_COLOR_WHITE if self.board.player_color == "white" else self.HOVER_COLOR_BLACK

        if surrender_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_surrender.move(int(3 * scale_factor), int(3 * scale_factor)), border_radius=int(12 * scale_factor))
        surrender_color = player_hover_color if surrender_hover else player_default_color
        pygame.draw.rect(self.canvas, surrender_color, self.button_rect_surrender, border_radius=int(12 * scale_factor))
        surrender_text = self.small_button_font.render("Poddaj się", True, player_text_color)
        self.canvas.blit(surrender_text, (
            self.button_rect_surrender.x + (self.button_rect_surrender.width - surrender_text.get_width()) // 2,
            self.button_rect_surrender.y + (self.button_rect_surrender.height - surrender_text.get_height()) // 2
        ))

        if undo_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_undo.move(int(3 * scale_factor), int(3 * scale_factor)), border_radius=int(12 * scale_factor))
        undo_color = player_hover_color if undo_hover else player_default_color
        pygame.draw.rect(self.canvas, undo_color, self.button_rect_undo, border_radius=int(12 * scale_factor))
        undo_text = self.small_button_font.render("Cofnij", True, player_text_color)
        self.canvas.blit(undo_text, (
            self.button_rect_undo.x + (self.button_rect_undo.width - undo_text.get_width()) // 2,
            self.button_rect_undo.y + (self.button_rect_undo.height - undo_text.get_height()) // 2
        ))

        if self.promotion_pending:
            self.draw_promotion_menu()

        if self.game_over:
            overlay = pygame.Surface(self.board.total_size)
            overlay.set_alpha(200)
            overlay.fill((150, 150, 150))
            self.canvas.blit(overlay, (0, 0))
            text_surface = self.font.render(self.game_over_message, True, (255, 0, 0))
            text_rect = text_surface.get_rect(center=(self.board.total_size[0] // 2, self.board.total_size[1] // 2 - int(30 * scale_factor)))
            self.canvas.blit(text_surface, text_rect)

            # Dodanie przycisku "Wyjdź"
            exit_button_width = int(200 * scale_factor)
            exit_button_height = int(40 * scale_factor)
            exit_button_rect = pygame.Rect(
                self.board.total_size[0] // 2 - exit_button_width // 2,
                self.board.total_size[1] // 2 + int(20 * scale_factor),
                exit_button_width,
                exit_button_height
            )
            
            exit_hover = exit_button_rect.collidepoint(mouse_pos)
            exit_color = self.HOVER_COLOR_WHITE if exit_hover else self.WHITE
            pygame.draw.rect(self.canvas, exit_color, exit_button_rect, border_radius=int(12 * scale_factor))
            exit_text = self.small_button_font.render("Wyjdź", True, self.BLACK)
            self.canvas.blit(exit_text, (
                exit_button_rect.x + (exit_button_rect.width - exit_text.get_width()) // 2,
                exit_button_rect.y + (exit_button_rect.height - exit_text.get_height()) // 2
            ))
            
            # Obsługa kliknięcia przycisku "Wyjdź" w metodzie process_events
            self.exit_button_rect = exit_button_rect

        pygame.display.update()

    def draw_promotion_menu(self):
        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        base_margin = 10
        base_spacing = 60
        margin = int(base_margin * scale_factor)
        spacing = int(base_spacing * scale_factor)
        
        overlay = pygame.Surface(self.board.total_size)
        overlay.set_alpha(150)
        overlay.fill((100, 100, 100))
        self.canvas.blit(overlay, (0, 0))
        promo_text = self.font.render("Wybierz figurę promocji:", True, (255, 255, 255))
        self.canvas.blit(promo_text, (self.board.board_width + margin, int(150 * scale_factor)))

        promotion_options = [
            ("queen", self.piece_images["queen"][self.promotion_piece.color]),
            ("rook", self.piece_images["rook"][self.promotion_piece.color]),
            ("bishop", self.piece_images["bishop"][self.promotion_piece.color]),
            ("knight", self.piece_images["knight"][self.promotion_piece.color])
        ]
        for i, (_, image) in enumerate(promotion_options):
            x, y = self.board.board_width + margin + i * spacing, int(200 * scale_factor)
            self.canvas.blit(image, (x, y))
            pygame.draw.rect(self.canvas, (255, 255, 255), (x, y, self.board.square_size, self.board.square_size), int(2 * scale_factor))

    def choose_color(self):
        BG_COLOR = (50, 50, 50)
        WHITE = (255, 255, 255)
        BLACK = (0, 0, 0)
        HOVER_COLOR = (150, 150, 150)
        SELECTED_COLOR = (81, 83, 85)

        scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
        base_button_width = 300
        base_button_height = 60
        base_button_spacing = 20
        base_start_y = 200
        
        button_width = int(base_button_width * scale_factor)
        button_height = int(base_button_height * scale_factor)
        button_spacing = int(base_button_spacing * scale_factor)
        start_y = int(base_start_y * scale_factor)
        
        screen_center_x = self.screen.get_width() // 2
        buttons = [
            pygame.Rect(screen_center_x - button_width // 2, start_y, button_width, button_height),
            pygame.Rect(screen_center_x - button_width // 2, start_y + button_height + button_spacing, button_width, button_height),
            pygame.Rect(screen_center_x - button_width // 2, start_y + 2 * (button_height + button_spacing), button_width, button_height)
        ]
        options = ["Biały", "Czarny", "Losuj"]
        colors = ['white', 'black', None]
        button_colors = [WHITE, BLACK, (100, 100, 100)]
        text_colors = [BLACK, WHITE, WHITE]
        
        RECT_COLOR = (50, 50, 50)
        base_rect_size = (500, 550)
        rect_size = (int(base_rect_size[0] * scale_factor), int(base_rect_size[1] * scale_factor))
        rect_x = (self.screen.get_width() - rect_size[0]) // 2
        rect_y = (self.screen.get_height() - rect_size[1]) // 2

        chosen = False
        while not chosen:
            settings.draw_chessboard(SCREEN, SCREEN.get_width(), SCREEN.get_height())
            pygame.draw.rect(SCREEN, RECT_COLOR, [rect_x, rect_y, *rect_size], border_radius=int(10 * scale_factor))
            mouse_pos = pygame.mouse.get_pos()

            title = self.font.render("Wybierz kolor", True, WHITE)
            title_x = screen_center_x - title.get_width() // 2
            title_y = int(100 * scale_factor)
            self.canvas.blit(title, (title_x, title_y))

            for i, button in enumerate(buttons):
                hover = button.collidepoint(mouse_pos)
                color = HOVER_COLOR if hover else button_colors[i]
                pygame.draw.rect(self.canvas, color, button, border_radius=int(10 * scale_factor))
                pygame.draw.rect(self.canvas, WHITE, button, int(2 * scale_factor), border_radius=int(10 * scale_factor))

                text = self.small_font.render(options[i], True, text_colors[i])
                text_rect = text.get_rect(center=button.center)
                self.canvas.blit(text, text_rect)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.VIDEORESIZE:
                    self.resize_background(event.w, event.h)
                    scale_factor = min(self.screen.get_width() / WIDTH, self.screen.get_height() / HEIGHT)
                    button_width = int(base_button_width * scale_factor)
                    button_height = int(base_button_height * scale_factor)
                    button_spacing = int(base_button_spacing * scale_factor)
                    start_y = int(base_start_y * scale_factor)
                    screen_center_x = self.screen.get_width() // 2
                    buttons = [
                        pygame.Rect(screen_center_x - button_width // 2, start_y, button_width, button_height),
                        pygame.Rect(screen_center_x - button_width // 2, start_y + button_height + button_spacing, button_width, button_height),
                        pygame.Rect(screen_center_x - button_width // 2, start_y + 2 * (button_height + button_spacing), button_width, button_height)
                    ]
                    rect_size = (int(base_rect_size[0] * scale_factor), int(base_rect_size[1] * scale_factor))
                    rect_x = (self.screen.get_width() - rect_size[0]) // 2
                    rect_y = (self.screen.get_height() - rect_size[1]) // 2
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    for i, button in enumerate(buttons):
                        if button.collidepoint(mouse_pos):
                            if i == 2:
                                self.board.player_color = random.choice(['white', 'black'])
                            else:
                                self.board.player_color = colors[i]
                            chosen = True
                            pygame.mixer.music.unload()
                            pygame.draw.rect(self.canvas, SELECTED_COLOR, button, border_radius=int(10 * scale_factor))
                            text = self.small_font.render(options[i], True, text_colors[i])
                            self.canvas.blit(text, text.get_rect(center=button.center))
                            pygame.display.update()
                            pygame.time.wait(200)
                            if self.board.player_color == "black":
                                self.turn = "white"

            self.screen.blit(pygame.transform.scale(self.canvas, self.screen.get_size()), (0, 0))
            pygame.display.update()

    def run(self):
        if self.turn == "white" and self.board.player_color == "black":
            self.bot_thinking = True
            self.bot_think_time = 500
            self.bot_move = self.engine.get_bot_move(self.pieces, self.en_passant_info, self.turn, self.move_number)
        
        while True:
            result = self.process_events()
            if result == "quit":
                return "quit"
            elif result == "main_menu":
                return "main_menu"
            if not self.game_over and not self.promotion_pending:
                self.update()
            self.render()
            delta = self.clock.tick(60)
            if self.bot_thinking:
                self.bot_think_time -= delta

if __name__ == "__main__":
    pgn_games = load_pgn_files("games")
    print(f"Loaded pgn_games: {pgn_games}, type: {type(pgn_games)}")
    game = ChessGame(SCREEN, pgn_games)
    game.run()