import pygame
import sys
import os
import importlib.util
try:
    import psutil
    import websocket
except ImportError:
    print("MODUŁY NIE TAK")


sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
import settings
import gamemodes
import style

# Funkcja do uzyskania poprawnej ścieżki zasobów
def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        base_path = sys._MEIPASS
    except AttributeError:
        # W trybie developerskim użyj ścieżki względem lokalizacji pliku
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

# Inicjalizacja Pygame
pygame.init()

# Konfiguracja ekranu
WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("WYBÓR TRYBU GRY - Motorola Science Cup 2025")

# Czcionki z poprawnymi ścieżkami
bfont = pygame.font.Font(resource_path(style.MONTSERRAT_BOLD), 40)
buttonfont = pygame.font.Font(resource_path(style.MONTSERRAT_SEMI_BOLD), 37)

# Kolory
WHITE = (255, 255, 255)
DARK_GRAY = (80, 80, 80)
HOVER_COLOR = settings.HOVER_COLOR
GRAYED_OUT_COLOR = (50, 50, 50)  # Kolor dla nieaktywnego przycisku

# Tryby gry
MODES = {
    "Lokalnie": "local_game",
    "Przez Internet": "internet_game"
}

# Ścieżka do pliku tokenu
HOME_DIR = os.path.expanduser("~")
TOKEN_FILE = os.path.join(HOME_DIR, ".azbest_chess")

def load_refresh_token():
    """Wczytuje refresh token z pliku."""
    if os.path.exists(TOKEN_FILE):
        with open(TOKEN_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()
        if len(lines) >= 2:
            return lines[1].strip()
    return None

def is_logged_in():
    token = load_refresh_token()
    if token and token != " ":
        return token
    return None

def draw_button(screen, text, font, rect, color, hover_color, enabled=True):
    """Rysuje przycisk z efektem hover, z opcją wyłączenia."""
    mouse = pygame.mouse.get_pos()
    hover = rect.collidepoint(mouse) and enabled

    button_color = hover_color if hover else (color if enabled else GRAYED_OUT_COLOR)
    pygame.draw.rect(screen, button_color, rect, border_radius=10)
    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)

    text_surf = font.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=rect.center)
    screen.blit(text_surf, text_rect)
    return enabled

def draw_mode_selection():
    """Rysuje ekran wyboru trybu z uwzględnieniem statusu logowania."""
    settings.load_settings()
    settings.draw_chessboard(SCREEN, SCREEN.get_width(), SCREEN.get_height())

    current_width, current_height = SCREEN.get_size()
    logged_in = is_logged_in()

    # Nagłówek
    title_text = bfont.render("Wybierz Tryb Gry", True, WHITE)
    SCREEN.blit(title_text, (current_width // 2 - title_text.get_width() // 2, 50))

    # Przyciski trybów gry
    button_width = int(current_width * 0.35)
    button_height = int(current_height * 0.15)
    y_offset = current_height // 3

    mode_rects = []
    for mode_name in MODES.keys():
        rect = pygame.Rect(current_width // 2 - button_width // 2, y_offset, button_width, button_height)
        # Dla "PRZEZ INTERNET" sprawdzamy, czy użytkownik jest zalogowany
        if mode_name == "Przez Internet":
            display_text = mode_name if logged_in else "NIEZALOGOWANY"
            enabled = logged_in
        else:
            display_text = mode_name
            enabled = True
        is_enabled = draw_button(SCREEN, display_text, buttonfont, rect, DARK_GRAY, HOVER_COLOR, enabled)
        mode_rects.append((rect, MODES[mode_name], is_enabled))
        y_offset += button_height + 20

    # Przycisk powrotu
    back_button_rect = pygame.Rect(current_width // 2 - button_width // 2, y_offset + 20, button_width, button_height // 2)
    draw_button(SCREEN, "Powrót", buttonfont, back_button_rect, DARK_GRAY, HOVER_COLOR)

    return mode_rects, back_button_rect

def load_game_module(folder, module_name="main"):
    """Ładuje moduł z podanego folderu z poprawną ścieżką."""
    folder_path = resource_path(folder)
    if not os.path.exists(folder_path):
        raise FileNotFoundError(f"Nie znaleziono folderu: {folder_path}")
    if folder_path not in sys.path:
        sys.path.append(folder_path)
    
    spec = importlib.util.spec_from_file_location(module_name, os.path.join(folder_path, f"{module_name}.py"))
    if spec is None:
        raise ImportError(f"Nie znaleziono pliku {module_name}.py w folderze {folder}")
    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)
    return module

def local_game():
    """Obsługuje uruchomienie gry lokalnej."""
    print("Uruchamiam grę lokalnie...")
    try:
        main_module = load_game_module(os.path.join("LAN"), "main")
        main_module.main()
    except (FileNotFoundError, ImportError) as e:
        print(f"Błąd: Nie udało się załadować gry z folderu LAN: {e}")

def internet_game():
    """Obsługuje uruchomienie gry przez Internet."""
    print("Uruchamiam grę przez internet...")
    try:
        main_module = load_game_module(os.path.join("online"), "main")
        main_module.main()
    except (FileNotFoundError, ImportError) as e:
        print(f"Błąd: Nie udało się załadować gry z folderu online: {e}")

def mode_selection_screen():
    mouse_released = False

    while True:
        mode_rects, back_button_rect = draw_mode_selection()
        pygame.display.update()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()

            if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_released = False

            if event.type == pygame.MOUSEBUTTONUP and event.button == 1 and not mouse_released:
                mouse_pos = pygame.mouse.get_pos()
                mouse_released = True

                if back_button_rect.collidepoint(mouse_pos):
                    pygame.time.delay(200)
                    gamemodes.main(SCREEN)
                    return

                for rect, action, enabled in mode_rects:
                    if rect.collidepoint(mouse_pos) and enabled:
                        pygame.time.delay(200)
                        pygame.mixer.music.pause()
                        if action == "local_game":
                            local_game()
                        elif action == "internet_game":
                            internet_game()
                        return

if __name__ == "__main__":
    mode_selection_screen()
