import pygame
import requests
import os
import settings
import style
import sys

pygame.init()

HOME_DIR = os.path.expanduser("~")
TOKEN_FILE = os.path.join(HOME_DIR, ".azbest_chess")
API_BASE = "https://azbest.org.pl/"

WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
RED = (255, 0, 0)
DARK_GRAY = (80, 80, 80)
HOVER_COLOR = settings.HOVER_COLOR

WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("KONTO - Motorola Science Cup 2025")

BUTTON_FONT = pygame.font.Font(style.MONTSERRAT_BOLD, 30)

def save_refresh_token(token):
    lines = []
    try:
        if os.path.exists(TOKEN_FILE):
            with open(TOKEN_FILE, "r", encoding="utf-8") as file:
                lines = file.readlines()
    except Exception as e:
        print(f"Błąd odczytu pliku: {e}")
        lines = []
    
    while len(lines) < 2:
        lines.append("\n")
    
    lines[1] = str(token) + "\n"
    
    try:
        with open(TOKEN_FILE, "w", encoding="utf-8") as file:
            file.writelines(lines)
    except Exception as e:
        print(f"Błąd zapisu do pliku: {e}")

def load_refresh_token():
    if os.path.exists(TOKEN_FILE):
        with open(TOKEN_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()
        if len(lines) >= 2:
            return lines[1].strip()
    return None

def delete_refresh_token():
    if os.path.exists(TOKEN_FILE):
        with open(TOKEN_FILE, "r") as file:
            lines = file.readlines()
        if len(lines) > 1:
            lines[1] = "\n"
            with open(TOKEN_FILE, "w") as file:
                file.writelines(lines)

def api_request(endpoint, method="GET", data=None, headers=None):
    url = f"{API_BASE}/{endpoint}"
    try:
        response = requests.request(method, url, json=data, headers=headers)
        return response
    except requests.exceptions.RequestException:
        return None

def login(username, password):
    data = {"username": username, "password": password}
    response = api_request("login", "POST", data)
    if response is None:
        return None, "Błąd połączenia z serwerem"
    if response.status_code == 200:
        tokens = response.json()
        save_refresh_token(tokens["refresh_token"])
        return tokens["access_token"], "Zalogowano pomyślnie"
    try:
        error_message = response.json().get("message", "Nieprawidłowe dane logowania")
    except ValueError:
        error_message = "Nieprawidłowe dane logowania"
    return None, error_message

def register(username, password):
    data = {"username": username, "password": password}
    response = api_request("register", "POST", data)
    if response is None:
        return "Błąd połączenia z serwerem"
    if response.status_code == 200:
        return "Rejestracja zakończona sukcesem"
    try:
        return response.json().get("message", "Błąd podczas rejestracji")
    except ValueError:
        return "Błąd podczas rejestracji"

def refresh_access_token():
    refresh_token = load_refresh_token()
    if not refresh_token:
        return None
    response = api_request("refresh", "POST", {"refresh_token": refresh_token})
    if response is None or response.status_code != 200:
        delete_refresh_token()
        return None
    tokens = response.json()
    save_refresh_token(tokens["refresh_token"])
    return tokens["access_token"]

def get_user_info(access_token):
    headers = {"Authorization": f"Bearer {access_token}"}
    response = api_request("user", "GET", headers=headers)
    if response is None:
        return "Błąd połączenia z serwerem"
    if response.status_code == 200:
        return response.json()
    elif response.status_code == 401:
        return "Nieautoryzowany"
    return "Błąd serwera"

def logout():
    refresh_token = load_refresh_token()
    if refresh_token:
        response = api_request("logout", "POST", {"refresh_token": refresh_token})
        if response and response.status_code == 200:
            delete_refresh_token()

def draw_text(screen, text, x, y, color=WHITE, size=25, center=False):
    font_obj = pygame.font.Font(style.MONTSERRAT_BOLD, size)
    text_surface = font_obj.render(text, True, color)
    if center:
        text_rect = text_surface.get_rect(center=(x, y))
        screen.blit(text_surface, text_rect)
    else:
        screen.blit(text_surface, (x, y))

def draw_button(screen, x, y, width, height, text):
    rect = pygame.Rect(x, y, width, height)
    mouse = pygame.mouse.get_pos()
    button_color = HOVER_COLOR if rect.collidepoint(mouse) else DARK_GRAY
    pygame.draw.rect(screen, button_color, rect, border_radius=10)
    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)
    text_surf = BUTTON_FONT.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=rect.center)
    screen.blit(text_surf, text_rect)
    return rect

def main(screen):
    access_token = refresh_access_token()
    if access_token:
        user_info = get_user_info(access_token)
        if isinstance(user_info, dict):  # Check if user_info is a dictionary
            user_SCREEN(user_info, screen)
            return
        elif user_info == "Nieautoryzowany":
            access_token = refresh_access_token()
            if access_token:
                user_info = get_user_info(access_token)
                if isinstance(user_info, dict):  # Check again after refresh
                    user_SCREEN(user_info, screen)
                    return
    auth_SCREEN(screen)

def auth_SCREEN(screen):
    running = True
    username = ""
    password = ""
    message = ""
    input_active = [False, False]
    is_registering = False
    clock = pygame.time.Clock()
    cursor_visible = True
    cursor_timer = 0
    
    # Key holding handling variables
    held_key = None
    key_timer = 0
    INITIAL_DELAY = 500
    REPEAT_DELAY = 33
    last_input_time = 0

    # Font for measuring text width
    input_font = pygame.font.Font(style.MONTSERRAT_BOLD, 32)
    MAX_INPUT_WIDTH = 220

    while running:
        current_width, current_height = screen.get_size()
        settings.draw_chessboard(screen, current_width, current_height)

        container_width = 520
        container_height = 240
        container_x = (current_width - container_width) // 2
        container_y = (current_height - container_height) // 2

        pygame.draw.rect(screen, DARK_GRAY, (container_x, container_y-15, container_width, container_height), border_radius=10)
        pygame.draw.rect(screen, WHITE, (container_x, container_y-15, container_width, container_height), 2, border_radius=10)

        title_text = "Rejestracja" if is_registering else "Logowanie"
        draw_text(screen, title_text, container_x + container_width // 2, container_y + 30, size=44, center=True)

        draw_text(screen, "Nazwa użytkownika:", container_x + 15, container_y + 80, size=21)
        username_rect = pygame.Rect(container_x + 250, container_y + 75, 240, 40)
        pygame.draw.rect(screen, DARK_GRAY, username_rect, 0, border_radius=5)
        pygame.draw.rect(screen, WHITE, username_rect, 2, border_radius=5)

        draw_text(screen, "Hasło:", container_x + 165, container_y + 140, size=23)
        password_rect = pygame.Rect(container_x + 250, container_y + 135, 240, 40)
        pygame.draw.rect(screen, DARK_GRAY, password_rect, 0, border_radius=5)
        pygame.draw.rect(screen, WHITE, password_rect, 2, border_radius=5)

        message_y = container_y + container_height + 20
        draw_text(screen, message, container_x + 15, message_y, BLACK, size=32)

        gap = 20
        btn_width = (container_width - 2 * gap) // 3
        button_y = message_y + 60

        bottom_margin = 20
        max_button_y = current_height - 60 - bottom_margin
        if button_y > max_button_y:
            button_y = max_button_y

        action_text = "OK" if is_registering else "Zaloguj"
        action_button = draw_button(screen, container_x, button_y, btn_width, 60, action_text)
        switch_text = "Logowanie" if is_registering else "Rejestracja"
        switch_button = draw_button(screen, container_x + btn_width + gap - 15, button_y, btn_width + 30, 60, switch_text)
        back_button = draw_button(screen, container_x + 2 * (btn_width + gap), button_y, btn_width, 60, "Powrót")

        cursor = "|" if cursor_visible else ""
        
        username_display = username + (cursor if input_active[0] else "")
        password_display = "*" * len(password) + (cursor if input_active[1] else "")
        
        while input_font.size(username_display)[0] > MAX_INPUT_WIDTH and len(username_display) > 0:
            username_display = username_display[:-1]
            if input_active[0]:
                username = username[:-1]
        
        while input_font.size(password_display)[0] > MAX_INPUT_WIDTH and len(password_display) > 0:
            password_display = password_display[:-1]
            if input_active[1]:
                password = password[:-1]

        draw_text(screen, username_display, username_rect.x + 10, username_rect.y, size=32)
        draw_text(screen, password_display, password_rect.x + 10, password_rect.y + 2, size=32)

        pygame.display.flip()

        current_time = pygame.time.get_ticks()
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.VIDEORESIZE:
                new_width = max(event.w, 800)
                new_height = max(event.h, 700)
                screen = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
            elif event.type == pygame.KEYDOWN:
                held_key = event.key
                key_timer = 0
                if input_active[0]:
                    if event.key == pygame.K_BACKSPACE and username:
                        username = username[:-1]
                    elif event.unicode.isalnum() or event.unicode in " _-":
                        new_text = username + event.unicode
                        if input_font.size(new_text)[0] <= MAX_INPUT_WIDTH:
                            username = new_text
                elif input_active[1]:
                    if event.key == pygame.K_BACKSPACE and password:
                        password = password[:-1]
                    elif event.unicode:
                        new_text = password + event.unicode
                        if input_font.size(new_text + "*")[0] <= MAX_INPUT_WIDTH:
                            password = new_text
                last_input_time = current_time
            elif event.type == pygame.KEYUP:
                if event.key == held_key:
                    held_key = None
                    key_timer = 0
            elif event.type == pygame.MOUSEBUTTONDOWN:
                pygame.time.delay(150)
                mx, my = event.pos
                input_active = [username_rect.collidepoint(mx, my), password_rect.collidepoint(mx, my)]
                if action_button.collidepoint(mx, my):
                    if is_registering:
                        message = register(username, password)
                    else:
                        access_token, message = login(username, password)
                        if access_token:
                            user_info = get_user_info(access_token)
                            if isinstance(user_info, dict):  # Check if user_info is valid
                                user_SCREEN(user_info, screen)
                                return
                elif switch_button.collidepoint(mx, my):
                    is_registering = not is_registering
                elif back_button.collidepoint(mx, my):
                    return

        if held_key and (current_time - last_input_time) >= INITIAL_DELAY:
            key_timer += clock.get_time()
            if key_timer >= REPEAT_DELAY:
                if held_key == pygame.K_BACKSPACE:
                    if input_active[0] and username:
                        username = username[:-1]
                    elif input_active[1] and password:
                        password = password[:-1]
                else:
                    char = pygame.key.name(held_key) if held_key != pygame.K_SPACE else " "
                    if (pygame.key.name(held_key).isalnum() or pygame.key.name(held_key) in ["space", "-", "_"]):
                        if input_active[0]:
                            new_text = username + char
                            if input_font.size(new_text)[0] <= MAX_INPUT_WIDTH:
                                username = new_text
                        elif input_active[1]:
                            new_text = password + char
                            if input_font.size(new_text + "*")[0] <= MAX_INPUT_WIDTH:
                                password = new_text
                key_timer -= REPEAT_DELAY

        cursor_timer += clock.get_time()
        if cursor_timer >= 500:
            cursor_visible = not cursor_visible
            cursor_timer = 0

        clock.tick(60)

def user_SCREEN(user_info, screen):
    running = True
    clock = pygame.time.Clock()
    while running:
        current_width, current_height = screen.get_size()
        settings.draw_chessboard(screen, current_width, current_height)

        container_width = current_width * 0.4
        container_height = current_height * 0.3
        container_x = (current_width - container_width) // 2
        container_y = (current_height - container_height) // 2
        
        pygame.draw.rect(screen, DARK_GRAY, (container_x, container_y-20, container_width, container_height), border_radius=15)
        pygame.draw.rect(screen, WHITE, (container_x, container_y-20, container_width, container_height), 2, border_radius=15)

        center_x = current_width // 2
        text_y_base = container_y + 40
        
        # Safely handle user_info as a dictionary
        display_name = user_info.get('display_name', user_info.get('username', 'Nieznany'))
        elo_points = user_info.get('elo', 'Brak danych')
        created_at = user_info.get('created_at', 'Brak danych')
        
        draw_text(screen, f"Użytkownik: {user_info['username']}", center_x - 125, text_y_base)
        draw_text(screen, f"Punkty ELO: {elo_points}", center_x, text_y_base + 50, center=True)
        draw_text(screen, f"Dołączył: {created_at}", center_x, text_y_base + 85, center=True)

        button_width = container_width * 0.4
        button_height = 50
        button_y = container_y + container_height + 20
        
        logout_button = draw_button(screen, center_x - button_width - 10, button_y, button_width, button_height, "Wyloguj")
        back_button = draw_button(screen, center_x + 10, button_y, button_width, button_height, "Powrót")

        pygame.display.flip()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.VIDEORESIZE:
                screen = pygame.display.set_mode((event.w, event.h), pygame.RESIZABLE)
            elif event.type == pygame.MOUSEBUTTONDOWN:
                pygame.time.delay(150)
                if logout_button.collidepoint(event.pos):
                    logout()
                    auth_SCREEN(screen)
                    return
                elif back_button.collidepoint(event.pos):
                    return

        clock.tick(60)

if __name__ == "__main__":
    screen = pygame.display.set_mode((settings.WIDTH, settings.HEIGHT), pygame.RESIZABLE)
    main(screen)