# engine.py
from utils import is_inside, pos_to_alg
from Piece import Piece, get_piece_at

class ChessEngine:
    piece_values = {"pawn": 10, "knight": 30, "bishop": 30, "rook": 50, "queen": 90, "king": 900}
    center_squares = [(3, 3), (3, 4), (4, 3), (4, 4)]  # d5, d4, e5, e4

    @staticmethod
    def get_piece_at(pos, pieces_list):
        for piece in pieces_list:
            if piece.pos == pos:
                return piece
        return None

    @staticmethod
    def simulate_move(piece, move, pieces_list, en_passant_info):
        new_pieces = [p.clone() for p in pieces_list]
        moving_piece = None
        for p in new_pieces:
            if p.id == piece.id:
                moving_piece = p
                break
        if moving_piece is None:
            return new_pieces, None

        original_pos = moving_piece.pos

        if moving_piece.type == "king" and abs(move[0] - original_pos[0]) == 2:
            if move[0] > original_pos[0]:  # Roszada krótka (O-O)
                rook_start = (7, original_pos[1])
                rook_end = (5, original_pos[1])
                moving_piece.pos = (6, original_pos[1])
            else:  # Roszada długa (O-O-O)
                rook_start = (0, original_pos[1])
                rook_end = (3, original_pos[1])
                moving_piece.pos = (2, original_pos[1])
            
            rook = ChessEngine.get_piece_at(rook_start, new_pieces)
            if rook and rook.type == "rook" and rook.color == moving_piece.color:
                rook.pos = rook_end
                rook.has_moved = True
            
            moving_piece.has_moved = True
            new_en_passant_info = None
        else:
            is_en_passant = False
            if moving_piece.type == "pawn" and en_passant_info is not None:
                if move == en_passant_info["target"]:
                    is_en_passant = True

            if not is_en_passant:
                target = ChessEngine.get_piece_at(move, new_pieces)
                if target and target.id != moving_piece.id:
                    new_pieces.remove(target)
            else:
                capture_pos = (move[0], original_pos[1])
                target = ChessEngine.get_piece_at(capture_pos, new_pieces)
                if target and target.id != moving_piece.id:
                    new_pieces.remove(target)

            moving_piece.pos = move
            new_en_passant_info = None
            if moving_piece.type == "pawn":
                if abs(move[1] - original_pos[1]) == 2:
                    passed_row = (original_pos[1] + move[1]) // 2
                    new_en_passant_info = {"target": (move[0], passed_row), "pawn_id": moving_piece.id}

                # Sprawdzamy, czy pionek dotarł do ostatniego rzędu
                if (moving_piece.color == "white" and move[1] == 0) or (moving_piece.color == "black" and move[1] == 7):
                    # Zamiast automatycznego wyboru, zwracamy specjalny znacznik
                    return new_pieces, {"promotion_needed": True, "pawn": moving_piece, "pos": move}

            moving_piece.has_moved = True

        return new_pieces, new_en_passant_info

    @staticmethod
    def get_valid_moves(piece, pieces_list, en_passant_info):
        candidate_moves = []
        col, row = piece.pos
        color = piece.color

        if piece.type == "pawn":
            direction = -1 if color == "white" else 1
            start_row = 6 if color == "white" else 1

            forward = (col, row + direction)
            if is_inside(*forward) and ChessEngine.get_piece_at(forward, pieces_list) is None:
                candidate_moves.append(forward)
                if row == start_row:
                    forward2 = (col, row + 2 * direction)
                    if is_inside(*forward2) and \
                    ChessEngine.get_piece_at(forward2, pieces_list) is None and \
                    ChessEngine.get_piece_at(forward, pieces_list) is None:
                        candidate_moves.append(forward2)
            for dc in [-1, 1]:
                diag = (col + dc, row + direction)
                if is_inside(*diag):
                    target = ChessEngine.get_piece_at(diag, pieces_list)
                    if target and target.color != color:
                        candidate_moves.append(diag)
            if en_passant_info is not None:
                target_square = en_passant_info["target"]
                if target_square[1] == row + direction and abs(target_square[0] - col) == 1:
                    candidate_moves.append(target_square)

        elif piece.type == "knight":
            knight_moves = [(2, 1), (2, -1), (-2, 1), (-2, -1),
                            (1, 2), (1, -2), (-1, 2), (-1, -2)]
            for dx, dy in knight_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = ChessEngine.get_piece_at(new_pos, pieces_list)
                    if not target or target.color != color:
                        candidate_moves.append(new_pos)

        elif piece.type in ["bishop", "rook", "queen"]:
            directions = []
            if piece.type in ["rook", "queen"]:
                directions += [(1, 0), (-1, 0), (0, 1), (0, -1)]
            if piece.type in ["bishop", "queen"]:
                directions += [(1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in directions:
                new_col, new_row = col, row
                while True:
                    new_col += dx
                    new_row += dy
                    if not is_inside(new_col, new_row):
                        break
                    target = ChessEngine.get_piece_at((new_col, new_row), pieces_list)
                    if not target:
                        candidate_moves.append((new_col, new_row))
                    else:
                        if target.color != color:
                            candidate_moves.append((new_col, new_row))
                        break

        elif piece.type == "king":
            king_moves = [(1, 0), (-1, 0), (0, 1), (0, -1),
                        (1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in king_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = ChessEngine.get_piece_at(new_pos, pieces_list)
                    if not target or target.color != color:
                        candidate_moves.append(new_pos)

            if not piece.has_moved and not ChessEngine.is_in_check(color, pieces_list):
                rook_short = ChessEngine.get_piece_at((7, row), pieces_list)
                if (rook_short and rook_short.type == "rook" and rook_short.color == color and 
                    not rook_short.has_moved):
                    path_clear = True
                    for c in [5, 6]:
                        if ChessEngine.get_piece_at((c, row), pieces_list) is not None:
                            path_clear = False
                            break
                    if path_clear:
                        safe = True 
                        for intermediate in [(4, row), (5, row), (6, row)]:
                            sim_pieces = [p.clone() for p in pieces_list]
                            king_sim = ChessEngine.get_piece_at((4, row), sim_pieces)
                            king_sim.pos = intermediate
                            if ChessEngine.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((6, row))

                rook_long = ChessEngine.get_piece_at((0, row), pieces_list)
                if (rook_long and rook_long.type == "rook" and rook_long.color == color and 
                    not rook_long.has_moved):
                    path_clear = True
                    for c in [1, 2, 3]:
                        if ChessEngine.get_piece_at((c, row), pieces_list) is not None:
                            path_clear = False
                            break
                    if path_clear:
                        safe = True
                        for intermediate in [(4, row), (3, row), (2, row)]:
                            sim_pieces = [p.clone() for p in pieces_list]
                            king_sim = ChessEngine.get_piece_at((4, row), sim_pieces)
                            king_sim.pos = intermediate
                            if ChessEngine.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((2, row))

        valid_moves = []
        for move in candidate_moves:
            sim_pieces, sim_en_passant = ChessEngine.simulate_move(piece, move, pieces_list, en_passant_info)
            if not ChessEngine.is_in_check(color, sim_pieces):
                valid_moves.append(move)

        return valid_moves

    @staticmethod
    def is_in_check(color, pieces_list):
        king = None
        for p in pieces_list:
            if p.type == "king" and p.color == color:
                king = p
                break
        if king is None:
            return True
        king_pos = king.pos
        for p in pieces_list:
            if p.color != color:
                if king_pos in p.get_attack_squares(pieces_list):
                    return True
        return False

    @staticmethod
    def has_any_valid_move(color, pieces_list, en_passant_info):
        for p in pieces_list:
            if p.color == color:
                if ChessEngine.get_valid_moves(p, pieces_list, en_passant_info):
                    return True
        return False

    @staticmethod
    def evaluate_king_safety(pieces, color):
        king = None
        for p in pieces:
            if p.type == "king" and p.color == color:
                king = p
                break
        if not king:
            return 0

        safety_score = 0
        col, row = king.pos

        if color == "white" and row >= 6 and (col <= 2 or col >= 5):
            safety_score += 20
        elif color == "black" and row <= 1 and (col <= 2 or col >= 5):
            safety_score += 20

        pawn_shield = 0
        direction = -1 if color == "white" else 1
        for dc in [-1, 0, 1]:
            check_pos = (col + dc, row + direction)
            if is_inside(*check_pos):
                piece = ChessEngine.get_piece_at(check_pos, pieces)
                if piece and piece.type == "pawn" and piece.color == color:
                    pawn_shield += 5
        safety_score += pawn_shield

        return safety_score

    @staticmethod
    def evaluate_position(pieces, en_passant_info, turn, bot_color):
        if not ChessEngine.has_any_valid_move(turn, pieces, en_passant_info):
            if ChessEngine.is_in_check(turn, pieces):
                return -10000 if turn == bot_color else 10000
            else:
                return 0

        eval_val = 0

        for p in pieces:
            value = ChessEngine.piece_values.get(p.type, 0)
            if p.color == bot_color:
                eval_val += value
            else:
                eval_val -= value

        center_control = 0
        for pos in ChessEngine.center_squares:
            piece = ChessEngine.get_piece_at(pos, pieces)
            if piece:
                if piece.color == bot_color:
                    if piece.type == "pawn":
                        center_control += 10
                    else:
                        center_control += 5
                else:
                    if piece.type == "pawn":
                        center_control -= 10
                    else:
                        center_control -= 5
        eval_val += center_control

        pawn_progress = 0
        for p in pieces:
            if p.type == "pawn":
                row = p.pos[1]
                if p.color == bot_color:
                    if bot_color == "white":
                        pawn_progress += (7 - row) * 2
                    else:
                        pawn_progress += row * 2
                else:
                    if bot_color == "white":
                        pawn_progress -= row * 2
                    else:
                        pawn_progress -= (7 - row) * 2
        eval_val += pawn_progress

        bot_king_safety = ChessEngine.evaluate_king_safety(pieces, bot_color)
        opp_color = "white" if bot_color == "black" else "black"
        opp_king_safety = ChessEngine.evaluate_king_safety(pieces, opp_color)
        eval_val += bot_king_safety - opp_king_safety

        return eval_val

    @staticmethod
    def minimax(pieces, en_passant_info, turn, depth, alpha, beta, bot_color):
        if depth == 0 or not ChessEngine.has_any_valid_move(turn, pieces, en_passant_info):
            return ChessEngine.evaluate_position(pieces, en_passant_info, turn, bot_color)

        moves = []
        for p in pieces:
            if p.color == turn:
                valid_moves = ChessEngine.get_valid_moves(p, pieces, en_passant_info)
                for move in valid_moves:
                    moves.append((p, move))

        if turn == bot_color:
            max_eval = -float('inf')
            for piece, move in moves:
                new_pieces, new_en_passant = ChessEngine.simulate_move(piece, move, pieces, en_passant_info)
                new_turn = "white" if turn == "black" else "black"
                eval_val = ChessEngine.minimax(new_pieces, new_en_passant, new_turn, depth - 1, alpha, beta, bot_color)
                max_eval = max(max_eval, eval_val)
                alpha = max(alpha, eval_val)
                if beta <= alpha:
                    break
            return max_eval
        else:
            min_eval = float('inf')
            for piece, move in moves:
                new_pieces, new_en_passant = ChessEngine.simulate_move(piece, move, pieces, en_passant_info)
                new_turn = "white" if turn == "black" else "black"
                eval_val = ChessEngine.minimax(new_pieces, new_en_passant, new_turn, depth - 1, alpha, beta, bot_color)
                min_eval = min(min_eval, eval_val)
                beta = min(beta, eval_val)
                if beta <= alpha:
                    break
            return min_eval

    @staticmethod
    def get_bot_move(pieces, en_passant_info, turn, depth):
        best_eval = -float('inf')
        best_move = None
        best_piece = None
        alpha = -float('inf')
        beta = float('inf')

        moves = []
        for p in pieces:
            if p.color == turn:
                valid_moves = ChessEngine.get_valid_moves(p, pieces, en_passant_info)
                for move in valid_moves:
                    moves.append((p, move))

        for piece, move in moves:
            new_pieces, new_en_passant = ChessEngine.simulate_move(piece, move, pieces, en_passant_info)
            new_turn = "white" if turn == "black" else "black"
            eval_val = ChessEngine.minimax(new_pieces, new_en_passant, new_turn, depth - 1, alpha, beta, turn)
            if eval_val > best_eval:
                best_eval = eval_val
                best_piece = piece
                best_move = move
                alpha = max(alpha, eval_val)

        return best_piece, best_move