import pygame
import sys
import os
from Board import Board
from Piece import Piece
from engine import ChessEngine
from utils import load_image, pos_to_alg

# Dodajemy ścieżkę nadrzędną do sys.path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from chess_assets import load_chess_pieces, get_chess_style
import settings
import style

def resource_path(relative_path):
    try:
        base_path = sys._MEIPASS
    except AttributeError:
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

def load_image(path, size):
    image = pygame.image.load(resource_path(path)).convert_alpha()
    return pygame.transform.smoothscale(image, (size, size))

move_sound = pygame.mixer.Sound(resource_path(os.path.join("assets/audio", settings.CURRENT_SOUND)))
pygame.mixer.Sound.set_volume(move_sound, (settings.sfprev_value) / 100)

class ChessGame:
    def __init__(self, screen):
        self.screen = screen
        pygame.font.init()

        # Base dimensions for scaling
        self.BASE_SQUARE_SIZE = 80
        self.BASE_SIDEBAR_WIDTH = 300
        self.BASE_WIDTH = self.BASE_SQUARE_SIZE * 8 + self.BASE_SIDEBAR_WIDTH
        self.BASE_HEIGHT = self.BASE_SQUARE_SIZE * 8
        self.SIDEBAR_WIDTH_RATIO = self.BASE_SIDEBAR_WIDTH / self.BASE_WIDTH

        # Initial setup with resizable window
        self.canvas = pygame.display.set_mode((self.BASE_WIDTH, self.BASE_HEIGHT), pygame.RESIZABLE)
        pygame.display.set_caption("SZACHY GRACZ VS KOMPUTER - Motorola Science Cup 2025")
        self.total_size = (self.BASE_WIDTH, self.BASE_HEIGHT)

        # Game state variables
        self.pieces = []
        self.piece_id_counter = 0
        self.turn = "white"  # Zawsze zaczyna biały
        self.en_passant_info = None
        self.move_history = []
        self.state_history = []
        self.position_counts = {}
        self.selected_piece = None
        self.game_over = False
        self.game_over_message = ""
        self.clock = pygame.time.Clock()
        self.move_pair = None

        self.white_time = 300
        self.black_time = 300
        self.last_update_time = pygame.time.get_ticks()

        self.bot_depth = 3
        self.bot_thinking = False
        self.bot_think_time = 0
        self.bot_move = None
        self.last_bot_move = None

        self.BLACK = (0, 0, 0)
        self.WHITE = (255, 255, 255)
        self.HOVER_COLOR_WHITE = (220, 220, 220)
        self.HOVER_COLOR_BLACK = (50, 50, 50)
        self.SHADOW_COLOR = (150, 150, 150)
        self.CHESS_WHITE = (240, 217, 181)
        self.BOT_MOVE_COLOR = (0, 0, 255)

        self.nerd_view_enabled = False
        self.button_hover_played = False
        self.player_color = None

        # Initial scaling
        self.update_scale()
        self.setup_pieces()

    def update_scale(self):
        current_width, current_height = self.canvas.get_size()
        self.total_size = (current_width, current_height)
        sidebar_width = int(current_width * self.SIDEBAR_WIDTH_RATIO)
        board_width = current_width - sidebar_width
        self.square_size = min(board_width // 8, current_height // 8)
        sidebar_width = current_width - (self.square_size * 8)

        self.scale_factor = self.square_size / self.BASE_SQUARE_SIZE
        self.board = Board(square_size=self.square_size, sidebar_width=sidebar_width, 
                          player_color=self.player_color)

        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(36 * self.scale_factor))
        self.font_smaller = pygame.font.Font(style.MONTSERRAT_BOLD, int(20 * self.scale_factor))
        self.small_font = pygame.font.Font(style.MONTSERRAT_REGULAR, int(24 * self.scale_factor))
        self.tiny_font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, int(18 * self.scale_factor))

        self.chess_pieces = load_chess_pieces(self.square_size)
        self.pionekBialy = self.chess_pieces["pionekB"]
        self.pionekCzarny = self.chess_pieces["pionekC"]
        self.wiezaBiala = self.chess_pieces["wiezaB"]
        self.wiezaCzarna = self.chess_pieces["wiezaC"]
        self.konBialy = self.chess_pieces["konB"]
        self.konCzarny = self.chess_pieces["konC"]
        self.goniecBialy = self.chess_pieces["goniecB"]
        self.goniecCzarny = self.chess_pieces["goniecC"]
        self.hetmanBialy = self.chess_pieces["hetmanB"]
        self.hetmanCzarny = self.chess_pieces["hetmanC"]
        self.krolBialy = self.chess_pieces["krolB"]
        self.krolCzarny = self.chess_pieces["krolC"]

        piece_map = {
            "pawn": ("pionekB", "pionekC"),
            "rook": ("wiezaB", "wiezaC"),
            "knight": ("konB", "konC"),
            "bishop": ("goniecB", "goniecC"),
            "queen": ("hetmanB", "hetmanC"),
            "king": ("krolB", "krolC")
        }
        for piece in self.pieces:
            white_key, black_key = piece_map[piece.type]
            piece.image = self.chess_pieces[white_key if piece.color == "white" else black_key]

        button_x = self.square_size * 8 + int(20 * self.scale_factor)
        button_width = min(int(200 * self.scale_factor), sidebar_width - int(40 * self.scale_factor))
        self.button_rect_surrender = pygame.Rect(button_x, int(540 * self.scale_factor), button_width + 50, int(40 * self.scale_factor))
        self.button_rect_nerd_view = pygame.Rect(button_x, int(590 * self.scale_factor), button_width + 50, int(40 * self.scale_factor))
        self.button_rect_back = pygame.Rect(self.total_size[0] // 2 - int(150 * self.scale_factor), self.total_size[1] // 2 + int(20 * self.scale_factor), int(350 * self.scale_factor), int(50 * self.scale_factor))

    def add_piece(self, piece_type, color, col, row, image_white, image_black):
        image = image_white if color == "white" else image_black
        piece = Piece(self.piece_id_counter, piece_type, color, (col, row), image)
        self.pieces.append(piece)
        self.piece_id_counter += 1

    def setup_pieces(self):
        self.pieces.clear()
        # Białe figury
        self.add_piece("rook", "white", 0, 7, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "white", 1, 7, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "white", 2, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "white", 3, 7, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "white", 4, 7, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "white", 5, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "white", 6, 7, self.konBialy, self.konCzarny)
        self.add_piece("rook", "white", 7, 7, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "white", col, 6, self.pionekBialy, self.pionekCzarny)
        # Czarne figury
        self.add_piece("rook", "black", 0, 0, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "black", 1, 0, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "black", 2, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "black", 3, 0, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "black", 4, 0, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "black", 5, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "black", 6, 0, self.konBialy, self.konCzarny)
        self.add_piece("rook", "black", 7, 0, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "black", col, 1, self.pionekBialy, self.pionekCzarny)

    def transform_pos(self, pos):
        if self.player_color == "black":
            return (7 - pos[0], 7 - pos[1])
        return pos

    def inverse_transform_pos(self, pos):
        if self.player_color == "black":
            return (7 - pos[0], 7 - pos[1])
        return pos

    def draw_pieces(self):
        for piece in self.pieces:
            display_pos = self.transform_pos(piece.pos)
            piece_copy = piece.clone()
            piece_copy.pos = display_pos
            piece_copy.draw(self.canvas, self.square_size)

    def get_piece_at(self, pos):
        for piece in self.pieces:
            if piece.pos == pos:
                return piece
        return None

    def is_insufficient_material(self):
        piece_count = {
            'white': {'pawn': 0, 'knight': 0, 'bishop': 0, 'rook': 0, 'queen': 0, 'king': 0},
            'black': {'pawn': 0, 'knight': 0, 'bishop': 0, 'rook': 0, 'queen': 0, 'king': 0}
        }
        for piece in self.pieces:
            piece_count[piece.color][piece.type] += 1

        for color in ['white', 'black']:
            counts = piece_count[color]
            if (counts['king'] == 1 and counts['pawn'] == 0 and counts['knight'] == 0 and 
                counts['bishop'] == 0 and counts['rook'] == 0 and counts['queen'] == 0):
                other_color = 'black' if color == 'white' else 'white'
                other_counts = piece_count[other_color]
                if (other_counts['king'] == 1 and other_counts['pawn'] == 0 and other_counts['knight'] == 0 and 
                    other_counts['bishop'] == 0 and other_counts['rook'] == 0 and other_counts['queen'] == 0):
                    return True
            if (counts['king'] == 1 and counts['pawn'] == 0 and counts['rook'] == 0 and counts['queen'] == 0 and 
                (counts['knight'] + counts['bishop']) <= 1):
                other_color = 'black' if color == 'white' else 'white'
                other_counts = piece_count[other_color]
                if (other_counts['king'] == 1 and other_counts['pawn'] == 0 and other_counts['knight'] == 0 and 
                    other_counts['bishop'] == 0 and other_counts['rook'] == 0 and other_counts['queen'] == 0):
                    return True
        return False

    def get_position_key(self):
        position = []
        for piece in sorted(self.pieces, key=lambda p: (p.pos[0], p.pos[1])):
            position.append(f"{piece.type}{piece.color}{piece.pos}")
        return "|".join(position) + f"|{self.turn}|{self.en_passant_info}"

    def surrender(self):
        self.game_over = True
        self.game_over_message = "Poddanie! " + ("Czarne" if self.turn == "white" else "Białe") + " wygrywają!"

    def propose_draw(self):
        self.game_over = True
        self.game_over_message = "Remis zaakceptowany!"

    def process_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return "quit"
            elif event.type == pygame.VIDEORESIZE:
                self.update_scale()
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_pos = pygame.mouse.get_pos()
                if self.game_over:
                    if self.button_rect_back.collidepoint(mouse_pos):
                        pygame.quit()
                        sys.exit()
                    return None

                if self.button_rect_surrender.collidepoint(mouse_pos):
                    self.surrender()
                    return None
                if self.button_rect_nerd_view.collidepoint(mouse_pos):
                    self.nerd_view_enabled = not self.nerd_view_enabled
                    return None

                if mouse_pos[0] < self.board.board_width and not self.bot_thinking and not self.game_over:
                    col = mouse_pos[0] // self.square_size
                    row = mouse_pos[1] // self.square_size
                    clicked_pos = self.inverse_transform_pos((col, row))

                    if self.turn == self.player_color:
                        if self.selected_piece is None:
                            piece = self.get_piece_at(clicked_pos)
                            if piece and piece.color == self.turn:
                                self.selected_piece = piece
                        else:
                            target_piece = self.get_piece_at(clicked_pos)
                            if target_piece and target_piece.color == self.turn:
                                self.selected_piece = target_piece
                            else:
                                valid_moves = ChessEngine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
                                if clicked_pos in valid_moves:
                                    self.state_history.append((
                                        [p.clone() for p in self.pieces],
                                        self.turn,
                                        self.en_passant_info
                                    ))
                                    orig_pos = self.selected_piece.pos
                                    move_notation = pos_to_alg(orig_pos) + "-" + pos_to_alg(clicked_pos)
                                    self.move_pair = (move_notation, None)
                                    self.pieces, result = ChessEngine.simulate_move(self.selected_piece, clicked_pos, self.pieces, self.en_passant_info)
                                    move_sound.play()

                                    if isinstance(result, dict) and result.get("promotion_needed"):
                                        pawn = result["pawn"]
                                        new_type = self.choose_promotion(pawn)
                                        for p in self.pieces:
                                            if p.id == pawn.id:
                                                p.type = new_type
                                                p.image = (self.hetmanBialy if new_type == "queen" else
                                                           self.wiezaBiala if new_type == "rook" else
                                                           self.goniecBialy if new_type == "bishop" else
                                                           self.konBialy) if p.color == "white" else \
                                                          (self.hetmanCzarny if new_type == "queen" else
                                                           self.wiezaCzarna if new_type == "rook" else
                                                           self.goniecCzarny if new_type == "bishop" else
                                                           self.konCzarny)
                                                break
                                        self.en_passant_info = None
                                    else:
                                        self.en_passant_info = result

                                    self.turn = "white" if self.player_color == "black" else "black"
                                    current_position = self.get_position_key()
                                    self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1
                                self.selected_piece = None
        return None

    def update_player_move(self):
        current_time = pygame.time.get_ticks()
        elapsed = (current_time - self.last_update_time) / 1000
        self.last_update_time = current_time
        if self.turn == "white":
            self.white_time = max(0, self.white_time - elapsed)
            if self.white_time <= 0:
                self.game_over = True
                self.game_over_message = "Koniec czasu! Czarne wygrywają!"
        else:
            self.black_time = max(0, self.black_time - elapsed)
            if self.black_time <= 0:
                self.game_over = True
                self.game_over_message = "Koniec czasu! Białe wygrywają!"

        if not self.game_over:
            if self.is_insufficient_material():
                self.game_over = True
                self.game_over_message = "Remis! Niewystarczający materiał"
            elif any(count >= 3 for count in self.position_counts.values()):
                self.game_over = True
                self.game_over_message = "Remis! Trzykrotne powtórzenie pozycji"
            elif not ChessEngine.has_any_valid_move(self.turn, self.pieces, self.en_passant_info):
                if ChessEngine.is_in_check(self.turn, self.pieces):
                    self.game_over = True
                    self.game_over_message = "Mat! Wygrywają " + ("Czarne" if self.turn == "white" else "Białe")
                else:
                    self.game_over = True
                    self.game_over_message = "Pat!"

    def update(self):
        bot_color = "white" if self.player_color == "black" else "black"

        if not self.game_over and not self.bot_thinking:
            current_time = pygame.time.get_ticks()
            elapsed = (current_time - self.last_update_time) / 1000
            self.last_update_time = current_time
            if self.turn == "white":
                self.white_time = max(0, self.white_time - elapsed)
                if self.white_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Czarne wygrywają!"
            else:
                self.black_time = max(0, self.black_time - elapsed)
                if self.black_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Białe wygrywają!"

            if self.turn == bot_color:
                self.bot_thinking = True
                self.bot_think_time = 500
                self.bot_move = ChessEngine.get_bot_move(self.pieces, self.en_passant_info, self.turn, self.bot_depth)

        if self.bot_thinking and self.bot_think_time <= 0 and not self.game_over:
            bot_piece, bot_move_pos = self.bot_move
            if bot_piece and bot_move_pos:
                orig_pos = bot_piece.pos
                move_notation = pos_to_alg(orig_pos) + "-" + pos_to_alg(bot_move_pos)
                if self.move_pair:
                    self.move_history.append((self.move_pair[0], move_notation))
                    self.move_pair = None
                self.pieces, result = ChessEngine.simulate_move(bot_piece, bot_move_pos, self.pieces, self.en_passant_info)
                move_sound.play()

                if isinstance(result, dict) and result.get("promotion_needed"):
                    pawn = result["pawn"]
                    new_type = "queen"
                    for p in self.pieces:
                        if p.id == pawn.id:
                            p.type = new_type
                            p.image = (self.hetmanBialy if new_type == "queen" else
                                       self.wiezaBiala if new_type == "rook" else
                                       self.goniecBialy if new_type == "bishop" else
                                       self.konBialy) if p.color == "white" else \
                                      (self.hetmanCzarny if new_type == "queen" else
                                       self.wiezaCzarna if new_type == "rook" else
                                       self.goniecCzarny if new_type == "bishop" else
                                       self.konCzarny)
                            break
                    self.en_passant_info = None
                else:
                    self.en_passant_info = result

                self.last_bot_move = (orig_pos, bot_move_pos)
                self.turn = self.player_color
                current_position = self.get_position_key()
                self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1
            self.bot_thinking = False
            self.bot_move = None

        if not self.game_over:
            if self.is_insufficient_material():
                self.game_over = True
                self.game_over_message = "Remis! Niewystarczający materiał"
            elif any(count >= 3 for count in self.position_counts.values()):
                self.game_over = True
                self.game_over_message = "Remis! Trzykrotne powtórzenie pozycji"
            elif not ChessEngine.has_any_valid_move(self.turn, self.pieces, self.en_passant_info):
                if ChessEngine.is_in_check(self.turn, self.pieces):
                    self.game_over = True
                    self.game_over_message = "Mat! Wygrywają " + ("Czarne" if self.turn == "white" else "Białe")
                else:
                    self.game_over = True
                    self.game_over_message = "Pat!"

    def render(self):
        self.canvas.fill((0, 0, 0))
        self.board.draw(self.canvas, self.player_color)
        self.draw_pieces()

        if self.last_bot_move:
            orig_pos, new_pos = self.last_bot_move
            for pos in [orig_pos, new_pos]:
                display_pos = self.transform_pos(pos)
                col, row = display_pos
                rect = pygame.Rect(col * self.square_size, row * self.square_size,
                                   self.square_size, self.square_size)
                pygame.draw.rect(self.canvas, self.BOT_MOVE_COLOR, rect, int(3 * self.scale_factor))

        if self.selected_piece and self.turn == self.player_color:
            display_pos = self.transform_pos(self.selected_piece.pos)
            col, row = display_pos
            rect = pygame.Rect(col * self.square_size, row * self.square_size,
                               self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (0, 255, 0), rect, int(3 * self.scale_factor))
            valid_moves = ChessEngine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
            for move in valid_moves:
                m_display_pos = self.transform_pos(move)
                m_col, m_row = m_display_pos
                s = pygame.Surface((self.square_size, self.square_size))
                s.set_alpha(100)
                s.fill((255, 255, 0))
                self.canvas.blit(s, (m_col * self.square_size, m_row * self.square_size))

        current_king = next((p for p in self.pieces if p.type == "king" and p.color == self.turn), None)
        if current_king and ChessEngine.is_in_check(self.turn, self.pieces):
            display_pos = self.transform_pos(current_king.pos)
            col, row = display_pos
            rect = pygame.Rect(col * self.square_size, row * self.square_size,
                               self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (255, 0, 0), rect, int(3 * self.scale_factor))
            check_text = self.font.render("Szach!", True, (255, 0, 0))
            self.canvas.blit(check_text, (int(10 * self.scale_factor), int(10 * self.scale_factor)))

        sidebar_rect = pygame.Rect(self.board.board_width, 0, self.board.sidebar_width, self.total_size[1])
        pygame.draw.rect(self.canvas, self.board.dark_color, sidebar_rect)

        bottom_sidebar_height = self.total_size[1] - (self.square_size * 8)
        bottom_sidebar_rect = pygame.Rect(0, self.board.board_height, self.total_size[0], bottom_sidebar_height)
        pygame.draw.rect(self.canvas, self.board.dark_color, bottom_sidebar_rect)

        turn_text = self.font_smaller.render("Tura: " + self.turn.capitalize(), True, self.WHITE)
        self.canvas.blit(turn_text, (self.board.board_width + int(20 * self.scale_factor), int(20 * self.scale_factor)))

        if not self.nerd_view_enabled:
            moves_title = self.font_smaller.render("Historia ruchów", True, self.WHITE)
            self.canvas.blit(moves_title, (self.board.board_width + int(20 * self.scale_factor), int(50 * self.scale_factor)))
            for i, (player_move, bot_move) in enumerate(self.move_history[-6:]):
                player_text = self.small_font.render(f"Gracz: {player_move}", True, self.WHITE)
                bot_text = self.small_font.render(f"Bot: {bot_move}", True, self.WHITE)
                self.canvas.blit(player_text, (self.board.board_width + int(20 * self.scale_factor), int(80 * self.scale_factor) + i * int(50 * self.scale_factor)))
                self.canvas.blit(bot_text, (self.board.board_width + int(20 * self.scale_factor), int(105 * self.scale_factor) + i * int(50 * self.scale_factor)))
        else:
            nerd_title = self.font_smaller.render("Widok dla nerdów", True, self.WHITE)
            self.canvas.blit(nerd_title, (self.board.board_width + int(20 * self.scale_factor), int(50 * self.scale_factor)))

            bot_color = "white" if self.player_color == "black" else "black"
            eval_score = ChessEngine.evaluate_position(self.pieces, self.en_passant_info, self.turn, bot_color)
            eval_text = self.tiny_font.render(f"Eval: {eval_score}", True, self.WHITE)
            self.canvas.blit(eval_text, (self.board.board_width + int(20 * self.scale_factor), int(80 * self.scale_factor)))

            en_passant_text = self.tiny_font.render(f"En Passant: {self.en_passant_info}", True, self.WHITE)
            self.canvas.blit(en_passant_text, (self.board.board_width + int(20 * self.scale_factor), int(100 * self.scale_factor)))

            pieces_text = self.tiny_font.render("Pieces:", True, self.WHITE)
            self.canvas.blit(pieces_text, (self.board.board_width + int(20 * self.scale_factor), int(120 * self.scale_factor)))
            for i, piece in enumerate(self.pieces[:10]):
                piece_info = f"{piece.type} {piece.color} {pos_to_alg(piece.pos)}"
                piece_text = self.tiny_font.render(piece_info, True, self.WHITE)
                self.canvas.blit(piece_text, (self.board.board_width + int(20 * self.scale_factor), int(140 * self.scale_factor) + i * int(20 * self.scale_factor)))

            position_count_text = self.tiny_font.render(f"Pos. repeats: {max(self.position_counts.values(), default=0)}", True, self.WHITE)
            self.canvas.blit(position_count_text, (self.board.board_width + int(20 * self.scale_factor), int(340 * self.scale_factor)))

        mouse_pos = pygame.mouse.get_pos()
        surrender_hover = self.button_rect_surrender.collidepoint(mouse_pos) and not self.game_over
        nerd_hover = self.button_rect_nerd_view.collidepoint(mouse_pos) and not self.game_over

        player_default_color = self.WHITE if self.player_color == "white" else self.BLACK
        player_text_color = self.BLACK if self.player_color == "white" else self.WHITE
        player_hover_color = self.HOVER_COLOR_WHITE if self.player_color == "white" else self.HOVER_COLOR_BLACK

        if any([surrender_hover, nerd_hover]) and not self.button_hover_played:
            self.button_hover_played = True
        elif not any([surrender_hover, nerd_hover]):
            self.button_hover_played = False

        if surrender_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_surrender.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        surrender_color = player_hover_color if surrender_hover else player_default_color
        pygame.draw.rect(self.canvas, surrender_color, self.button_rect_surrender, border_radius=int(12 * self.scale_factor))
        pygame.draw.rect(self.canvas, player_text_color, self.button_rect_surrender, int(2 * self.scale_factor), border_radius=int(12 * self.scale_factor))
        surrender_text = self.font_smaller.render(f"Poddaj się", True, player_text_color)
        self.canvas.blit(surrender_text, (
            self.button_rect_surrender.x + (self.button_rect_surrender.width - surrender_text.get_width()) // 2,
            self.button_rect_surrender.y + (self.button_rect_surrender.height - surrender_text.get_height()) // 2
        ))

        if nerd_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_nerd_view.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        nerd_color = player_hover_color if nerd_hover else player_default_color
        pygame.draw.rect(self.canvas, nerd_color, self.button_rect_nerd_view, border_radius=int(12 * self.scale_factor))
        pygame.draw.rect(self.canvas, player_text_color, self.button_rect_nerd_view, int(2 * self.scale_factor), border_radius=int(12 * self.scale_factor))
        nerd_text = self.font_smaller.render("Widok dla nerdów", True, player_text_color)
        self.canvas.blit(nerd_text, (
            self.button_rect_nerd_view.x + (self.button_rect_nerd_view.width - nerd_text.get_width()) // 2,
            self.button_rect_nerd_view.y + (self.button_rect_nerd_view.height - nerd_text.get_height()) // 2
        ))

        if self.game_over:
            overlay = pygame.Surface(self.total_size, pygame.SRCALPHA)
            overlay.fill((0, 0, 0, 180))
            self.canvas.blit(overlay, (0, 0))
            
            text_surface = self.font.render(self.game_over_message, True, (255, 255, 255))
            text_rect = text_surface.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - int(50 * self.scale_factor)))
            self.canvas.blit(text_surface, text_rect)

            back_hover = self.button_rect_back.collidepoint(mouse_pos)
            if back_hover:
                pygame.draw.rect(self.canvas, (50, 50, 50), 
                               self.button_rect_back.move(int(5 * self.scale_factor), int(5 * self.scale_factor)), 
                               border_radius=int(10 * self.scale_factor))
            button_color = (100, 100, 100) if back_hover else (70, 70, 70)
            pygame.draw.rect(self.canvas, button_color, self.button_rect_back, border_radius=int(10 * self.scale_factor))
            pygame.draw.rect(self.canvas, (255, 255, 255), self.button_rect_back, int(2 * self.scale_factor), border_radius=int(10 * self.scale_factor))
            
            back_text = self.font_smaller.render("Wyjdź", True, (255, 255, 255))
            self.canvas.blit(back_text, (
                self.button_rect_back.x + (self.button_rect_back.width - back_text.get_width()) // 2,
                self.button_rect_back.y + (self.button_rect_back.height - back_text.get_height()) // 2
            ))

        pygame.display.update()

    def choose_promotion(self, pawn):
        promotion_options = ["queen", "rook", "bishop", "knight"]
        images = {
            "queen": self.hetmanBialy if pawn.color == "white" else self.hetmanCzarny,
            "rook": self.wiezaBiala if pawn.color == "white" else self.wiezaCzarna,
            "bishop": self.goniecBialy if pawn.color == "white" else self.goniecCzarny,
            "knight": self.konBialy if pawn.color == "white" else self.konCzarny
        }
    
        overlay = pygame.Surface(self.total_size)
        overlay.set_alpha(200)
        overlay.fill((100, 100, 100))
        self.canvas.blit(overlay, (0, 0))
    
        promotion_size = self.square_size
        total_width = promotion_size * 4
        x0 = (self.total_size[0] - total_width) // 2
        y0 = (self.total_size[1] - promotion_size) // 2
    
        for i, option in enumerate(promotion_options):
            rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
            pygame.draw.rect(self.canvas, (255, 255, 255), rect)
            pygame.draw.rect(self.canvas, (0, 0, 0), rect, int(2 * self.scale_factor))
            img = images[option]
            if img:
                img_rect = img.get_rect()
                offset_x = (promotion_size - img_rect.width) // 2
                offset_y = (promotion_size - img_rect.height) // 2
                self.canvas.blit(img, (rect.x + offset_x, rect.y + offset_y))
    
        pygame.display.update()
    
        chosen = None
        while chosen is None:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    pos = event.pos
                    for i, option in enumerate(promotion_options):
                        rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
                        if rect.collidepoint(pos):
                            chosen = option
                            break
    
        return chosen

    def choose_difficulty(self):
        COLORS = {
            'text': (255, 255, 255),
            'hover': (100, 100, 100),
            'selected': (81, 83, 85),
            'button': (70, 70, 70),
            'background': (50, 50, 50),
            'border': (200, 200, 200),
            'container': (60, 60, 60)
        }
        
        DIFFICULTIES = [
            {"name": "Łatwy", "depth": 1, "description": "Dla początkujących graczy - komputer popełnia błędy"},
            {"name": "Średni", "depth": 2, "description": "Dla średniozaawansowanych - umiarkowane wyzwanie"},
            {"name": "Zaawansowany", "depth": 3, "description": "Dla ekspertów - komputer gra na wysokim poziomie"}
        ]

        button_width = int(450 * self.scale_factor)
        button_height = int(70 * self.scale_factor)
        button_spacing = int(40 * self.scale_factor)
        container_padding = int(50 * self.scale_factor)
        border_radius = int(15 * self.scale_factor)
        
        title_height = self.font.get_height()
        desc_height = self.tiny_font.get_height() if DIFFICULTIES[0].get('description') else 0
        
        total_height = (title_height + container_padding + 
                    len(DIFFICULTIES) * button_height + 
                    (len(DIFFICULTIES) - 1) * button_spacing +
                    container_padding)
        
        container_rect = pygame.Rect(
            (self.total_size[0] - button_width - container_padding * 2) // 2,
            (self.total_size[1] - total_height) // 2,
            button_width + container_padding * 2,
            total_height
        )
        
        buttons = []
        for i, diff in enumerate(DIFFICULTIES):
            buttons.append(pygame.Rect(
                container_rect.x + container_padding,
                container_rect.y + container_padding + title_height + i * (button_height + button_spacing),
                button_width,
                button_height
            ))

        chosen = False
        while not chosen:
            self.canvas.fill(COLORS['background'])
            settings.draw_chessboard(self.canvas, self.canvas.get_width(), self.canvas.get_height())
            
            pygame.draw.rect(self.canvas, COLORS['container'], container_rect, border_radius=border_radius)
            pygame.draw.rect(self.canvas, COLORS['border'], container_rect, 3, border_radius=border_radius)
            
            mouse_pos = pygame.mouse.get_pos()

            title = self.font.render("Wybierz poziom trudności", True, COLORS['text'])
            title_x = container_rect.x + (container_rect.width - title.get_width()) // 2
            title_y = container_rect.y + container_padding // 2
            self.canvas.blit(title, (title_x, title_y))

            for i, (button, diff) in enumerate(zip(buttons, DIFFICULTIES)):
                hover = button.collidepoint(mouse_pos)
                color = COLORS['hover'] if hover else COLORS['button']
                
                pygame.draw.rect(self.canvas, color, button, border_radius=border_radius)
                pygame.draw.rect(self.canvas, COLORS['border'], button, 2, border_radius=border_radius)
                
                text = self.font.render(diff['name'], True, COLORS['text'])
                text_rect = text.get_rect(center=button.center)
                self.canvas.blit(text, text_rect)
                
                desc_text = self.tiny_font.render(diff['description'], True, COLORS['text'])
                desc_x = button.x + (button.width - desc_text.get_width()) // 2
                desc_y = button.bottom + int(10 * self.scale_factor)
                self.canvas.blit(desc_text, (desc_x, desc_y))

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    return False
                elif event.type == pygame.VIDEORESIZE:
                    self.update_scale()
                    return self.choose_difficulty()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    for i, button in enumerate(buttons):
                        if button.collidepoint(mouse_pos):
                            self.bot_depth = DIFFICULTIES[i]['depth']
                            chosen = True
                            
                            pygame.draw.rect(self.canvas, COLORS['selected'], button, border_radius=border_radius)
                            text = self.font.render(DIFFICULTIES[i]['name'], True, COLORS['text'])
                            self.canvas.blit(text, text.get_rect(center=button.center))
                            pygame.display.update()
                            pygame.time.wait(200)

            pygame.display.update()
        
        return True

    def choose_color(self):
        COLORS = {
            'background': (50, 50, 50),
            'white': (255, 255, 255),
            'black': (0, 0, 0),
            'hover': (100, 100, 100),
            'selected': (81, 83, 85),
            'border': (200, 200, 200),
            'container': (60, 60, 60),
            'text_white': (0, 0, 0),
            'text_black': (255, 255, 255)
        }
        
        OPTIONS = [
            {"name": "Biały", "color": 'white', "description": "Zaczynasz grę jako białe"},
            {"name": "Czarny", "color": 'black', "description": "Zaczynasz grę jako czarne"}
        ]

        button_width = int(350 * self.scale_factor)
        button_height = int(70 * self.scale_factor)
        button_spacing = int(40 * self.scale_factor)
        container_padding = int(50 * self.scale_factor)
        border_radius = int(15 * self.scale_factor)
        
        title_height = self.font.get_height()
        desc_height = self.tiny_font.get_height()
        total_height = (title_height + container_padding * 2 + 
                    len(OPTIONS) * button_height + 
                    (len(OPTIONS) - 1) * button_spacing)
        
        additional_space = len(OPTIONS) * self.tiny_font.get_height() * 2

        container_rect = pygame.Rect(
            (self.total_size[0] - button_width - container_padding * 2) // 2,
            max(20, (self.total_size[1] - total_height - additional_space) // 2),
            button_width + container_padding * 2,
            total_height + additional_space + int(40 * self.scale_factor)
        )

        buttons = []
        for i, option in enumerate(OPTIONS):
            buttons.append(pygame.Rect(
                container_rect.x + container_padding,
                container_rect.y + container_padding + title_height + i * (button_height + button_spacing),
                button_width,
                button_height
            ))

        chosen = False
        while not chosen:
            self.canvas.fill(COLORS['background'])
            settings.draw_chessboard(self.canvas, self.canvas.get_width(), self.canvas.get_height())
            
            pygame.draw.rect(self.canvas, COLORS['container'], container_rect, border_radius=border_radius)
            pygame.draw.rect(self.canvas, COLORS['border'], container_rect, 3, border_radius=border_radius)
            
            mouse_pos = pygame.mouse.get_pos()

            title = self.font.render("Wybierz swój kolor", True, COLORS['white'])
            title_x = container_rect.x + (container_rect.width - title.get_width()) // 2
            title_y = container_rect.y + container_padding // 2
            self.canvas.blit(title, (title_x, title_y))

            for i, (button, option) in enumerate(zip(buttons, OPTIONS)):
                hover = button.collidepoint(mouse_pos)
                btn_color = COLORS['hover'] if hover else COLORS[option['color']]
                text_color = COLORS['text_white'] if option['color'] == 'white' else COLORS['text_black']
                
                pygame.draw.rect(self.canvas, btn_color, button, border_radius=border_radius)
                pygame.draw.rect(self.canvas, COLORS['border'], button, 2, border_radius=border_radius)
                
                text = self.font.render(option['name'], True, text_color)
                text_rect = text.get_rect(center=button.center)
                self.canvas.blit(text, text_rect)
                
                desc_text = self.tiny_font.render(option['description'], True, COLORS['white'])
                desc_x = button.x + (button.width - desc_text.get_width()) // 2
                desc_y = button.bottom + int(10 * self.scale_factor)
                self.canvas.blit(desc_text, (desc_x, desc_y))

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.VIDEORESIZE:
                    self.update_scale()
                    return self.choose_color()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    for i, button in enumerate(buttons):
                        if button.collidepoint(mouse_pos):
                            self.player_color = OPTIONS[i]['color']
                            self.board.player_color = self.player_color
                            chosen = True
                            
                            pygame.draw.rect(self.canvas, COLORS['selected'], button, border_radius=border_radius)
                            text = self.small_font.render(OPTIONS[i]['name'], True, 
                                                        COLORS['text_white'] if OPTIONS[i]['color'] == 'white' else COLORS['text_black'])
                            self.canvas.blit(text, text.get_rect(center=button.center))
                            pygame.display.update()
                            pygame.time.wait(200)

            pygame.display.update()    
        return True

    def run(self):
        if not self.choose_difficulty() or not self.choose_color():
            pygame.quit()
            sys.exit()

        self.update_scale()
        bot_color = "white" if self.player_color == "black" else "black"

        if self.player_color == "black":
            self.turn = "white"
            self.update()
            self.render()

        while True:
            self.update_scale()
            result = self.process_events()
            if result == "quit":
                pygame.quit()
                sys.exit()
            elif result == "main_menu":
                return "main_menu"

            self.update()
            self.render()
            delta = self.clock.tick(60)
            if self.bot_thinking:
                self.bot_think_time -= delta

if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((940, 640), pygame.RESIZABLE)
    game = ChessGame(screen)
    game.run()
    pygame.quit()