from utils import *
from piece import *


class ChessEngine:
    def get_attack_squares(self, piece, pieces_list):
        attacks = []
        col, row = piece.pos

        if piece.type == "pawn":
            direction = -1 if piece.color == "white" else 1
            for dc in [-1, 1]:
                diag = (col + dc, row + direction)
                if is_inside(*diag):
                    attacks.append(diag)
        elif piece.type == "knight":
            knight_moves = [(2, 1), (2, -1), (-2, 1), (-2, -1),
                           (1, 2), (1, -2), (-1, 2), (-1, -2)]
            for dx, dy in knight_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    attacks.append(new_pos)
        elif piece.type in ["bishop", "rook", "queen"]:
            directions = []
            if piece.type in ["rook", "queen"]:
                directions += [(1, 0), (-1, 0), (0, 1), (0, -1)]
            if piece.type in ["bishop", "queen"]:
                directions += [(1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in directions:
                new_col, new_row = col, row
                while True:
                    new_col += dx
                    new_row += dy
                    if not is_inside(new_col, new_row):
                        break
                    attacks.append((new_col, new_row))
                    if get_piece_at((new_col, new_row), pieces_list) is not None:
                        break
        elif piece.type == "king":
            king_moves = [(1, 0), (-1, 0), (0, 1), (0, -1),
                         (1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in king_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    attacks.append(new_pos)
        return attacks

    def is_in_check(self, color, pieces_list):
        king = next((p for p in pieces_list if p.type == "king" and p.color == color), None)
        if king is None:
            return True
        king_pos = king.pos
        for p in pieces_list:
            if p.color != color and king_pos in self.get_attack_squares(p, pieces_list):
                return True
        return False

    def simulate_move(self, piece, move, pieces_list, en_passant_info):
        new_pieces = [p.clone() for p in pieces_list]
        moving_piece = next((p for p in new_pieces if p.id == piece.id), None)
        if moving_piece is None:
            return new_pieces, None

        original_pos = moving_piece.pos

        if moving_piece.type == "king" and abs(move[0] - original_pos[0]) == 2:
            if move[0] > original_pos[0]:
                rook_start = (7, original_pos[1])
                rook_end = (original_pos[0] + 1, original_pos[1])
            else:
                rook_start = (0, original_pos[1])
                rook_end = (original_pos[0] - 1, original_pos[1])
            for p in new_pieces:
                if p.pos == rook_start and p.type == "rook" and p.color == moving_piece.color:
                    p.pos = rook_end
                    p.has_moved = True
                    break
            moving_piece.pos = move
            new_en_passant_info = None
        else:
            is_en_passant = False
            if moving_piece.type == "pawn" and en_passant_info and move == en_passant_info["target"]:
                is_en_passant = True

            if not is_en_passant:
                target = next((p for p in new_pieces if p.id != moving_piece.id and p.pos == move), None)
                if target:
                    new_pieces.remove(target)
            else:
                capture_pos = (move[0], original_pos[1])
                target = next((p for p in new_pieces if p.id != moving_piece.id and p.pos == capture_pos and p.type == "pawn"), None)
                if target:
                    new_pieces.remove(target)

            moving_piece.pos = move
            new_en_passant_info = None
            if moving_piece.type == "pawn" and abs(move[1] - original_pos[1]) == 2:
                passed_row = (original_pos[1] + move[1]) // 2
                new_en_passant_info = {"target": (move[0], passed_row), "pawn_id": moving_piece.id}
            moving_piece.has_moved = True

        return new_pieces, new_en_passant_info

    def get_valid_moves(self, piece, pieces_list, en_passant_info):
        candidate_moves = []
        col, row = piece.pos
        color = piece.color

        if piece.type == "pawn":
            direction = -1 if color == "white" else 1
            start_row = 6 if color == "white" else 1
            forward = (col, row + direction)
            if is_inside(*forward) and get_piece_at(forward, pieces_list) is None:
                candidate_moves.append(forward)
                if row == start_row:
                    forward2 = (col, row + 2 * direction)
                    if is_inside(*forward2) and get_piece_at(forward2, pieces_list) is None and get_piece_at(forward, pieces_list) is None:
                        candidate_moves.append(forward2)
            for dc in [-1, 1]:
                diag = (col + dc, row + direction)
                if is_inside(*diag):
                    target = get_piece_at(diag, pieces_list)
                    if target and target.color != color:
                        candidate_moves.append(diag)
            if en_passant_info and en_passant_info["target"][1] == row + direction and abs(en_passant_info["target"][0] - col) == 1:
                candidate_moves.append(en_passant_info["target"])
        elif piece.type == "knight":
            knight_moves = [(2, 1), (2, -1), (-2, 1), (-2, -1),
                           (1, 2), (1, -2), (-1, 2), (-1, -2)]
            for dx, dy in knight_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = get_piece_at(new_pos, pieces_list)
                    if target is None or target.color != color:
                        candidate_moves.append(new_pos)
        elif piece.type in ["bishop", "rook", "queen"]:
            directions = []
            if piece.type in ["rook", "queen"]:
                directions += [(1, 0), (-1, 0), (0, 1), (0, -1)]
            if piece.type in ["bishop", "queen"]:
                directions += [(1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in directions:
                new_col, new_row = col, row
                while True:
                    new_col += dx
                    new_row += dy
                    if not is_inside(new_col, new_row):
                        break
                    target = get_piece_at((new_col, new_row), pieces_list)
                    if target is None:
                        candidate_moves.append((new_col, new_row))
                    else:
                        if target.color != color:
                            candidate_moves.append((new_col, new_row))
                        break
        elif piece.type == "king":
            king_moves = [(1, 0), (-1, 0), (0, 1), (0, -1),
                         (1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in king_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    target = get_piece_at(new_pos, pieces_list)
                    if target is None or target.color != color:
                        candidate_moves.append(new_pos)
            if not piece.has_moved and not self.is_in_check(color, pieces_list):
                if get_piece_at((col+1, row), pieces_list) is None and get_piece_at((col+2, row), pieces_list) is None:
                    rook = get_piece_at((7, row), pieces_list)
                    if rook and rook.type == "rook" and rook.color == color and not rook.has_moved:
                        safe = True
                        for pos in [(col+1, row), (col+2, row)]:
                            sim_pieces, _ = self.simulate_move(piece, pos, pieces_list, en_passant_info)
                            if self.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((col+2, row))
                if get_piece_at((col-1, row), pieces_list) is None and get_piece_at((col-2, row), pieces_list) is None and get_piece_at((col-3, row), pieces_list) is None:
                    rook = get_piece_at((0, row), pieces_list)
                    if rook and rook.type == "rook" and rook.color == color and not rook.has_moved:
                        safe = True
                        for pos in [(col-1, row), (col-2, row)]:
                            sim_pieces, _ = self.simulate_move(piece, pos, pieces_list, en_passant_info)
                            if self.is_in_check(color, sim_pieces):
                                safe = False
                                break
                        if safe:
                            candidate_moves.append((col-2, row))
        valid_moves = []
        for move in candidate_moves:
            sim_pieces, sim_en_passant = self.simulate_move(piece, move, pieces_list, en_passant_info)
            if not self.is_in_check(color, sim_pieces):
                valid_moves.append(move)
        return valid_moves

    def has_any_valid_move(self, color, pieces_list, en_passant_info):
        for p in pieces_list:
            if p.color == color and self.get_valid_moves(p, pieces_list, en_passant_info):
                return True
        return False

    def get_hint(self, piece, pieces_list, en_passant_info):
        valid_moves = self.get_valid_moves(piece, pieces_list, en_passant_info)
        if not valid_moves:
            return None
        capturing_moves = []
        for move in valid_moves:
            target = get_piece_at(move, pieces_list)
            if target and target.color != piece.color:
                capturing_moves.append(move)
        if capturing_moves:
            return random.choice(capturing_moves)
        return random.choice(valid_moves)
