import pygame
import sys
import os
from utils import asset_path, pos_to_alg, load_image
from board import Board
from piece import Piece
import engine

# Dodajemy ścieżkę nadrzędną do sys.path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from chess_assets import load_chess_pieces, get_chess_style
import settings
import style

# Funkcja do uzyskania poprawnej ścieżki zasobów
def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        base_path = sys._MEIPASS
    except AttributeError:
        # W trybie developerskim użyj ścieżki względem lokalizacji pliku
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

# Inicjalizacja ustawień
settings.load_settings()

# Ładowanie dźwięku ruchu z poprawną ścieżką
move_sound = pygame.mixer.Sound(resource_path(os.path.join("assets/audio", settings.CURRENT_SOUND)))
pygame.mixer.Sound.set_volume(move_sound, (settings.sfprev_value) / 100)

class Game:
    def __init__(self, screen):
        self.screen = screen
        pygame.font.init()

        # Base dimensions for scaling
        self.BASE_SQUARE_SIZE = 80
        self.BASE_SIDEBAR_WIDTH = 300
        self.BASE_WIDTH = self.BASE_SQUARE_SIZE * 8 + self.BASE_SIDEBAR_WIDTH
        self.BASE_HEIGHT = self.BASE_SQUARE_SIZE * 8
        self.SIDEBAR_WIDTH_RATIO = self.BASE_SIDEBAR_WIDTH / self.BASE_WIDTH  # ~0.319 (sidebar is ~31.9% of base width)

        # Initial setup
        self.canvas = pygame.display.set_mode((self.BASE_WIDTH, self.BASE_HEIGHT), pygame.RESIZABLE)
        pygame.display.set_caption("SZACHY GRACZ VS GRACZ - Motorola Science Cup 2025")
        self.total_size = (self.BASE_WIDTH, self.BASE_HEIGHT)  # Initialize total_size

        # Game state variables
        self.game_mode = None
        self.white_time = 0
        self.black_time = 0
        self.increment = 0
        self.last_update_time = pygame.time.get_ticks()

        self.pieces = []
        self.piece_id_counter = 0
        self.turn = "white"
        self.selected_piece = None
        self.en_passant_info = None
        self.move_history = []
        self.state_history = []
        self.position_counts = {}
        self.game_over = False
        self.game_over_message = ""
        self.clock = pygame.time.Clock()

        self.animating = False
        self.animating_piece = None
        self.anim_start_pos = None
        self.anim_end_pos = None
        self.anim_start_time = 0
        self.anim_duration = 500
        self.notation_type = "short"

        self.update_scale()  # Initial scaling
        self.choose_mode()
        self.setup_pieces()

    def update_scale(self):
        """Update scaling factor and all dependent sizes proportionally based on current window size."""
        current_width, current_height = self.canvas.get_size()
        self.total_size = (current_width, current_height)  # Use full window size

        # Proportional scaling: sidebar takes a fixed percentage of width, board takes the rest
        sidebar_width = int(current_width * self.SIDEBAR_WIDTH_RATIO)
        board_width = current_width - sidebar_width
        self.square_size = min(board_width // 8, current_height // 8)  # Board scales with smaller dimension
        sidebar_width = current_width - (self.square_size * 8)  # Adjust sidebar to fill remaining width

        self.scale_factor = self.square_size / self.BASE_SQUARE_SIZE  # Scale factor based on square size
        self.board = Board(square_size=self.square_size, sidebar_width=sidebar_width)

        # Update fonts
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(36 * self.scale_factor))
        self.small_font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, int(24 * self.scale_factor))
        self.small_button_font = pygame.font.Font(style.MONTSERRAT_REGULAR, int(21 * self.scale_factor))

        # Reload chess pieces at high quality for the current square size
        self.chess_pieces = load_chess_pieces(self.square_size)
        self.pionekBialy = self.chess_pieces["pionekB"]
        self.pionekCzarny = self.chess_pieces["pionekC"]
        self.wiezaBiala = self.chess_pieces["wiezaB"]
        self.wiezaCzarna = self.chess_pieces["wiezaC"]
        self.konBialy = self.chess_pieces["konB"]
        self.konCzarny = self.chess_pieces["konC"]
        self.goniecBialy = self.chess_pieces["goniecB"]
        self.goniecCzarny = self.chess_pieces["goniecC"]
        self.hetmanBialy = self.chess_pieces["hetmanB"]
        self.hetmanCzarny = self.chess_pieces["hetmanC"]
        self.krolBialy = self.chess_pieces["krolB"]
        self.krolCzarny = self.chess_pieces["krolC"]

        # Update existing pieces with new high-quality images
        piece_map = {
            "pawn": ("pionekB", "pionekC"),
            "rook": ("wiezaB", "wiezaC"),
            "knight": ("konB", "konC"),
            "bishop": ("goniecB", "goniecC"),
            "queen": ("hetmanB", "hetmanC"),
            "king": ("krolB", "krolC")
        }
        for piece in self.pieces:
            white_key, black_key = piece_map[piece.type]
            piece.image = self.chess_pieces[white_key if piece.color == "white" else black_key]

        # Update button positions to fit right sidebar proportionally
        button_x = self.square_size * 8 + int(20 * self.scale_factor)  # Start at right sidebar edge
        button_width = min(int(200 * self.scale_factor), sidebar_width - int(40 * self.scale_factor))  # Fit within sidebar
        self.button_rect_surrender = pygame.Rect(button_x, int(420 * self.scale_factor), button_width, int(40 * self.scale_factor))
        self.button_rect_draw = pygame.Rect(button_x, int(470 * self.scale_factor), button_width, int(40 * self.scale_factor))
        self.button_rect_undo = pygame.Rect(button_x, int(520 * self.scale_factor), button_width, int(40 * self.scale_factor))
        self.button_rect_notation = pygame.Rect(button_x, int(570 * self.scale_factor), button_width, int(40 * self.scale_factor))
        self.button_rect_back = pygame.Rect(self.total_size[0] // 2 - int(100 * self.scale_factor), self.total_size[1] // 2 + int(20 * self.scale_factor), int(200 * self.scale_factor), int(40 * self.scale_factor))

    def choose_mode(self):
        # Pobieramy kolory z aktualnego zestawu szachowego
        chess_light = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]  # Jasny kolor
        chess_dark = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]   # Ciemny kolor
        
        # Ustawienia kolorów
        BG_COLOR = chess_dark  # Tło w ciemnym kolorze
        DIALOG_COLOR = chess_light  # Kontener w jasnym kolorze
        BUTTON_COLOR = chess_light  # Przyciski w JASNYM kolorze (zmiana)
        TEXT_COLOR = chess_dark  # Tekst w ciemnym kolorze
        SHADOW_COLOR = (
            max(chess_dark[0] - 30, 0),
            max(chess_dark[1] - 30, 0),
            max(chess_dark[2] - 30, 0)
        )  # Cień

        # Wymiary dialogu
        dialog_width = min(int(self.total_size[0] * 0.7), 800)
        dialog_height = min(int(self.total_size[1] * 0.6), 600)
        dialog_x = (self.total_size[0] - dialog_width) // 2
        dialog_y = (self.total_size[1] - dialog_height) // 2
        dialog_rect = pygame.Rect(dialog_x, dialog_y, dialog_width, dialog_height)

        # Tło
        self.canvas.fill(BG_COLOR)
        
        # Kontener dialogu
        pygame.draw.rect(self.canvas, SHADOW_COLOR, dialog_rect.move(5, 5), border_radius=15)
        pygame.draw.rect(self.canvas, DIALOG_COLOR, dialog_rect, border_radius=15)
        pygame.draw.rect(self.canvas, chess_dark, dialog_rect, 3, border_radius=15)

        # Tytuł
        title_font = pygame.font.Font(style.MONTSERRAT_BOLD, min(int(36 * self.scale_factor), 40))
        title_text = title_font.render("WYBÓR TRYBU GRY", True, chess_dark)
        title_rect = title_text.get_rect(center=(dialog_rect.centerx, dialog_rect.y + int(dialog_height * 0.15)))
        self.canvas.blit(title_text, title_rect)

        # Przyciski (zwiększone rozmiary)
        button_width = int(dialog_width * 0.45)  # 45% szerokości
        button_height = int(dialog_height * 0.22)  # 22% wysokości
        button_spacing = int(dialog_width * 0.04)
        button_y = dialog_rect.y + int(dialog_height * 0.55)

        # Utworzenie przycisków
        buttons = [
            pygame.Rect(dialog_rect.centerx - button_width - button_spacing//2, button_y, button_width, button_height),
            pygame.Rect(dialog_rect.centerx + button_spacing//2, button_y, button_width, button_height)
        ]
        button_labels = ["KLASYCZNY", "BŁYSKAWICZNY"]
        button_times = ["5 minut", "3 minuty"]

        # Rysowanie przycisków
        mouse_pos = pygame.mouse.get_pos()
        button_font = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, min(int(26 * self.scale_factor), 30))
        
        for i, button in enumerate(buttons):
            # Efekt hover
            hover = button.collidepoint(mouse_pos)
            color = BUTTON_COLOR
            
            # Rysowanie przycisku
            pygame.draw.rect(self.canvas, color, button, border_radius=12)
            pygame.draw.rect(self.canvas, chess_dark, button, 2, border_radius=12)
            
            # Tekst główny
            label_text = button_font.render(button_labels[i], True, TEXT_COLOR)
            label_rect = label_text.get_rect(centerx=button.centerx, y=button.centery - 20)
            self.canvas.blit(label_text, label_rect)
            
            # Czas gry
            time_font = pygame.font.Font(style.MONTSERRAT_REGULAR, min(int(20 * self.scale_factor), 24))
            time_text = time_font.render(button_times[i], True, TEXT_COLOR)
            time_rect = time_text.get_rect(centerx=button.centerx, y=button.centery + 10)
            self.canvas.blit(time_text, time_rect)

        pygame.display.update()

        # Obsługa wyboru
        while self.game_mode is None:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    pos = event.pos
                    if buttons[0].collidepoint(pos):
                        self.game_mode = "klasyczny"
                        self.white_time = 300
                        self.black_time = 300
                    elif buttons[1].collidepoint(pos):
                        self.game_mode = "błyskawiczny"
                        self.white_time = 180
                        self.black_time = 180
                elif event.type == pygame.VIDEORESIZE:
                    self.update_scale()
                    return self.choose_mode()
            
    def apply_increment(self):
        if self.turn == "white":
            self.white_time += self.increment
        else:
            self.black_time += self.increment

    def add_piece(self, piece_type, color, col, row, image_white, image_black):
        image = image_white if color == "white" else image_black
        self.pieces.append(Piece(self.piece_id_counter, piece_type, color, (col, row), image))
        self.piece_id_counter += 1

    def setup_pieces(self):
        self.pieces.clear()  # Clear existing pieces before re-adding
        self.add_piece("rook", "white", 0, 7, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "white", 1, 7, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "white", 2, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "white", 3, 7, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "white", 4, 7, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "white", 5, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "white", 6, 7, self.konBialy, self.konCzarny)
        self.add_piece("rook", "white", 7, 7, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "white", col, 6, self.pionekBialy, self.pionekCzarny)

        self.add_piece("rook", "black", 0, 0, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "black", 1, 0, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "black", 2, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "black", 3, 0, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "black", 4, 0, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "black", 5, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "black", 6, 0, self.konBialy, self.konCzarny)
        self.add_piece("rook", "black", 7, 0, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "black", col, 1, self.pionekBialy, self.pionekCzarny)

    def get_position_key(self):
        board_state = [f"{piece.type}{piece.color}{piece.pos}" for piece in sorted(self.pieces, key=lambda p: (p.pos[0], p.pos[1]))]
        return "|".join(board_state) + f"|{self.turn}|{self.en_passant_info}"

    def draw_pieces(self):
        for piece in self.pieces:
            if piece == self.animating_piece and self.animating:
                anim_col, anim_row = self.get_anim_pos()
                x = anim_col * self.square_size
                y = anim_row * self.square_size
                image_rect = piece.image.get_rect()
                offset_x = (self.square_size - image_rect.width) // 2
                offset_y = (self.square_size - image_rect.height) // 2
                self.canvas.blit(piece.image, (x + offset_x, y + offset_y))
            else:
                piece.draw(self.canvas, self.square_size, left_offset=0)

    def get_anim_pos(self):
        elapsed = pygame.time.get_ticks() - self.anim_start_time
        if elapsed >= self.anim_duration:
            self.animating = False
            return self.anim_end_pos
        progress = elapsed / self.anim_duration
        start_col, start_row = self.anim_start_pos
        end_col, end_row = self.anim_end_pos
        current_col = start_col + (end_col - start_col) * progress
        current_row = start_row + (end_row - start_row) * progress
        return (current_col, current_row)

    def surrender(self):
        self.game_over = True
        self.game_over_message = "Poddanie! " + ("Czarne" if self.turn == "white" else "Białe") + " wygrywają!"
        print(self.game_over_message)

    def propose_draw(self):
        self.render()
        dialog_width, dialog_height = int(600 * self.scale_factor), int(300 * self.scale_factor)
        dialog_x = (self.total_size[0] - dialog_width) // 2
        dialog_y = (self.total_size[1] - dialog_height) // 2
        dialog_rect = pygame.Rect(dialog_x, dialog_y, dialog_width, dialog_height)
        
        chess_dark = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]
        chess_light = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]
        shadow_color = (50, 50, 50)
        
        overlay = pygame.Surface(self.total_size, pygame.SRCALPHA)
        overlay.fill((100, 100, 100, 200))
        self.canvas.blit(overlay, (0, 0))

        pygame.draw.rect(self.canvas, shadow_color, dialog_rect.move(int(5 * self.scale_factor), int(5 * self.scale_factor)), border_radius=int(15 * self.scale_factor))
        pygame.draw.rect(self.canvas, chess_dark, dialog_rect, border_radius=int(15 * self.scale_factor))
        pygame.draw.rect(self.canvas, (0, 0, 0), dialog_rect, int(3 * self.scale_factor), border_radius=int(15 * self.scale_factor))

        player_color = "Białe" if self.turn == "white" else "Czarne"
        title_text = self.font.render("Propozycja remisu", True, (255, 255, 255))
        question_text = self.small_font.render(f"{player_color} proponują remis. Przyjmujesz?", True, (255, 255, 255))
        
        self.canvas.blit(title_text, title_text.get_rect(center=(dialog_rect.centerx, dialog_rect.y + int(40 * self.scale_factor))))
        self.canvas.blit(question_text, question_text.get_rect(center=(dialog_rect.centerx, dialog_rect.y + int(100 * self.scale_factor))))

        button_width, button_height = int(120 * self.scale_factor), int(50 * self.scale_factor)
        button_y = dialog_rect.y + int(180 * self.scale_factor)
        button_spacing = int(30 * self.scale_factor)
        
        yes_button = pygame.Rect(dialog_rect.centerx - button_width - button_spacing // 2, button_y, button_width, button_height)
        no_button = pygame.Rect(dialog_rect.centerx + button_spacing // 2, button_y, button_width, button_height)

        pygame.draw.rect(self.canvas, shadow_color, yes_button.move(int(5 * self.scale_factor), int(5 * self.scale_factor)), border_radius=int(8 * self.scale_factor))
        pygame.draw.rect(self.canvas, shadow_color, no_button.move(int(5 * self.scale_factor), int(5 * self.scale_factor)), border_radius=int(8 * self.scale_factor))
        pygame.draw.rect(self.canvas, chess_light, yes_button, border_radius=int(8 * self.scale_factor))
        pygame.draw.rect(self.canvas, chess_light, no_button, border_radius=int(8 * self.scale_factor))
        pygame.draw.rect(self.canvas, (0, 0, 0), yes_button, int(2 * self.scale_factor), border_radius=int(8 * self.scale_factor))
        pygame.draw.rect(self.canvas, (0, 0, 0), no_button, int(2 * self.scale_factor), border_radius=int(8 * self.scale_factor))

        yes_text = self.small_font.render("Tak", True, (0, 0, 0))
        no_text = self.small_font.render("Nie", True, (0, 0, 0))
        self.canvas.blit(yes_text, yes_text.get_rect(center=yes_button.center))
        self.canvas.blit(no_text, no_text.get_rect(center=no_button.center))

        pygame.display.update()

        choice = None
        while choice is None:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    pos = event.pos
                    if yes_button.collidepoint(pos):
                        choice = True
                    elif no_button.collidepoint(pos):
                        choice = False

        if choice:
            self.game_over = True
            self.game_over_message = "Remis przyjęty!"
            print(self.game_over_message)

    def undo_move(self):
        if self.state_history:
            self.pieces, self.turn, self.en_passant_info = self.state_history.pop()
            current_position = self.get_position_key()
            self.position_counts[current_position] = self.position_counts.get(current_position, 0) - 1
            if self.position_counts[current_position] <= 0:
                del self.position_counts[current_position]
            self.selected_piece = None
            self.animating = False
            self.game_over = False
            if self.move_history:
                self.move_history.pop()
            print("Move undone")

    def check_insufficient_material(self):
        if len(self.pieces) > 4:
            return False

        white_pieces = [p for p in self.pieces if p.color == "white"]
        black_pieces = [p for p in self.pieces if p.color == "black"]

        if len(white_pieces) == 1 and len(black_pieces) == 1:
            return True
        if len(white_pieces) == 1 and len(black_pieces) == 2:
            other_piece = next(p for p in black_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(black_pieces) == 1 and len(white_pieces) == 2:
            other_piece = next(p for p in white_pieces if p.type != "king")
            return other_piece.type in ["knight", "bishop"]
        if len(white_pieces) == 2 and len(black_pieces) == 2:
            white_non_king = next(p for p in white_pieces if p.type != "king")
            black_non_king = next(p for p in black_pieces if p.type != "king")
            if white_non_king.type == "bishop" and black_non_king.type == "bishop":
                white_color = (white_non_king.pos[0] + white_non_king.pos[1]) % 2
                black_color = (black_non_king.pos[0] + black_non_king.pos[1]) % 2
                return white_color == black_color
        return False

    def format_time(self, seconds):
        minutes = int(seconds // 60)
        seconds = int(seconds % 60)
        return f"{minutes:02d}:{seconds:02d}"

    def format_move_long(self, move, piece):
        start_pos, end_pos = move.split("-")
        piece_names = {
            "pawn": "Pion",
            "rook": "Wieża",
            "knight": "Skoczek",
            "bishop": "Goniec",
            "queen": "Hetman",
            "king": "Król"
        }
        piece_name = piece_names.get(piece.type, piece.type.capitalize())
        return f"{piece_name} {start_pos} na {end_pos}"

    def process_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return "quit"
            if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_pos = pygame.mouse.get_pos()
                if self.game_over:
                    if self.button_rect_back.collidepoint(mouse_pos):
                        pygame.quit()
                        sys.exit()
                    return None
                if self.animating:
                    return None

                if self.button_rect_surrender.collidepoint(mouse_pos):
                    self.surrender()
                    return None
                if self.button_rect_draw.collidepoint(mouse_pos):
                    self.propose_draw()
                    return None
                if self.button_rect_undo.collidepoint(mouse_pos):
                    self.undo_move()
                    return None
                if self.button_rect_notation.collidepoint(mouse_pos):
                    self.notation_type = "long" if self.notation_type == "short" else "short"
                    return None

                if mouse_pos[0] < self.board.board_width and mouse_pos[1] < self.board.board_height:
                    col = mouse_pos[0] // self.square_size
                    row = mouse_pos[1] // self.square_size
                    clicked_pos = (col, row)

                    if self.selected_piece is None:
                        piece = next((p for p in self.pieces if p.pos == clicked_pos and p.color == self.turn), None)
                        if piece:
                            self.selected_piece = piece
                    else:
                        target_piece = next((p for p in self.pieces if p.pos == clicked_pos and p.color == self.turn), None)
                        if target_piece:
                            self.selected_piece = target_piece
                        else:
                            valid_moves = engine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
                            if clicked_pos in valid_moves:
                                self.state_history.append(([p.clone() for p in self.pieces], self.turn, self.en_passant_info))
                                self.animating = True
                                self.animating_piece = self.selected_piece
                                self.anim_start_pos = self.selected_piece.pos
                                self.anim_end_pos = clicked_pos
                                self.anim_start_time = pygame.time.get_ticks()
                                move_notation = pos_to_alg(self.anim_start_pos) + "-" + pos_to_alg(self.anim_end_pos)
                                self.move_history.append(move_notation)
                                self.pieces, self.en_passant_info = engine.simulate_move(self.selected_piece, clicked_pos, self.pieces, self.en_passant_info)
                                moved_piece = next((p for p in self.pieces if p.id == self.selected_piece.id), None)
                                if moved_piece and moved_piece.type == "pawn":
                                    if (moved_piece.color == "white" and moved_piece.pos[1] == 0) or (moved_piece.color == "black" and moved_piece.pos[1] == 7):
                                        self.handle_promotion(moved_piece)
                                self.turn = "black" if self.turn == "white" else "white"
                                self.apply_increment()
                                current_position = self.get_position_key()
                                move_sound.play()
                                self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1
                            self.selected_piece = None
        return None

    def handle_promotion(self, pawn):
        promotion_options = ["queen", "rook", "bishop", "knight"]
        images = {
            "queen": self.hetmanBialy if pawn.color == "white" else self.hetmanCzarny,
            "rook": self.wiezaBiala if pawn.color == "white" else self.wiezaCzarna,
            "bishop": self.goniecBialy if pawn.color == "white" else self.goniecCzarny,
            "knight": self.konBialy if pawn.color == "white" else self.konCzarny
        }
        overlay = pygame.Surface(self.total_size)
        overlay.set_alpha(200)
        overlay.fill((100, 100, 100))
        self.canvas.blit(overlay, (0, 0))
        promotion_size = self.square_size
        total_width = promotion_size * 4
        x0 = (self.total_size[0] - total_width) // 2
        y0 = (self.total_size[1] - promotion_size) // 2
        for i, option in enumerate(promotion_options):
            rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
            pygame.draw.rect(self.canvas, (255, 255, 255), rect)
            pygame.draw.rect(self.canvas, (0, 0, 0), rect, int(2 * self.scale_factor))
            img = images[option]
            if img:
                img_rect = img.get_rect()
                offset_x = (promotion_size - img_rect.width) // 2
                offset_y = (promotion_size - img_rect.height) // 2
                self.canvas.blit(img, (rect.x + offset_x, rect.y + offset_y))
        pygame.display.update()

        chosen = None
        while chosen is None:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    pos = event.pos
                    for i, option in enumerate(promotion_options):
                        rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
                        if rect.collidepoint(pos):
                            chosen = option
                            break
        pawn.type = chosen
        pawn.image = images[chosen]

    def update(self):
        if self.animating:
            elapsed = pygame.time.get_ticks() - self.anim_start_time
            if elapsed >= self.anim_duration:
                self.animating = False
                self.animating_piece.pos = self.anim_end_pos

        if not self.game_over:
            current_time = pygame.time.get_ticks()
            elapsed = (current_time - self.last_update_time) / 1000
            self.last_update_time = current_time

            if self.turn == "white" and not self.animating:
                self.white_time = max(0, self.white_time - elapsed)
                if self.white_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Czarne wygrywają!"
            elif self.turn == "black" and not self.animating:
                self.black_time = max(0, self.black_time - elapsed)
                if self.black_time <= 0:
                    self.game_over = True
                    self.game_over_message = "Koniec czasu! Białe wygrywają!"

            if not engine.has_any_valid_move(self.turn, self.pieces, self.en_passant_info):
                if engine.is_in_check(self.turn, self.pieces):
                    self.game_over = True
                    self.game_over_message = "Szach mat! " + ("Czarne" if self.turn == "white" else "Białe") + " wygrywają!"
                else:
                    self.game_over = True
                    self.game_over_message = "Pat!"
            elif self.check_insufficient_material():
                self.game_over = True
                self.game_over_message = "Remis przez niewystarczającą ilość materiału!"
            elif any(count >= 3 for count in self.position_counts.values()):
                self.game_over = True
                self.game_over_message = "Remis przez potrójne powtórzenie!"

    def render(self):
        self.canvas.fill((0, 0, 0))  # This will be covered by board and sidebars
        self.board.draw(self.canvas)
        self.draw_pieces()

        if self.selected_piece:
            col, row = self.selected_piece.pos
            rect = pygame.Rect(col * self.square_size, row * self.square_size, self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (0, 255, 0), rect, int(3 * self.scale_factor))
            valid_moves = engine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
            for move in valid_moves:
                m_col, m_row = move
                s = pygame.Surface((self.square_size, self.square_size))
                s.set_alpha(100)
                s.fill((255, 255, 0))
                self.canvas.blit(s, (m_col * self.square_size, m_row * self.square_size))

        current_king = next((p for p in self.pieces if p.type == "king" and p.color == self.turn), None)
        if current_king and engine.is_in_check(self.turn, self.pieces):
            col, row = current_king.pos
            rect = pygame.Rect(col * self.square_size, row * self.square_size, self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (153, 0, 0), rect, int(3 * self.scale_factor))
            check_text = self.font.render("Szach!", True, (153, 0, 0))
            self.canvas.blit(check_text, (int(10 * self.scale_factor), int(10 * self.scale_factor)))

        # Right sidebar (full height)
        right_sidebar_rect = pygame.Rect(self.board.board_width, 0, self.board.sidebar_width, self.total_size[1])
        pygame.draw.rect(self.canvas, self.board.dark_color, right_sidebar_rect)

        # Bottom sidebar (full width, below board)
        bottom_sidebar_height = self.total_size[1] - (self.square_size * 8)
        bottom_sidebar_rect = pygame.Rect(0, self.board.board_height, self.total_size[0], bottom_sidebar_height)
        pygame.draw.rect(self.canvas, self.board.dark_color, bottom_sidebar_rect)

        white_clock_text = self.small_font.render(f"Białe: {self.format_time(self.white_time)}", True, (255, 255, 255))
        black_clock_text = self.small_font.render(f"Czarne: {self.format_time(self.black_time)}", True, (255, 255, 255))
        turn_text = self.small_font.render("Ruch: " + self.turn.capitalize(), True, (255, 255, 255))
        mode_text = self.small_font.render(f"Tryb: {self.game_mode.capitalize()}", True, (255, 255, 255))
        moves_title = self.small_font.render("Historia ruchów", True, (255, 255, 255))

        sidebar_x = self.board.board_width + int(20 * self.scale_factor)
        self.canvas.blit(white_clock_text, (sidebar_x, int(20 * self.scale_factor)))
        self.canvas.blit(black_clock_text, (sidebar_x, int(50 * self.scale_factor)))
        self.canvas.blit(turn_text, (sidebar_x, int(80 * self.scale_factor)))
        self.canvas.blit(mode_text, (sidebar_x, int(110 * self.scale_factor)))
        self.canvas.blit(moves_title, (sidebar_x, int(140 * self.scale_factor)))

        self.BLACK = (0, 0, 0)
        self.WHITE = (255, 255, 255)
        self.HOVER_COLOR_WHITE = (220, 220, 220)
        self.HOVER_COLOR_BLACK = (50, 50, 50)
        self.SHADOW_COLOR = (150, 150, 150)

        mouse_pos = pygame.mouse.get_pos()
        surrender_hover = self.button_rect_surrender.collidepoint(mouse_pos) and not self.game_over
        draw_hover = self.button_rect_draw.collidepoint(mouse_pos) and not self.game_over
        undo_hover = self.button_rect_undo.collidepoint(mouse_pos) and not self.game_over
        notation_hover = self.button_rect_notation.collidepoint(mouse_pos) and not self.game_over

        default_color = self.WHITE if self.turn == "white" else self.BLACK
        text_color = self.BLACK if self.turn == "white" else self.WHITE
        hover_color = self.HOVER_COLOR_WHITE if self.turn == "white" else self.HOVER_COLOR_BLACK

        if surrender_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_surrender.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        surrender_color = hover_color if surrender_hover else default_color
        pygame.draw.rect(self.canvas, surrender_color, self.button_rect_surrender, border_radius=int(12 * self.scale_factor))
        surrender_text = self.small_button_font.render("Poddaj się " + self.turn.capitalize(), True, text_color)
        self.canvas.blit(surrender_text, (
            self.button_rect_surrender.x + (self.button_rect_surrender.width - surrender_text.get_width()) // 2,
            self.button_rect_surrender.y + (self.button_rect_surrender.height - surrender_text.get_height()) // 2
        ))

        if draw_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_draw.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        draw_color = hover_color if draw_hover else default_color
        pygame.draw.rect(self.canvas, draw_color, self.button_rect_draw, border_radius=int(12 * self.scale_factor))
        draw_text = self.small_button_font.render("Zaproponuj remis", True, text_color)
        self.canvas.blit(draw_text, (
            self.button_rect_draw.x + (self.button_rect_draw.width - draw_text.get_width()) // 2,
            self.button_rect_draw.y + (self.button_rect_draw.height - draw_text.get_height()) // 2
        ))

        if undo_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_undo.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        undo_color = hover_color if undo_hover else default_color
        pygame.draw.rect(self.canvas, undo_color, self.button_rect_undo, border_radius=int(12 * self.scale_factor))
        undo_text = self.small_button_font.render("Cofnij ruch", True, text_color)
        self.canvas.blit(undo_text, (
            self.button_rect_undo.x + (self.button_rect_undo.width - undo_text.get_width()) // 2,
            self.button_rect_undo.y + (self.button_rect_undo.height - undo_text.get_height()) // 2
        ))

        if notation_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_notation.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
        notation_color = hover_color if notation_hover else default_color
        pygame.draw.rect(self.canvas, notation_color, self.button_rect_notation, border_radius=int(12 * self.scale_factor))
        notation_text = self.small_button_font.render(f"Notacja: {'Krótka' if self.notation_type == 'short' else 'Długa'}", True, text_color)
        self.canvas.blit(notation_text, (
            self.button_rect_notation.x + (self.button_rect_notation.width - notation_text.get_width()) // 2,
            self.button_rect_notation.y + (self.button_rect_notation.height - notation_text.get_height()) // 2
        ))

        for i, move in enumerate(self.move_history[-8:]):
            if self.notation_type == "short":
                move_display = move
            else:
                end_pos = move.split("-")[1]
                end_col = "abcdefgh".index(end_pos[0])
                end_row = 8 - int(end_pos[1])
                piece = next((p for p in self.pieces if p.pos == (end_col, end_row)), None)
                move_display = self.format_move_long(move, piece) if piece else move
            move_text = self.small_button_font.render(move_display, True, (255, 255, 255))
            self.canvas.blit(move_text, (sidebar_x, int(170 * self.scale_factor) + i * int(25 * self.scale_factor)))

        if self.game_over:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(200)
            overlay.fill((255, 255, 255))
            self.canvas.blit(overlay, (0, 0))

            text_surface = self.font.render(self.game_over_message, True, (0, 0, 0))
            text_rect = text_surface.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - int(50 * self.scale_factor)))
            self.canvas.blit(text_surface, text_rect)

            back_hover = self.button_rect_back.collidepoint(mouse_pos)
            if back_hover:
                pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_back.move(int(3 * self.scale_factor), int(3 * self.scale_factor)), border_radius=int(12 * self.scale_factor))
            back_color = self.HOVER_COLOR_BLACK if back_hover else self.BLACK
            pygame.draw.rect(self.canvas, back_color, self.button_rect_back, border_radius=int(12 * self.scale_factor))
            back_text = self.small_button_font.render("Wyjdź", True, self.WHITE)
            self.canvas.blit(back_text, (
                self.button_rect_back.x + (self.button_rect_back.width - back_text.get_width()) // 2,
                self.button_rect_back.y + (self.button_rect_back.height - back_text.get_height()) // 2
            ))

        pygame.display.update()

    def run(self):
        while True:
            self.update_scale()  # Update scaling every frame
            result = self.process_events()
            if result == "quit":
                pygame.quit()
                sys.exit()
            elif result == "main_menu":
                return "main_menu"

            self.update()
            self.render()
            self.clock.tick(60)

# Example usage
if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((940, 640), pygame.RESIZABLE)
    game = Game(screen)
    game.run()
    pygame.quit()