import pygame
import settings
import os
import style
import requests
import json

API_BASE = "https://azbest.org.pl/"

def api_request(endpoint, method="GET", data=None, headers=None):
    url = f"{API_BASE}{endpoint}"
    try:
        response = requests.request(method, url, json=data, headers=headers)
        return response
    except requests.exceptions.RequestException:
        return None

HOME_DIR = os.path.expanduser("~")
CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

SCREEN = pygame.display.set_mode((1280, 720), pygame.RESIZABLE)
pygame.display.set_caption("RANKING - Motorla Science Cup 2025")
settings.load_settings()

# Colors
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
BLACK2 = (50, 50, 50)
GRAY = (200, 200, 200)
DARK_GRAY = (80, 80, 80)
DARK_BROWN = (132, 83, 67)
DARKER_BROWN = (85, 58, 50)
BLUE = (100, 149, 237)
GREEN = (0, 255, 0)
HOVER_COLOR = BLACK2

def draw_text(screen, text, x, y, color=BLACK, size=28, font_type="medium", hover=False, bg_color=None, center=False):
    font = style.MONTSERRAT_BOLD if font_type == "bold" else style.MONTSERRAT_MEDIUM
    font_obj = pygame.font.Font(font, int(size))
    text_surface = font_obj.render(text, True, color)
    if center:
        text_rect = text_surface.get_rect(center=(x, y))
    else:
        text_rect = text_surface.get_rect(topleft=(x, y))

    if bg_color:
        bg_rect = pygame.Rect(text_rect.x - 10, text_rect.y - 5, text_rect.width + 20, text_rect.height + 10)
        pygame.draw.rect(screen, bg_color, bg_rect, border_radius=5)

    if hover and text_rect.collidepoint(pygame.mouse.get_pos()):
        hover_rect = pygame.Rect(text_rect.x - 10, text_rect.y - 5, text_rect.width + 20, text_rect.height + 10)
        pygame.draw.rect(screen, HOVER_COLOR, hover_rect, border_radius=5)

    screen.blit(text_surface, text_rect)
    return text_rect

def draw_button(screen, text, x, y, w, h, color, hover_color):
    mouse = pygame.mouse.get_pos()
    click = pygame.mouse.get_pressed()
    rect = pygame.Rect(x, y, w, h)

    if rect.collidepoint(mouse):
        pygame.draw.rect(screen, hover_color, rect, border_radius=10)
        if click[0]:
            pygame.time.delay(80)
            return "main_menu"
    else:
        pygame.draw.rect(screen, color, rect, border_radius=10)

    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)

    font_size = 30 * (screen.get_width() / 1280)
    text_surf = pygame.font.Font(style.MONTSERRAT_BOLD, int(font_size)).render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=(x + w // 2, y + h // 2))
    screen.blit(text_surf, text_rect)
    return None

def get_top_10():
    response = api_request("/top/10")
    if response and response.status_code == 200:
        return response.json()
    return []

def draw_top_10(screen, top_players, x, y, container_width):
    # Responsive table width (80% of container width, with min/max bounds)
    table_width = max(min(container_width * 0.8, 800), 400)
    table_x = x + (container_width - table_width) // 2

    # Base font sizes
    base_header_size = 38
    base_row_size = 24
    row_spacing = 40 * (screen.get_height() / 720)

    # Calculate font size based on table width to fit content
    font_scale = table_width / 600  # Original table width was 600
    header_size = max(base_header_size * font_scale, 18)  # Minimum size 18
    row_size = max(base_row_size * font_scale, 14)       # Minimum size 14

    # Draw header
    draw_text(screen, "TOP 10 GRACZY", table_x + table_width // 2, y + 15, color=WHITE, size=header_size, font_type="bold", center=True)

    # Dynamic column widths based on table_width
    col_widths = {
        "position": int(table_width * 0.1),  # 10% for position (#)
        "nickname": int(table_width * 0.45), # 45% for nickname
        "elo": int(table_width * 0.20),      # 20% for ELO
        "date": int(table_width * 0.25)      # 25% for date
    }

    # Calculate column positions
    col1_x = table_x                            # Position
    col2_x = col1_x + col_widths["position"]    # Nickname
    col3_x = col2_x + col_widths["nickname"]    # ELO
    col4_x = col3_x + col_widths["elo"]         # Created date

    # Draw table headers
    header_y = y + int(60 * (screen.get_height() / 720))
    draw_text(screen, "#", col1_x, header_y, color=WHITE, size=row_size, font_type="bold")
    draw_text(screen, "Pseudonim", col2_x, header_y, color=WHITE, size=row_size, font_type="bold")
    draw_text(screen, "ELO", col3_x, header_y, color=WHITE, size=row_size, font_type="bold")
    draw_text(screen, "Dołączono", col4_x, header_y, color=WHITE, size=row_size, font_type="bold")

    # Draw horizontal line
    line_y = y + int(90 * (screen.get_height() / 720)) + 20
    pygame.draw.line(screen, WHITE, (table_x, line_y), (table_x + table_width, line_y), 2)

    # Draw player data
    for i, player in enumerate(top_players[:10], 1):
        y_pos = line_y + int(row_spacing * i) - 15

        # Truncate text if too long, with dynamic limits based on column width
        max_nick_chars = int(col_widths["nickname"] / (row_size * 0.6))  # Approximate chars based on font size
        max_date_chars = int(col_widths["date"] / (row_size * 0.6))
        nick = player['nick'][:max_nick_chars] + "..." if len(player['nick']) > max_nick_chars else player['nick']
        date = player['created_at'][:max_date_chars] if len(player['created_at']) > max_date_chars else player['created_at']

        draw_text(screen, str(i), col1_x, y_pos, color=WHITE, size=row_size, font_type="bold")
        draw_text(screen, nick, col2_x, y_pos, color=WHITE, size=row_size, font_type="medium")
        draw_text(screen, str(player['elo']), col3_x, y_pos, color=WHITE, size=row_size, font_type="medium")
        draw_text(screen, date, col4_x, y_pos, color=WHITE, size=row_size, font_type="medium")

def main(screen):
    clock = pygame.time.Clock()
    is_fullscreen = False
    running = True

    top_players = get_top_10()

    while running:
        width, height = screen.get_size()
        settings.draw_chessboard(screen, width, height)
        
        chess_light = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]
        chess_dark = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]

        container_width = int(width * 0.9)
        container_height = int(height * 0.80)
        container_x = (width - container_width) // 2
        container_y = (height - container_height) // 2
        pygame.draw.rect(screen, chess_dark, (container_x, container_y-25, container_width, container_height))

        draw_top_10(screen, top_players, container_x, container_y, container_width)

        events = pygame.event.get()
        for event in events:
            if event.type == pygame.QUIT:
                pygame.quit()
                exit()
            elif event.type == pygame.VIDEORESIZE and not is_fullscreen:
                new_width = max(event.w, 800)
                new_height = max(event.h, 700)
                screen = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_F11:
                is_fullscreen = not is_fullscreen
                if is_fullscreen:
                    screen = pygame.display.set_mode((width, height), pygame.FULLSCREEN)
                else:
                    screen = pygame.display.set_mode((width, height), pygame.RESIZABLE)

        button_width = int(width * 0.15)
        button_height = int(50 * (height / 720))
        button_x = (width - button_width) // 2
        button_y = height - int(90 * (height / 720))
        if draw_button(screen, "Powrót", button_x, button_y, button_width, button_height, DARK_GRAY, HOVER_COLOR) == "main_menu":
            running = False

        pygame.display.flip()
        clock.tick(60)

    return "main_menu"

if __name__ == "__main__":
    main(SCREEN)