import pygame
import style
from pygame.locals import *
import os
import sys

# Initialize Pygame
pygame.init()
pygame.mixer.init()

HOME_DIR = os.path.expanduser("~")
CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

# Reference resolution
REF_WIDTH, REF_HEIGHT = 1280, 720
# Set initial screen with minimum size consideration
WIDTH, HEIGHT = pygame.display.get_surface().get_size() if pygame.display.get_surface() else (REF_WIDTH, REF_HEIGHT)
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("Azbest Chess Settings")  # Optional: Add a title

# Colors
BLACK = (0, 0, 0)
BLACK2 = (50, 50, 50)
GRAY = (200, 200, 200)
LIGHT_GRAY = (140, 140, 140)
DARK_GRAY = (80, 80, 80)
BLUE = (100, 149, 237)
GREEN = (0, 255, 0)
HOVER_COLOR = BLACK2

PAGES = ["Ogólne", "Audio"]
current_page = "Ogólne"

CURRENT_PIECE_SET = "Classic"
CURRENT_CHESS_SET = 0
CURRENT_SOUND = "pionek2.mp3"

is_fullSCREEN = False

# Funkcja do uzyskania poprawnej ścieżki zasobów
def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        base_path = sys._MEIPASS
    except AttributeError:
        # W trybie developerskim użyj ścieżki względem lokalizacji pliku
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

# Load sound files with resource_path
SOUND_1 = pygame.mixer.Sound(resource_path("assets/audio/pionek.mp3"))
SOUND_2 = pygame.mixer.Sound(resource_path("assets/audio/pionek2.mp3"))

# Chessboard color sets
CHESS_SETS = [
    ((240, 217, 181), (181, 136, 99)),
    ((200, 200, 200), (70, 70, 70)),
    ((0, 255, 0), (100, 149, 237)),
    ((255, 182, 193), (139, 0, 139)),
    ((255, 215, 0), (255, 69, 0)),
    ((173, 216, 230), (70, 130, 180)),
    ((255, 250, 240), (105, 105, 105))
]

background_color = style.WHITE

# Scaling function
def scale(value, axis="both"):
    scale_x = WIDTH / REF_WIDTH
    scale_y = HEIGHT / REF_HEIGHT
    if axis == "x":
        return int(value * scale_x)
    elif axis == "y":
        return int(value * scale_y)
    return int(value * min(scale_x, scale_y))

# Font scaling
def get_scaled_font(size):
    return pygame.font.Font(resource_path(style.MONTSERRAT_BOLD), scale(size))  # Poprawiona ścieżka do fontu

NAME_FONT = get_scaled_font(30)
font = get_scaled_font(30)

def draw_chessboard(screen, width, height):
    global CHESS_SETS, CURRENT_CHESS_SET
    color1, color2 = CHESS_SETS[CURRENT_CHESS_SET]
    square_size = 50
    for y in range(0, height + square_size, square_size):
        for x in range(0, width + square_size, square_size):
            color = color1 if (x // square_size + y // square_size) % 2 == 0 else color2
            pygame.draw.rect(screen, color, (x, y, square_size, square_size))

def draw_text(screen, text, x, y, color=BLACK, size=28, hover=False, bg_color=None, center=False):
    font_obj = get_scaled_font(size)
    text_surface = font_obj.render(text, True, color)
    if center:
        text_rect = text_surface.get_rect(center=(scale(x, "x"), scale(y, "y")))
    else:
        text_rect = text_surface.get_rect(topleft=(scale(x, "x"), scale(y, "y")))

    if bg_color:
        bg_rect = pygame.Rect(text_rect.x - scale(10), text_rect.y - scale(5), text_rect.width + scale(20),
                              text_rect.height + scale(10))
        pygame.draw.rect(screen, bg_color, bg_rect, border_radius=scale(5))

    if hover and text_rect.collidepoint(pygame.mouse.get_pos()):
        hover_rect = pygame.Rect(text_rect.x - scale(10), text_rect.y - scale(5), text_rect.width + scale(20),
                                 text_rect.height + scale(10))
        pygame.draw.rect(screen, HOVER_COLOR, hover_rect, border_radius=scale(5))

    screen.blit(text_surface, text_rect)

def draw_button(screen, text, x, y, w, h, color, hover_color):
    mouse = pygame.mouse.get_pos()
    click = pygame.mouse.get_pressed()
    rect = pygame.Rect(scale(x, "x"), scale(y, "y"), scale(w, "x"), scale(h, "y"))

    if rect.collidepoint(mouse):
        pygame.draw.rect(screen, hover_color, rect, border_radius=scale(10))
        if click[0]:
            pygame.time.delay(80)
            return "main_menu"
    else:
        pygame.draw.rect(screen, color, rect, border_radius=scale(10))

    pygame.draw.rect(screen, style.WHITE, rect, scale(2), border_radius=scale(10))

    text_surf = NAME_FONT.render(text, True, style.WHITE)
    text_rect = text_surf.get_rect(center=(scale(x, "x") + scale(w, "x") // 2, scale(y, "y") + scale(h, "y") // 2))
    screen.blit(text_surf, text_rect)
    return None

def switch_page(page):
    global current_page
    current_page = page
    handle_widgets(page)

def create_nav_buttons():
    buttons = []
    for i, page_name in enumerate(PAGES):
        button_rect = pygame.Rect(scale(40 + i * 180, "x"), scale(70, "y"), scale(170, "x"), scale(50, "y"))
        buttons.append((page_name, button_rect))
    return buttons

def draw_chess_buttons(screen):
    global CHESS_SETS
    button_size = scale(60)
    start_x, start_y = scale(100, "x"), scale(200, "y")

    for i, (color1, color2) in enumerate(CHESS_SETS):
        button_rect = pygame.Rect(start_x + i * (button_size + scale(10)), start_y, button_size, button_size)

        pygame.draw.rect(screen, color1, (button_rect.x, button_rect.y, button_size // 2, button_size // 2))
        pygame.draw.rect(screen, color2,
                         (button_rect.x + button_size // 2, button_rect.y, button_size // 2, button_size // 2))
        pygame.draw.rect(screen, color2,
                         (button_rect.x, button_rect.y + button_size // 2, button_size // 2, button_size // 2))
        pygame.draw.rect(screen, color1, (
        button_rect.x + button_size // 2, button_rect.y + button_size // 2, button_size // 2, button_size // 2))
        pygame.draw.rect(screen, style.WHITE, button_rect, scale(2))

        if button_rect.collidepoint(pygame.mouse.get_pos()):
            hover_surface = pygame.Surface((button_size, button_size), pygame.SRCALPHA)
            pygame.draw.rect(hover_surface, (255, 255, 255, 128), (0, 0, button_size, button_size))
            screen.blit(hover_surface, (button_rect.x, button_rect.y))

        if pygame.mouse.get_pressed()[0] and button_rect.collidepoint(pygame.mouse.get_pos()):
            pygame.time.delay(150)
            global CURRENT_CHESS_SET
            CURRENT_CHESS_SET = i
            save_settings()

def draw_piece_buttons(screen):
    global CURRENT_PIECE_SET
    button_size = scale(60)
    start_x, start_y = scale(730, "x"), scale(200, "y")
    styles = ["Classic", "Modern", "Fantasy"]
    icons = {
        "Classic": resource_path("assets/pieces/classic/hetmanB.png"),
        "Modern": resource_path("assets/pieces/modern/hetmanB.png"),
        "Fantasy": resource_path("assets/pieces/fantasy/hetmanB.png")
    }

    for i, style_name in enumerate(styles):
        button_rect = pygame.Rect(start_x + i * (button_size + scale(20)), start_y, button_size, button_size)
        pygame.draw.rect(screen, GRAY if CURRENT_PIECE_SET == style_name else DARK_GRAY, button_rect,
                         border_radius=scale(5))
        piece_image = pygame.image.load(icons[style_name])
        piece_image = pygame.transform.smoothscale(piece_image, (button_size - scale(10), button_size - scale(10)))
        screen.blit(piece_image, (button_rect.x + scale(5), button_rect.y + scale(5)))

        if button_rect.collidepoint(pygame.mouse.get_pos()):
            hover_surface = pygame.Surface((button_size, button_size), pygame.SRCALPHA)
            pygame.draw.rect(hover_surface, (255, 255, 255, 26), (0, 0, button_size, button_size),
                             border_radius=scale(5))
            screen.blit(hover_surface, (button_rect.x, button_rect.y))

            if pygame.mouse.get_pressed()[0]:
                pygame.time.delay(150)
                CURRENT_PIECE_SET = style_name
                save_settings()

def ogolne_page():
    draw_text(SCREEN, "Wybierz kolor szachownicy", 95, 150, color=style.WHITE)
    draw_chess_buttons(SCREEN)
    draw_text(SCREEN, "Wybierz styl figur", 730, 150, color=style.WHITE)
    draw_piece_buttons(SCREEN)

# Custom Slider Class
class CustomSlider:
    def __init__(self, screen, x, y, width, height, min_val, max_val, initial_val):
        self.screen = screen
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.min = min_val
        self.max = max_val
        self.value = initial_val
        self.handle_width = scale(20)
        self.dragging = False
        self.rect = pygame.Rect(self.x, self.y-23, self.width, self.height)
        self.handle_x = self.x + int((self.value - self.min) / (self.max - self.min) * (self.width - self.handle_width))

    def draw(self):
        pygame.draw.rect(self.screen, GRAY, (self.x, self.y-23, self.width, self.height))
        pygame.draw.rect(self.screen, LIGHT_GRAY, (self.handle_x, self.y-23, self.handle_width, self.height))
        pygame.draw.rect(self.screen, DARK_GRAY, (self.handle_x, self.y-23, self.handle_width, self.height), 1)

    def update(self, events):
        mouse_pos = pygame.mouse.get_pos()
        for event in events:
            if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                if self.rect.collidepoint(mouse_pos):
                    self.dragging = True
            elif event.type == pygame.MOUSEBUTTONUP and event.button == 1:
                self.dragging = False
            elif event.type == pygame.MOUSEMOTION and self.dragging:
                new_x = min(max(mouse_pos[0] - self.handle_width // 2, self.x), self.x + self.width - self.handle_width)
                self.handle_x = new_x
                self.value = self.min + (self.handle_x - self.x) / (self.width - self.handle_width) * (
                            self.max - self.min)

    def get_value(self):
        return int(self.value)

    def set_value(self, value):
        self.value = max(self.min, min(self.max, value))
        self.handle_x = self.x + int((self.value - self.min) / (self.max - self.min) * (self.width - self.handle_width))

    def hide(self):
        self.rect = pygame.Rect(0, 0, 0, 0)

    def show(self):
        self.rect = pygame.Rect(self.x, self.y-23, self.width, self.height)

    def update_position(self):
        self.rect = pygame.Rect(self.x, self.y-23, self.width, self.height)
        self.handle_x = self.x + int((self.value - self.min) / (self.max - self.min) * (self.width - self.handle_width))

# Audio Page: Sliders and Sound Selection
audio_sliders = [
    CustomSlider(SCREEN, scale(REF_WIDTH // 2 - 100, "x"), scale(200, "y"), scale(200, "x"), scale(20, "y"), 0, 100,
                 50),
    CustomSlider(SCREEN, scale(REF_WIDTH // 2 - 100, "x"), scale(280, "y"), scale(200, "x"), scale(20, "y"), 0, 100, 50)
]
for slider in audio_sliders:
    slider.hide()

def draw_sound_buttons(screen):
    global CURRENT_SOUND
    button_size = scale(40)
    start_x, start_y = scale(REF_WIDTH // 2 - 100, "x"), scale(400, "y")
    sounds = ["pionek.mp3", "pionek2.mp3"]

    mouse_pos = pygame.mouse.get_pos()

    for i, sound_name in enumerate(sounds):
        button_rect = pygame.Rect(start_x + i * (button_size + scale(10)), start_y, button_size, button_size)
        color = GRAY if CURRENT_SOUND == sound_name else (
            LIGHT_GRAY if button_rect.collidepoint(mouse_pos) else DARK_GRAY)
        pygame.draw.rect(screen, color, button_rect, border_radius=scale(5))
        pygame.draw.rect(screen, style.WHITE, button_rect, scale(2))
        text_surf = NAME_FONT.render(str(i + 1), True, BLACK)
        text_rect = text_surf.get_rect(center=button_rect.center)
        screen.blit(text_surf, text_rect)

def audio_page():
    draw_text(SCREEN, "Efekty dźwiękowe", REF_WIDTH // 2 - 370, 170, color=style.WHITE, size=25)
    draw_text(SCREEN, "Muzyka", REF_WIDTH // 2 - 231, 250, color=style.WHITE, size=25)
    for slider in audio_sliders:
        slider.draw()
    draw_text(SCREEN, "Wybierz dźwięk ruchu", REF_WIDTH // 2 - 330, 350, color=style.WHITE)
    draw_sound_buttons(SCREEN)

page_functions = {
    "Ogólne": ogolne_page,
    "Audio": audio_page
}

def handle_widgets(page):
    global NAME_FONT, font
    NAME_FONT = get_scaled_font(30)
    font = get_scaled_font(30)
    if page == "Ogólne":
        for slider in audio_sliders:
            slider.hide()
    elif page == "Audio":
        for i, slider in enumerate(audio_sliders):
            slider.show()
            slider.x = scale(REF_WIDTH // 2 - 100, "x")
            slider.y = scale(200 if i == 0 else 280, "y")
            slider.width = scale(200, "x")
            slider.height = scale(20, "y")
            slider.handle_width = scale(20)
            slider.update_position()

def main():
    global WIDTH, HEIGHT, SCREEN, is_fullSCREEN, CURRENT_SOUND
    WIDTH, HEIGHT = pygame.display.get_surface().get_size() if pygame.display.get_surface() else (REF_WIDTH, REF_HEIGHT)
    SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)

    load_settings()
    buttons = create_nav_buttons()
    clock = pygame.time.Clock()
    switch_page("Ogólne")
    while True:
        draw_chessboard(SCREEN, WIDTH, HEIGHT)

        chess_dark = CHESS_SETS[CURRENT_CHESS_SET][1]

        container_width = scale(REF_WIDTH - 80, "x")
        container_height = scale(REF_HEIGHT // 2 + 180, "y")
        container_x = scale((REF_WIDTH - (REF_WIDTH - 80)) // 2, "x")
        container_y = scale(REF_HEIGHT // 2 - 300, "y")
        pygame.draw.rect(SCREEN, chess_dark, (container_x, container_y, container_width, container_height))

        events = pygame.event.get()
        for event in events:
            if event.type == pygame.QUIT:
                pygame.quit()
                exit()
            elif event.type == pygame.VIDEORESIZE and not is_fullSCREEN:
                new_width = max(event.w, 800)  # Enforce minimum width
                new_height = max(event.h, 700)  # Enforce minimum height
                WIDTH, HEIGHT = new_width, new_height
                SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
                buttons = create_nav_buttons()
                handle_widgets(current_page)
            elif event.type == pygame.KEYDOWN and event.key == pygame.K_F11:
                toggle_fullSCREEN()
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                for page_name, button_rect in buttons:
                    if button_rect.collidepoint(event.pos):
                        switch_page(page_name)
                if current_page == "Audio":
                    mouse_pos = event.pos
                    start_x, start_y = scale(REF_WIDTH // 2 - 100, "x"), scale(400, "y")
                    button_size = scale(40)
                    sound_rect_1 = pygame.Rect(start_x, start_y, button_size, button_size)
                    sound_rect_2 = pygame.Rect(start_x + button_size + scale(10), start_y, button_size, button_size)
                    if sound_rect_1.collidepoint(mouse_pos):
                        pygame.time.delay(150)
                        CURRENT_SOUND = "pionek.mp3"
                        SOUND_1.play()
                        save_settings()
                    elif sound_rect_2.collidepoint(mouse_pos):
                        pygame.time.delay(150)
                        CURRENT_SOUND = "pionek2.mp3"
                        SOUND_2.play()
                        save_settings()

        for page_name, button_rect in buttons:
            if current_page == page_name:
                active_color = tuple(max(0, c - 20) for c in chess_dark)
                pygame.draw.rect(SCREEN, active_color, button_rect)
            else:
                pygame.draw.rect(SCREEN, chess_dark, button_rect)
            text_surf = font.render(page_name, True, style.WHITE)
            text_rect = text_surf.get_rect(center=button_rect.center)
            SCREEN.blit(text_surf, text_rect)

        if draw_button(SCREEN, "Powrót", REF_WIDTH // 2 - 100, REF_HEIGHT - 90, 200, 50, DARK_GRAY,
                       HOVER_COLOR) == "main_menu":
            save_settings()
            return "main_menu"

        check_music_slider()
        check_sound_effect_slider()
        if current_page == "Audio":
            for slider in audio_sliders:
                slider.update(events)
        page_functions[current_page]()
        pygame.display.flip()
        clock.tick(60)

sound_effect_slider = audio_sliders[0]
menu_music_slider = audio_sliders[1]
prev_value = None
sfprev_value = None

def check_music_slider():
    global prev_value
    current_value = menu_music_slider.get_value()
    if current_value != prev_value:
        volume = current_value / 100
        pygame.mixer.music.set_volume(volume)
        prev_value = current_value

def check_sound_effect_slider():
    global sfprev_value
    sfcurrent_value = sound_effect_slider.get_value()
    if sfcurrent_value != sfprev_value:
        sfvolume = sfcurrent_value / 100
        SOUND_1.set_volume(sfvolume)
        SOUND_2.set_volume(sfvolume)
        sfprev_value = sfcurrent_value

def save_settings():
    lines = []
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()

    while len(lines) < 7:
        lines.append("\n")

    lines[2] = str(CURRENT_CHESS_SET) + "\n"
    lines[3] = CURRENT_PIECE_SET + "\n"
    lines[4] = str(menu_music_slider.get_value()) + "\n"
    lines[5] = str(sound_effect_slider.get_value()) + "\n"
    lines[6] = CURRENT_SOUND + "\n"

    with open(CONFIG_FILE, "w", encoding="utf-8") as file:
        file.writelines(lines)

def load_settings():
    global CURRENT_CHESS_SET, CURRENT_PIECE_SET, CURRENT_SOUND, prev_value, sfprev_value
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()

        if len(lines) >= 3:
            try:
                CURRENT_CHESS_SET = int(lines[2].strip())
            except ValueError:
                CURRENT_CHESS_SET = 0

        if len(lines) >= 4:
            CURRENT_PIECE_SET = lines[3].strip()

        if len(lines) >= 5:
            try:
                volume = float(lines[4].strip()) / 100
                pygame.mixer.music.set_volume(volume)
                menu_music_slider.set_value(volume * 100)
                prev_value = menu_music_slider.get_value()
            except ValueError:
                menu_music_slider.set_value(50)
                pygame.mixer.music.set_volume(0.5)
                prev_value = 50

        if len(lines) >= 6:
            try:
                sfvolume = float(lines[5].strip()) / 100
                sound_effect_slider.set_value(sfvolume * 100)
                SOUND_1.set_volume(sfvolume)
                SOUND_2.set_volume(sfvolume)
                sfprev_value = sound_effect_slider.get_value()
            except ValueError:
                sound_effect_slider.set_value(50)
                SOUND_1.set_volume(0.5)
                SOUND_2.set_volume(0.5)
                sfprev_value = 50

        if len(lines) >= 7:
            CURRENT_SOUND = lines[6].strip()
            if CURRENT_SOUND not in ["pionek.mp3", "pionek2.mp3"]:
                CURRENT_SOUND = "pionek2.mp3"
    else:
        SOUND_1.set_volume(0.5)
        SOUND_2.set_volume(0.5)
        save_settings()

def toggle_fullSCREEN():
    global is_fullSCREEN, SCREEN, WIDTH, HEIGHT
    is_fullSCREEN = not is_fullSCREEN
    if is_fullSCREEN:
        SCREEN = pygame.display.set_mode((0, 0), pygame.FULLSCREEN)
    else:
        SCREEN = pygame.display.set_mode((REF_WIDTH, REF_HEIGHT), pygame.RESIZABLE)
    WIDTH, HEIGHT = SCREEN.get_size()
    handle_widgets(current_page)

if __name__ == "__main__":
    main()