package main

import (
	"database/sql"
	"encoding/json"
	"log"
)

func calculateEloChange(playerRating, opponentRating int, outcome string) int {
	ratingDiff := playerRating - opponentRating // Positive if player is higher-rated

	switch outcome {
	case "win":
		if ratingDiff < 0 { // Underdog wins
			return 25
		} else if ratingDiff > 200 { // Much higher-rated wins
			return 15
		}
		return 20
	case "loss":
		if ratingDiff > 0 { // Favorite loses
			return -25
		} else if ratingDiff < -200 { // Much lower-rated loses
			return -15
		}
		return -20
	case "draw":
		if ratingDiff > 100 { // Higher-rated draws
			return -5
		} else if ratingDiff < -100 { // Lower-rated draws
			return 5
		}
		return 0
	default:
		return 0 // No change for invalid outcomes
	}
}

// updateElo updates player ratings and logs the changes as a separate event
func updateElo(db *sql.DB, room *Room, reason string) {
	if room.EloUpdated { // Prevent multiple updates
		return
	}
	room.EloUpdated = true

	// Fetch current ratings
	var whiteRating, blackRating int
	err := db.QueryRow("SELECT elo FROM users WHERE id = ?", room.WhitePlayer.UserID).Scan(&whiteRating)
	if err != nil {
		log.Printf("Failed to fetch white player rating: %v", err)
		return
	}
	err = db.QueryRow("SELECT elo FROM users WHERE id = ?", room.BlackPlayer.UserID).Scan(&blackRating)
	if err != nil {
		log.Printf("Failed to fetch black player rating: %v", err)
		return
	}

	// Determine outcome for each player
	var whiteOutcome, blackOutcome string
	switch reason {
	case "checkmate":
		if room.WinnerID == room.WhitePlayer.UserID {
			whiteOutcome = "win"
			blackOutcome = "loss"
		} else if room.WinnerID == room.BlackPlayer.UserID {
			whiteOutcome = "loss"
			blackOutcome = "win"
		} else {
			log.Printf("No winner set for checkmate in room UID %d", room.UID)
			return
		}
	case "draw", "stalemate", "50-move-rule", "insufficient-material":
		whiteOutcome = "draw"
		blackOutcome = "draw"
	case "abandoned":
		if room.WinnerID == room.WhitePlayer.UserID {
			whiteOutcome = "win"
			blackOutcome = "loss"
		} else if room.WinnerID == room.BlackPlayer.UserID {
			whiteOutcome = "loss"
			blackOutcome = "win"
		} else {
			log.Printf("No winner set for abandoned in room UID %d", room.UID)
			return
		}
	default:
		log.Printf("Unknown reason '%s' for Elo update in room UID %d", reason, room.UID)
		return
	}

	// Calculate Elo changes
	whiteEloChange := calculateEloChange(whiteRating, blackRating, whiteOutcome)
	blackEloChange := calculateEloChange(blackRating, whiteRating, blackOutcome)

	// Update ratings in the users table
	_, err = db.Exec("UPDATE users SET elo = elo + ? WHERE id = ?", whiteEloChange, room.WhitePlayer.UserID)
	if err != nil {
		log.Printf("Failed to update white player Elo: %v", err)
		return
	}
	_, err = db.Exec("UPDATE users SET elo = elo + ? WHERE id = ?", blackEloChange, room.BlackPlayer.UserID)
	if err != nil {
		log.Printf("Failed to update black player Elo: %v", err)
		return
	}

	// Log Elo changes in game_events as a separate event
	timeRemaining := int(room.GameState.WhiteTimeRemaining.Milliseconds())
	payload := struct {
		Reason         string `json:"reason"`
		WhiteEloChange int    `json:"white_elo_change"`
		BlackEloChange int    `json:"black_elo_change"`
	}{
		Reason:         reason,
		WhiteEloChange: whiteEloChange,
		BlackEloChange: blackEloChange,
	}
	payloadBytes, _ := json.Marshal(payload)
	_, err = db.Exec(`
		INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
		VALUES (?, 'elo_update', ?, ?, ?)`,
		room.UID, room.WhitePlayer.UserID, string(payloadBytes), timeRemaining)
	if err != nil {
		log.Printf("Failed to log elo_update for room UID %d: %v", room.UID, err)
	}
}
