package main

import (
	"context"
	"database/sql"
	"encoding/json"
	"fmt"
	"log"
	"net/http"
	"time"

	"github.com/coder/websocket"
)

func sendWSMessage(conn *websocket.Conn, msgType string, payload any) error {
	if conn == nil {
		return fmt.Errorf("connection is nil")
	}
	msg := struct {
		Type    string `json:"type"`
		Payload any    `json:"payload"`
	}{
		Type:    msgType,
		Payload: payload,
	}
	msgBytes, err := json.Marshal(msg)
	if err != nil {
		return fmt.Errorf("error marshaling message: %w", err)
	}
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()
	if err := conn.Write(ctx, websocket.MessageText, msgBytes); err != nil {
		return fmt.Errorf("error sending message: %w", err)
	}
	fmt.Printf("sent %s message  \"%v\"\n", msgType, payload)
	return nil
}

func notifyPlayers(room *Room, msgType string, payload any) {
	if room.WhitePlayer.Conn != nil {
		sendWSMessage(room.WhitePlayer.Conn, msgType, payload)
	}
	if room.BlackPlayer.Conn != nil {
		sendWSMessage(room.BlackPlayer.Conn, msgType, payload)
	}
}

func gameHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	userID := getUserIDFromContext(r)
	conn, ok := r.Context().Value(connKey).(**websocket.Conn)
	if !ok {
		return
	}
	var role string
	role, ok = r.Context().Value(roleKey).(string)
	if !ok {
		return
	}
	var room *Room
	room, ok = r.Context().Value(roomKey).(*Room)
	if !ok {
		return
	}
	var err error
	*conn, err = websocket.Accept(w, r, nil)
	if err != nil {
		return
	}

	var gameEndReason string // Track the reason for game end

	defer func() {
		roomsMu.Lock()
		room.mu.Lock()
		if gameEndReason == "" { // Default to surrender if no reason set
			room.makeWinner(room.GetOpponent(userID).UserID)
			gameEndReason = "abandoned"
			sendWSMessage(room.GetOpponent(userID).Conn, EventTypeGameOver, EventGameOver{Reason: "surrender"})
		}
		updateElo(db, room, gameEndReason) // Update Elo once at the end
		room.close(db)
		room.mu.Unlock()
		roomsMu.Unlock()
	}()

	log.Printf("Established websocket connection | IP: %s | UserID: %d", GetIP(r), userID)
	sendWSMessage(*conn, EventTypeColorAssignment, EventColorAssignment{PlayerColor: role})
	if room.arePlayersConnected() {
		notifyPlayers(room, EventTypeStart, nil)
		room.startTimer()
		notifyPlayers(room, EventTypeTimeUpdate, EventTimeUpdate{
			WhiteTime: room.GameState.WhiteTimeRemaining.Seconds(),
			BlackTime: room.GameState.BlackTimeRemaining.Seconds(),
		})
		room.GameState.LastTimeUpdateSent = time.Now()
	}
	var msg WSMessage
	var msgBytes []byte
	var timeRemaining int

	for {
		_, msgBytes, err = (*conn).Read(r.Context())
		if err != nil {
			return
		}
		if string(msgBytes) == "ping" {
			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
			defer cancel()
			if err := (*conn).Write(ctx, websocket.MessageText, []byte("pong")); err != nil {
				log.Printf("Error sending pong: %v", err)
				return
			}
			continue
		}
		err = json.Unmarshal(msgBytes, &msg)
		if err != nil {
			return
		}
		fmt.Println(msg)
		room.mu.Lock()
		switch msg.Type {
		case EventTypeMove:
			var moveEvent EventMove
			if err := json.Unmarshal(msgBytes, &struct {
				Payload *EventMove `json:"payload"`
			}{
				Payload: &moveEvent,
			}); err != nil {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Invalid move format",
				})
				continue
			}

			if (role == "white" && room.GameState.Turn != "white") ||
				(role == "black" && room.GameState.Turn != "black") {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Not your turn",
				})
				continue
			}

			if err := room.makeMove(moveEvent.Move); err != nil {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: err.Error(),
				})
				continue
			}

			playerID := room.WhitePlayer.UserID
			if role == "black" {
				playerID = room.BlackPlayer.UserID
			}
			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			_, err = db.Exec(`
				INSERT INTO moves (room_id, move_number, player_id, move_notation, time_remaining, is_check, is_checkmate, is_capture)
				SELECT ?, (SELECT COALESCE(MAX(move_number), 0) + 1 FROM moves WHERE room_id = ?), ?, ?, ?, ?, ?, ?`,
				room.UID, room.UID, playerID, moveEvent.Move, timeRemaining, room.GameState.IsCheck, room.GameState.IsCheckmate, false)
			if err != nil {
				log.Printf("Failed to log move for room UID %d: %v", room.UID, err)
			}

			sendWSMessage(room.GetOpponent(userID).Conn, EventTypeMove, moveEvent)

			if room.GameState.IsCheckmate || room.GameState.IsDraw {
				if room.GameState.IsCheckmate {
					gameEndReason = "checkmate"
					if role == "white" {
						room.makeWinner(room.WhitePlayer.UserID)
					} else {
						room.makeWinner(room.BlackPlayer.UserID)
					}
				} else if room.GameState.IsStalemate {
					gameEndReason = "stalemate"
				} else if room.GameState.HalfMoveClock >= 50 {
					gameEndReason = "50-move-rule"
				} else if room.GameState.isInsufficientMaterial() {
					gameEndReason = "insufficient-material"
				} else {
					gameEndReason = "draw"
				}

				// Log game over event
				payload, _ := json.Marshal(EventGameOver{Reason: gameEndReason})
				_, err = db.Exec(`
					INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
					VALUES (?, 'game_over', ?, ?, ?)`,
					room.UID, userID, string(payload), timeRemaining)
				if err != nil {
					log.Printf("Failed to log game over for room UID %d: %v", room.UID, err)
				}

				_, err = db.Exec(`
					UPDATE rooms
					SET white_time_remaining = ?, black_time_remaining = ?, status = 'finished', result = ?, ended_at = CURRENT_TIMESTAMP
					WHERE id = ?`,
					room.GameState.WhiteTimeRemaining.Milliseconds(),
					room.GameState.BlackTimeRemaining.Milliseconds(),
					gameEndReason, room.UID)
				if err != nil {
					log.Printf("Failed to update room times for UID %d: %v", room.UID, err)
				}

				room.mu.Unlock()
				notifyPlayers(room, EventTypeGameOver, EventGameOver{Reason: gameEndReason})
				return
			}
			room.mu.Unlock()

		case EventTypeSurrender:
			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			_, err = db.Exec(`
				INSERT INTO game_events (room_id, event_type, player_id, time_remaining)
				VALUES (?, 'surrender', ?, ?)`,
				room.UID, userID, timeRemaining)
			if err != nil {
				log.Printf("Failed to log surrender for room UID %d: %v", room.UID, err)
			}

			_, err = db.Exec(`
				UPDATE rooms
				SET white_time_remaining = ?, black_time_remaining = ?, status = 'finished', result = 'abandoned', ended_at = CURRENT_TIMESTAMP
				WHERE id = ?`,
				room.GameState.WhiteTimeRemaining.Milliseconds(),
				room.GameState.BlackTimeRemaining.Milliseconds(),
				room.UID)
			if err != nil {
				log.Printf("Failed to update room for surrender UID %d: %v", room.UID, err)
			}

			gameEndReason = "abandoned"
			room.makeWinner(room.GetOpponent(userID).UserID)
			room.mu.Unlock()
			return

		case EventTypeDrawRequest:
			if (role == "white" && room.GameState.Turn != "white") ||
				(role == "black" && room.GameState.Turn != "black") {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Cannot request draw during opponent's turn",
				})
				continue
			}

			if room.DrawRequestedBy != "" {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "There's already a pending draw request",
				})
				continue
			}

			room.DrawRequestedBy = role
			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			_, err = db.Exec(`
				INSERT INTO game_events (room_id, event_type, player_id, time_remaining)
				VALUES (?, 'draw_request', ?, ?)`,
				room.UID, userID, timeRemaining)
			if err != nil {
				log.Printf("Failed to log draw request for room UID %d: %v", room.UID, err)
			}

			sendWSMessage(room.GetOpponent(userID).Conn, EventTypeDrawRequest, nil)
			room.mu.Unlock()

		case EventTypeDrawResponse:
			if room.DrawRequestedBy == role {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "You cannot respond to your own draw request",
				})
				continue
			}

			var drawResponse EventDrawRequest
			if err := json.Unmarshal(msgBytes, &struct {
				Payload *EventDrawRequest `json:"payload"`
			}{
				Payload: &drawResponse,
			}); err != nil {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Invalid draw response format",
				})
				continue
			}

			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			payload, _ := json.Marshal(drawResponse)
			_, err = db.Exec(`
				INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
				VALUES (?, 'draw_response', ?, ?, ?)`,
				room.UID, userID, string(payload), timeRemaining)
			if err != nil {
				log.Printf("Failed to log draw response for room UID %d: %v", room.UID, err)
			}

			room.DrawRequestedBy = ""
			sendWSMessage(room.GetOpponent(userID).Conn, EventTypeDrawResponse, drawResponse)
			if drawResponse.Accept {
				room.GameState.IsDraw = true
				gameEndReason = "draw"

				// Log game over event
				payload, _ := json.Marshal(EventGameOver{Reason: "draw"})
				_, err = db.Exec(`
					INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
					VALUES (?, 'game_over', ?, ?, ?)`,
					room.UID, userID, string(payload), timeRemaining)
				if err != nil {
					log.Printf("Failed to log game over for room UID %d: %v", room.UID, err)
				}

				_, err = db.Exec(`
					UPDATE rooms
					SET white_time_remaining = ?, black_time_remaining = ?, status = 'finished', result = 'draw', ended_at = CURRENT_TIMESTAMP
					WHERE id = ?`,
					room.GameState.WhiteTimeRemaining.Milliseconds(),
					room.GameState.BlackTimeRemaining.Milliseconds(),
					room.UID)
				if err != nil {
					log.Printf("Failed to update room for draw UID %d: %v", room.UID, err)
				}

				room.mu.Unlock()
				return
			}
			room.mu.Unlock()

		case EventTypeUndoRequest:
			if (role == "white" && room.GameState.Turn == "white") ||
				(role == "black" && room.GameState.Turn == "black") {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Cannot request undo during your own turn",
				})
				continue
			}

			if len(room.GameState.MoveHistory) == 0 {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "No moves to undo",
				})
				continue
			}

			if room.UndoRequestedBy != "" {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Undo request already in progress",
				})
				continue
			}

			lastMoveColor := "white"
			if len(room.GameState.MoveHistory)%2 == 0 {
				lastMoveColor = "black"
			}
			if (role == "white" && lastMoveColor != "white") ||
				(role == "black" && lastMoveColor != "black") {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "You can only undo your own last move",
				})
				continue
			}
			room.UndoRequestedBy = role

			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			_, err = db.Exec(`
				INSERT INTO game_events (room_id, event_type, player_id, time_remaining)
				VALUES (?, 'undo_request', ?, ?)`,
				room.UID, userID, timeRemaining)
			if err != nil {
				log.Printf("Failed to log undo request for room UID %d: %v", room.UID, err)
			}

			sendWSMessage(room.GetOpponent(userID).Conn, EventTypeUndoRequest, nil)
			room.mu.Unlock()

		case EventTypeUndoResponse:
			if room.UndoRequestedBy == "" {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "No undo request",
				})
				continue
			}
			var undoResponse EventUndoRequest
			if err := json.Unmarshal(msgBytes, &struct {
				Payload *EventUndoRequest `json:"payload"`
			}{
				Payload: &undoResponse,
			}); err != nil {
				room.mu.Unlock()
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Invalid undo response format",
				})
				continue
			}

			if (room.UndoRequestedBy == "white" && room.GameState.Turn == "black") ||
				(room.UndoRequestedBy == "black" && room.GameState.Turn == "white") {
				room.UndoRequestedBy = ""

				timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
				if role == "black" {
					timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
				}
				payload, _ := json.Marshal(undoResponse)
				_, err := db.Exec(`
					INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
					VALUES (?, 'undo_response', ?, ?, ?)`,
					room.UID, userID, string(payload), timeRemaining)
				if err != nil {
					log.Printf("Failed to log undo response for room UID %d: %v", room.UID, err)
				}

				if undoResponse.Accept {
					if err := room.undoLastMove(); err != nil {
						room.mu.Unlock()
						sendWSMessage(*conn, EventTypeError, EventError{
							Message: err.Error(),
						})
						continue
					}
					notifyPlayers(room, EventTypeUndoResponse, undoResponse)
				} else {
					sendWSMessage(room.GetOpponent(userID).Conn, EventTypeUndoResponse, undoResponse)
				}
			} else {
				sendWSMessage(*conn, EventTypeError, EventError{
					Message: "Invalid undo response",
				})
			}
			room.mu.Unlock()

		default:
			timeRemaining = int(room.GameState.WhiteTimeRemaining.Milliseconds())
			if role == "black" {
				timeRemaining = int(room.GameState.BlackTimeRemaining.Milliseconds())
			}
			_, err = db.Exec(`
				INSERT INTO game_events (room_id, event_type, player_id, payload, time_remaining)
				VALUES (?, 'unknown_message', ?, ?, ?)`,
				room.UID, userID, string(msgBytes), timeRemaining)
			if err != nil {
				log.Printf("Failed to log unknown message for room UID %d: %v", room.UID, err)
			}

			sendWSMessage(*conn, EventTypeError, EventError{
				Message: "Unknown message type",
			})
			room.mu.Unlock()
		}
	}
}
