package main

import (
	"errors"
	"fmt"
	"time"
	"unicode"
)

const (
	Empty  = '-'
	Pawn   = 'p'
	Rook   = 'r'
	Knight = 'n'
	Bishop = 'b'
	Queen  = 'q'
	King   = 'k'
)

type GameState struct {
	Board       [8][8]rune
	Turn        string // "white" or "black"
	WhiteCastle struct {
		Kingside  bool
		Queenside bool
	}
	BlackCastle struct {
		Kingside  bool
		Queenside bool
	}
	EnPassantSquare    string // "-" or square like "e3"
	HalfMoveClock      int    // For 50-move rule
	FullMoveNumber     int    // Increments after black's move
	IsCheck            bool
	IsCheckmate        bool
	IsStalemate        bool
	IsDraw             bool
	WhiteTimeRemaining time.Duration
	BlackTimeRemaining time.Duration
	Timer              *time.Timer
	LastMoveTime       time.Time
	LastTimeUpdateSent time.Time
	MoveHistory        MoveHistory
}

type Move struct {
	From      string
	To        string
	Promotion rune // For pawn promotion
}

func NewGameState() GameState {
	board := [8][8]rune{
		{'r', 'n', 'b', 'q', 'k', 'b', 'n', 'r'},
		{'p', 'p', 'p', 'p', 'p', 'p', 'p', 'p'},
		{'-', '-', '-', '-', '-', '-', '-', '-'},
		{'-', '-', '-', '-', '-', '-', '-', '-'},
		{'-', '-', '-', '-', '-', '-', '-', '-'},
		{'-', '-', '-', '-', '-', '-', '-', '-'},
		{'P', 'P', 'P', 'P', 'P', 'P', 'P', 'P'},
		{'R', 'N', 'B', 'Q', 'K', 'B', 'N', 'R'},
	}

	return GameState{
		Board:              board,
		Turn:               "white",
		EnPassantSquare:    "-",
		FullMoveNumber:     1,
		WhiteCastle:        struct{ Kingside, Queenside bool }{true, true},
		BlackCastle:        struct{ Kingside, Queenside bool }{true, true},
		WhiteTimeRemaining: 5 * time.Minute,
		BlackTimeRemaining: 5 * time.Minute,
	}
}

func algebraicToIndices(alg string) (int, int, error) {
	if len(alg) != 2 {
		return 0, 0, errors.New("invalid algebraic notation")
	}

	file := rune(alg[0])
	rank := rune(alg[1])

	if file < 'a' || file > 'h' || rank < '1' || rank > '8' {
		return 0, 0, errors.New("invalid algebraic notation")
	}

	col := int(file - 'a')
	row := 7 - int(rank-'1') // Convert to 0-based index from top

	return col, row, nil
}

func indicesToAlgebraic(col, row int) string {
	file := rune('a' + col)
	rank := rune('1' + (7 - row))
	return string(file) + string(rank)
}

func isValidSquare(col, row int) bool {
	return col >= 0 && col < 8 && row >= 0 && row < 8
}

func (gs *GameState) getPieceAt(col, row int) rune {
	if !isValidSquare(col, row) {
		return Empty
	}
	return gs.Board[row][col]
}

func (gs *GameState) isOpponent(col, row int, isWhite bool) bool {
	piece := gs.getPieceAt(col, row)
	if piece == Empty {
		return false
	}
	pieceIsWhite := unicode.IsUpper(piece)
	return pieceIsWhite != isWhite
}

func (gs *GameState) isEmpty(col, row int) bool {
	return gs.getPieceAt(col, row) == Empty
}

func (gs *GameState) switchTurn() {
	if gs.Turn == "white" {
		gs.Turn = "black"
	} else {
		gs.Turn = "white"
	}
}

func (gs *GameState) isValidMove(move Move) (bool, error) {
	fromCol, fromRow, err := algebraicToIndices(move.From)
	if err != nil {
		fmt.Printf("Invalid from notation: %v\n", err)
		return false, err
	}
	toCol, toRow, err := algebraicToIndices(move.To)
	if err != nil {
		fmt.Printf("Invalid to notation: %v\n", err)
		return false, err
	}

	piece := gs.getPieceAt(fromCol, fromRow)
	if piece == Empty {
		fmt.Printf("No piece at %s\n", move.From)
		return false, errors.New("no piece at starting position")
	}

	isWhite := unicode.IsUpper(piece)
	if (isWhite && gs.Turn != "white") || (!isWhite && gs.Turn != "black") {
		fmt.Printf("Move %s-%s attempted out of turn\n", move.From, move.To)
		return false, errors.New("not your turn")
	}

	pieceType := unicode.ToLower(piece)
	fmt.Printf("Validating move: %s to %s, piece: %c\n", move.From, move.To, piece)

	switch pieceType {
	case Pawn:
		return gs.isValidPawnMove(fromCol, fromRow, toCol, toRow, isWhite)
	case Rook:
		return gs.isValidRookMove(fromCol, fromRow, toCol, toRow, isWhite)
	case Knight:
		return gs.isValidKnightMove(fromCol, fromRow, toCol, toRow, isWhite)
	case Bishop:
		return gs.isValidBishopMove(fromCol, fromRow, toCol, toRow, isWhite)
	case Queen:
		return gs.isValidQueenMove(fromCol, fromRow, toCol, toRow, isWhite)
	case King:
		return gs.isValidKingMove(fromCol, fromRow, toCol, toRow, isWhite)
	default:
		fmt.Printf("Invalid piece type: %c\n", piece)
		return false, errors.New("invalid piece type")
	}
}

func (gs *GameState) isValidPawnMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	direction := -1 // White pawns move up
	if !isWhite {
		direction = 1 // Black pawns move down
	}

	// Normal move forward
	if fromCol == toCol {
		if toRow == fromRow+direction && gs.isEmpty(toCol, toRow) {
			return true, nil
		}
		startRow := 6 // White pawn start
		if !isWhite {
			startRow = 1 // Black pawn start
		}
		if fromRow == startRow && toRow == fromRow+2*direction &&
			gs.isEmpty(toCol, toRow) && gs.isEmpty(toCol, toRow-direction) {
			return true, nil
		}
		return false, errors.New("invalid pawn move")
	}

	// Diagonal capture (including en passant)
	if abs(toCol-fromCol) == 1 && toRow == fromRow+direction {
		if gs.isOpponent(toCol, toRow, isWhite) {
			return true, nil // Normal capture
		}
		// En passant: Check if the move targets the en passant square
		targetSquare := indicesToAlgebraic(toCol, toRow)
		if gs.EnPassantSquare == targetSquare {
			capturedCol := toCol
			capturedRow := fromRow // Pawn to be captured is on the same row
			if gs.getPieceAt(capturedCol, capturedRow) == Pawn && gs.isOpponent(capturedCol, capturedRow, isWhite) {
				return true, nil
			}
		}
		return false, errors.New("invalid pawn capture")
	}

	return false, errors.New("invalid pawn move")
}

func (gs *GameState) isValidRookMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	if fromCol != toCol && fromRow != toRow {
		return false, errors.New("rook must move in straight line")
	}

	stepCol := 0
	stepRow := 0
	if fromCol != toCol {
		stepCol = 1
		if toCol < fromCol {
			stepCol = -1
		}
	} else {
		stepRow = 1
		if toRow < fromRow {
			stepRow = -1
		}
	}

	col, row := fromCol+stepCol, fromRow+stepRow
	for col != toCol || row != toRow {
		if !gs.isEmpty(col, row) {
			return false, errors.New("path is not clear")
		}
		col += stepCol
		row += stepRow
	}

	if !gs.isEmpty(toCol, toRow) && !gs.isOpponent(toCol, toRow, isWhite) {
		return false, errors.New("cannot capture your own piece")
	}
	return true, nil
}

func (gs *GameState) isValidKnightMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	colDiff := abs(toCol - fromCol)
	rowDiff := abs(toRow - fromRow)

	if !((colDiff == 2 && rowDiff == 1) || (colDiff == 1 && rowDiff == 2)) {
		return false, errors.New("invalid knight move")
	}

	if !gs.isEmpty(toCol, toRow) && !gs.isOpponent(toCol, toRow, isWhite) {
		return false, errors.New("cannot capture your own piece")
	}
	return true, nil
}

func (gs *GameState) isValidBishopMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	if abs(toCol-fromCol) != abs(toRow-fromRow) {
		return false, errors.New("bishop must move diagonally")
	}

	colStep := 1
	if toCol < fromCol {
		colStep = -1
	}
	rowStep := 1
	if toRow < fromRow {
		rowStep = -1
	}

	col, row := fromCol+colStep, fromRow+rowStep
	for col != toCol && row != toRow {
		if !gs.isEmpty(col, row) {
			return false, errors.New("path is not clear")
		}
		col += colStep
		row += rowStep
	}

	if !gs.isEmpty(toCol, toRow) && !gs.isOpponent(toCol, toRow, isWhite) {
		return false, errors.New("cannot capture your own piece")
	}
	return true, nil
}

func (gs *GameState) isValidQueenMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	if fromCol == toCol || fromRow == toRow {
		return gs.isValidRookMove(fromCol, fromRow, toCol, toRow, isWhite)
	}
	if abs(toCol-fromCol) == abs(toRow-fromRow) {
		return gs.isValidBishopMove(fromCol, fromRow, toCol, toRow, isWhite)
	}
	return false, errors.New("invalid queen move")
}

func (gs *GameState) isValidKingMove(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	if abs(toCol-fromCol) <= 1 && abs(toRow-fromRow) <= 1 {
		if !gs.isEmpty(toCol, toRow) && !gs.isOpponent(toCol, toRow, isWhite) {
			return false, errors.New("cannot capture own piece")
		}
		return true, nil
	}
	if fromRow == toRow && abs(toCol-fromCol) == 2 {
		return gs.isValidCastling(fromCol, fromRow, toCol, toRow, isWhite)
	}
	return false, errors.New("invalid king move")
}

func (gs *GameState) isValidCastling(fromCol, fromRow, toCol, toRow int, isWhite bool) (bool, error) {
	if gs.isKingInCheck(isWhite) {
		return false, errors.New("cannot castle while in check")
	}
	if fromCol == toCol {
		return false, errors.New("invalid castling: no movement")
	}

	castlingSide := "kingside"
	rookCol := 7
	rookToCol := 5
	if toCol < fromCol { // Queenside
		castlingSide = "queenside"
		rookCol = 0
		rookToCol = 3
	}
	if isWhite {
		if (castlingSide == "kingside" && !gs.WhiteCastle.Kingside) ||
			(castlingSide == "queenside" && !gs.WhiteCastle.Queenside) {
			return false, errors.New("castling no longer allowed")
		}
	} else {
		if (castlingSide == "kingside" && !gs.BlackCastle.Kingside) ||
			(castlingSide == "queenside" && !gs.BlackCastle.Queenside) {
			return false, errors.New("castling no longer allowed")
		}
	}

	start, end := min(fromCol, rookCol), max(fromCol, rookCol)
	for col := start + 1; col < end; col++ {
		if !gs.isEmpty(col, fromRow) {
			return false, errors.New("castling path is not clear")
		}
	}

	tempState := *gs
	tempState.Board[fromRow][toCol] = tempState.Board[fromRow][fromCol]
	tempState.Board[fromRow][fromCol] = Empty
	tempState.Board[fromRow][rookToCol] = tempState.Board[fromRow][rookCol]
	tempState.Board[fromRow][rookCol] = Empty
	tempState.Turn = "black"
	if isWhite {
		tempState.Turn = "white"
	}
	if tempState.isKingInCheck(isWhite) {
		return false, errors.New("cannot castle into check")
	}

	step := 1
	if toCol < fromCol {
		step = -1
	}
	for col := fromCol + step; col != toCol; col += step {
		tempState := *gs
		tempState.Board[fromRow][col] = tempState.Board[fromRow][fromCol]
		tempState.Board[fromRow][fromCol] = Empty
		tempState.Turn = "black"
		if isWhite {
			tempState.Turn = "white"
		}
		if tempState.isKingInCheck(isWhite) {
			return false, errors.New("cannot castle through check")
		}
	}

	return true, nil
}

func (gs *GameState) isKingInCheck(isWhite bool) bool {
	var kingCol, kingRow int
	var found bool
	lookingFor := 'K'
	if !isWhite {
		lookingFor = 'k'
	}

	for row := 0; row < 8; row++ {
		for col := 0; col < 8; col++ {
			if gs.Board[row][col] == lookingFor {
				kingCol = col
				kingRow = row
				found = true
				break
			}
		}
		if found {
			break
		}
	}

	if !found {
		return false
	}

	pawnDir := 1 // Black pawns attack upward
	if !isWhite {
		pawnDir = -1 // White pawns attack downward
	}
	for _, colOffset := range []int{-1, 1} {
		col := kingCol + colOffset
		row := kingRow + pawnDir
		if isValidSquare(col, row) {
			piece := gs.getPieceAt(col, row)
			if unicode.ToLower(piece) == Pawn && gs.isOpponent(col, row, isWhite) {
				return true
			}
		}
	}

	knightMoves := [][2]int{
		{-2, -1}, {-2, 1}, {-1, -2}, {-1, 2},
		{1, -2}, {1, 2}, {2, -1}, {2, 1},
	}
	for _, move := range knightMoves {
		col := kingCol + move[0]
		row := kingRow + move[1]
		if isValidSquare(col, row) {
			piece := gs.getPieceAt(col, row)
			if unicode.ToLower(piece) == Knight && gs.isOpponent(col, row, isWhite) {
				return true
			}
		}
	}

	rookDirections := [][2]int{{0, 1}, {1, 0}, {0, -1}, {-1, 0}}
	for _, dir := range rookDirections {
		col, row := kingCol+dir[0], kingRow+dir[1]
		for isValidSquare(col, row) {
			piece := gs.getPieceAt(col, row)
			if piece != Empty {
				if (unicode.ToLower(piece) == Rook || unicode.ToLower(piece) == Queen) && gs.isOpponent(col, row, isWhite) {
					return true
				}
				break
			}
			col += dir[0]
			row += dir[1]
		}
	}

	bishopDirections := [][2]int{{1, 1}, {1, -1}, {-1, 1}, {-1, -1}}
	for _, dir := range bishopDirections {
		col, row := kingCol+dir[0], kingRow+dir[1]
		for isValidSquare(col, row) {
			piece := gs.getPieceAt(col, row)
			if piece != Empty {
				if (unicode.ToLower(piece) == Bishop || unicode.ToLower(piece) == Queen) && gs.isOpponent(col, row, isWhite) {
					return true
				}
				break
			}
			col += dir[0]
			row += dir[1]
		}
	}

	kingDirections := [][2]int{
		{-1, -1}, {-1, 0}, {-1, 1},
		{0, -1}, {0, 1},
		{1, -1}, {1, 0}, {1, 1},
	}
	for _, dir := range kingDirections {
		col := kingCol + dir[0]
		row := kingRow + dir[1]
		if isValidSquare(col, row) {
			piece := gs.getPieceAt(col, row)
			if unicode.ToLower(piece) == King && gs.isOpponent(col, row, isWhite) {
				return true
			}
		}
	}

	return false
}

func (gs *GameState) generateAllMoves(isWhite bool) []Move {
	var moves []Move
	for row := 0; row < 8; row++ {
		for col := 0; col < 8; col++ {
			piece := gs.Board[row][col]
			if piece == Empty {
				continue
			}

			pieceIsWhite := unicode.IsUpper(piece)
			if pieceIsWhite != isWhite {
				continue
			}

			pieceType := unicode.ToLower(piece)
			from := indicesToAlgebraic(col, row)

			switch pieceType {
			case Pawn:
				moves = append(moves, gs.generatePawnMoves(col, row, isWhite, from)...)
			case Rook:
				moves = append(moves, gs.generateRookMoves(col, row, isWhite, from)...)
			case Knight:
				moves = append(moves, gs.generateKnightMoves(col, row, isWhite, from)...)
			case Bishop:
				moves = append(moves, gs.generateBishopMoves(col, row, isWhite, from)...)
			case Queen:
				moves = append(moves, gs.generateQueenMoves(col, row, isWhite, from)...)
			case King:
				moves = append(moves, gs.generateKingMoves(col, row, isWhite, from)...)
			}
		}
	}

	var validMoves []Move
	for _, move := range moves {
		if valid, _ := gs.isValidMove(move); valid && !gs.wouldBeInCheckAfterMove(move, isWhite) {
			validMoves = append(validMoves, move)
		}
	}
	return validMoves
}

func (gs *GameState) generatePawnMoves(col, row int, isWhite bool, from string) []Move {
	var moves []Move
	direction := -1 // White pawns move up
	if !isWhite {
		direction = 1 // Black pawns move down
	}

	if isValidSquare(col, row+direction) && gs.isEmpty(col, row+direction) {
		moves = append(moves, Move{From: from, To: indicesToAlgebraic(col, row+direction)})
	}

	startRow := 6 // White's pawn start row
	if !isWhite {
		startRow = 1 // Black's pawn start row
	}
	if row == startRow && gs.isEmpty(col, row+direction) && gs.isEmpty(col, row+2*direction) {
		moves = append(moves, Move{From: from, To: indicesToAlgebraic(col, row+2*direction)})
	}

	for _, colOffset := range []int{-1, 1} {
		newCol := col + colOffset
		newRow := row + direction
		if isValidSquare(newCol, newRow) {
			if gs.isOpponent(newCol, newRow, isWhite) {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			}
			targetSquare := indicesToAlgebraic(newCol, row)
			if gs.EnPassantSquare == targetSquare {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			}
		}
	}

	if (isWhite && row+direction == 0) || (!isWhite && row+direction == 7) {
		for i := range moves {
			moves[i].Promotion = 'Q' // Default to queen promotion
		}
	}

	return moves
}

func (gs *GameState) generateRookMoves(col, row int, isWhite bool, from string) []Move {
	var moves []Move
	directions := [][2]int{{0, 1}, {1, 0}, {0, -1}, {-1, 0}}

	for _, dir := range directions {
		newCol, newRow := col+dir[0], row+dir[1]
		for isValidSquare(newCol, newRow) {
			if gs.isEmpty(newCol, newRow) {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			} else {
				if gs.isOpponent(newCol, newRow, isWhite) {
					moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
				}
				break
			}
			newCol += dir[0]
			newRow += dir[1]
		}
	}
	return moves
}

func (gs *GameState) generateKnightMoves(col, row int, isWhite bool, from string) []Move {
	var moves []Move
	knightMoves := [][2]int{
		{-2, -1}, {-2, 1}, {-1, -2}, {-1, 2},
		{1, -2}, {1, 2}, {2, -1}, {2, 1},
	}

	for _, move := range knightMoves {
		newCol := col + move[0]
		newRow := row + move[1]
		if isValidSquare(newCol, newRow) {
			if gs.isEmpty(newCol, newRow) || gs.isOpponent(newCol, newRow, isWhite) {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			}
		}
	}
	return moves
}

func (gs *GameState) generateBishopMoves(col, row int, isWhite bool, from string) []Move {
	var moves []Move
	directions := [][2]int{{1, 1}, {1, -1}, {-1, 1}, {-1, -1}}

	for _, dir := range directions {
		newCol, newRow := col+dir[0], row+dir[1]
		for isValidSquare(newCol, newRow) {
			if gs.isEmpty(newCol, newRow) {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			} else {
				if gs.isOpponent(newCol, newRow, isWhite) {
					moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
				}
				break
			}
			newCol += dir[0]
			newRow += dir[1]
		}
	}
	return moves
}

func (gs *GameState) generateQueenMoves(col, row int, isWhite bool, from string) []Move {
	moves := gs.generateRookMoves(col, row, isWhite, from)
	return append(moves, gs.generateBishopMoves(col, row, isWhite, from)...)
}

func (gs *GameState) generateKingMoves(col, row int, isWhite bool, from string) []Move {
	var moves []Move
	directions := [][2]int{
		{-1, -1}, {-1, 0}, {-1, 1},
		{0, -1}, {0, 1},
		{1, -1}, {1, 0}, {1, 1},
	}

	for _, dir := range directions {
		newCol := col + dir[0]
		newRow := row + dir[1]
		if isValidSquare(newCol, newRow) {
			if gs.isEmpty(newCol, newRow) || gs.isOpponent(newCol, newRow, isWhite) {
				moves = append(moves, Move{From: from, To: indicesToAlgebraic(newCol, newRow)})
			}
		}
	}

	if isWhite {
		if gs.WhiteCastle.Kingside && gs.isEmpty(5, 7) && gs.isEmpty(6, 7) {
			if !gs.isKingInCheck(true) && !gs.wouldBeInCheckAfterMove(Move{From: from, To: "f1"}, true) {
				moves = append(moves, Move{From: from, To: "g1"})
			}
		}
		if gs.WhiteCastle.Queenside && gs.isEmpty(1, 7) && gs.isEmpty(2, 7) && gs.isEmpty(3, 7) {
			if !gs.isKingInCheck(true) && !gs.wouldBeInCheckAfterMove(Move{From: from, To: "d1"}, true) {
				moves = append(moves, Move{From: from, To: "c1"})
			}
		}
	} else {
		if gs.BlackCastle.Kingside && gs.isEmpty(5, 0) && gs.isEmpty(6, 0) {
			if !gs.isKingInCheck(false) && !gs.wouldBeInCheckAfterMove(Move{From: from, To: "f8"}, false) {
				moves = append(moves, Move{From: from, To: "g8"})
			}
		}
		if gs.BlackCastle.Queenside && gs.isEmpty(1, 0) && gs.isEmpty(2, 0) && gs.isEmpty(3, 0) {
			if !gs.isKingInCheck(false) && !gs.wouldBeInCheckAfterMove(Move{From: from, To: "d8"}, false) {
				moves = append(moves, Move{From: from, To: "c8"})
			}
		}
	}

	return moves
}

func (gs *GameState) wouldBeInCheckAfterMove(move Move, isWhite bool) bool {
	tempState := *gs
	fromCol, fromRow, _ := algebraicToIndices(move.From)
	toCol, toRow, _ := algebraicToIndices(move.To)

	piece := tempState.Board[fromRow][fromCol]
	pieceType := unicode.ToLower(piece)

	tempState.Board[toRow][toCol] = piece
	tempState.Board[fromRow][fromCol] = Empty

	if pieceType == Pawn {
		if fromCol != toCol && tempState.isEmpty(toCol, toRow) { // En passant
			capturedRow := fromRow + 1
			if isWhite {
				capturedRow = fromRow - 1
			}
			tempState.Board[capturedRow][toCol] = Empty
		}
	} else if pieceType == King && abs(toCol-fromCol) == 2 { // Castling
		rookFromCol := 0
		rookToCol := 3
		if toCol > fromCol { // Kingside
			rookFromCol = 7
			rookToCol = 5
		}
		tempState.Board[fromRow][rookToCol] = tempState.Board[fromRow][rookFromCol]
		tempState.Board[fromRow][rookFromCol] = Empty
	}

	return tempState.isKingInCheck(isWhite)
}

func (gs *GameState) hasLegalMoves(isWhite bool) bool {
	moves := gs.generateAllMoves(isWhite)
	fmt.Printf("Generated %d moves for %v\n", len(moves), isWhite)
	for _, move := range moves {
		if !gs.wouldBeInCheckAfterMove(move, isWhite) {
			return true
		}
	}
	return false
}

func (gs *GameState) makeMove(move Move) error {
	fromCol, fromRow, err := algebraicToIndices(move.From)
	if err != nil {
		return err
	}
	toCol, toRow, err := algebraicToIndices(move.To)
	if err != nil {
		return err
	}

	piece := gs.getPieceAt(fromCol, fromRow)
	isWhite := unicode.IsUpper(piece)
	pieceType := unicode.ToLower(piece)

	if valid, err := gs.isValidMove(move); !valid {
		fmt.Printf("Move %s-%s invalid: %v\n", move.From, move.To, err)
		return err
	}

	tempState := *gs
	wasCapture := false

	switch pieceType {
	case Pawn:
		if fromCol != toCol && gs.isEmpty(toCol, toRow) { // En passant
			capturedRow := fromRow // Captured pawn is on the same row
			tempState.Board[capturedRow][toCol] = Empty
			wasCapture = true
		} else if !gs.isEmpty(toCol, toRow) {
			wasCapture = true
		}
		if abs(toRow-fromRow) == 2 {
			enPassantRow := (fromRow + toRow) / 2
			tempState.EnPassantSquare = indicesToAlgebraic(fromCol, enPassantRow)
		} else {
			tempState.EnPassantSquare = "-"
		}
		if (isWhite && toRow == 0) || (!isWhite && toRow == 7) {
			promotion := move.Promotion
			if promotion == 0 {
				promotion = 'Q'
			}
			promotion = unicode.ToUpper(promotion)
			if promotion != 'Q' && promotion != 'R' && promotion != 'B' && promotion != 'N' {
				return errors.New("invalid promotion piece")
			}
			piece = promotion
			if !isWhite {
				piece = unicode.ToLower(promotion)
			}
		}
	case King:
		if abs(toCol-fromCol) == 2 { // Castling
			rookFromCol := 7
			rookToCol := 5
			if toCol < fromCol {
				rookFromCol = 0
				rookToCol = 3
			}
			tempState.Board[fromRow][rookToCol] = tempState.Board[fromRow][rookFromCol]
			tempState.Board[fromRow][rookFromCol] = Empty
		}
		if isWhite {
			tempState.WhiteCastle.Kingside = false
			tempState.WhiteCastle.Queenside = false
		} else {
			tempState.BlackCastle.Kingside = false
			tempState.BlackCastle.Queenside = false
		}
	case Rook:
		if isWhite {
			if fromCol == 0 && fromRow == 7 {
				tempState.WhiteCastle.Queenside = false
			} else if fromCol == 7 && fromRow == 7 {
				tempState.WhiteCastle.Kingside = false
			}
		} else {
			if fromCol == 0 && fromRow == 0 {
				tempState.BlackCastle.Queenside = false
			} else if fromCol == 7 && fromRow == 0 {
				tempState.BlackCastle.Kingside = false
			}
		}
	}

	if !wasCapture && !gs.isEmpty(toCol, toRow) {
		wasCapture = true
	}

	tempState.Board[toRow][toCol] = piece
	tempState.Board[fromRow][fromCol] = Empty

	if tempState.isKingInCheck(isWhite) {
		fmt.Printf("Move %s-%s leaves king in check\n", move.From, move.To)
		return errors.New("move leaves your king in check")
	}

	*gs = tempState
	gs.switchTurn()

	if gs.Turn == "black" {
		gs.FullMoveNumber++
	}
	if pieceType == Pawn || wasCapture {
		gs.HalfMoveClock = 0
	} else {
		gs.HalfMoveClock++
	}

	opponentInCheck := gs.isKingInCheck(!isWhite)
	opponentHasMoves := gs.hasLegalMoves(!isWhite)

	gs.IsCheck = opponentInCheck
	gs.IsCheckmate = opponentInCheck && !opponentHasMoves
	gs.IsStalemate = !opponentInCheck && !opponentHasMoves
	gs.IsDraw = gs.IsStalemate || gs.HalfMoveClock >= 50 || gs.isInsufficientMaterial()

	fmt.Printf("After move %v: Check=%v, Checkmate=%v, Stalemate=%v, Draw=%v, Has moves? %v\n",
		move, gs.IsCheck, gs.IsCheckmate, gs.IsStalemate, gs.IsDraw, opponentHasMoves)

	return nil
}

func (gs *GameState) isInsufficientMaterial() bool {
	pieceCount := make(map[rune]int)
	var whiteBishops, blackBishops [][2]int
	for row := 0; row < 8; row++ {
		for col := 0; col < 8; col++ {
			piece := gs.Board[row][col]
			if piece != Empty {
				pieceCount[unicode.ToLower(piece)]++
				if piece == 'B' {
					whiteBishops = append(whiteBishops, [2]int{col, row})
				} else if piece == 'b' {
					blackBishops = append(blackBishops, [2]int{col, row})
				}
			}
		}
	}

	if len(pieceCount) == 1 && pieceCount['k'] > 0 {
		return true
	}

	if len(pieceCount) == 2 && pieceCount['k'] == 2 {
		if pieceCount['b'] == 1 || pieceCount['n'] == 1 {
			return true
		}
	}

	totalBishops := pieceCount['b']
	if totalBishops > 0 && len(pieceCount) == 2 && pieceCount['k'] == 2 {
		allSameColor := true
		bishops := append(whiteBishops, blackBishops...)
		if len(bishops) > 0 {
			baseColor := (bishops[0][0] + bishops[0][1]) % 2
			for _, pos := range bishops[1:] {
				if (pos[0]+pos[1])%2 != baseColor {
					allSameColor = false
					break
				}
			}
		}
		if allSameColor {
			return true
		}
	}

	if len(pieceCount) == 2 && pieceCount['k'] == 2 && pieceCount['n'] == 2 {
		return true
	}

	return false
}

func min(a, b int) int {
	if a < b {
		return a
	}
	return b
}

func max(a, b int) int {
	if a > b {
		return a
	}
	return b
}

func abs(x int) int {
	if x < 0 {
		return -x
	}
	return x
}

func (gs *GameState) StartTimer(notifyFunc func(string, interface{})) {
	if gs.Timer != nil {
		gs.Timer.Stop()
	}

	gs.LastMoveTime = time.Now()
	gs.Timer = time.AfterFunc(1*time.Second, func() {
		gs.onTimerTick(notifyFunc)
	})
}

func (gs *GameState) onTimerTick(notifyFunc func(string, interface{})) {
	now := time.Now()
	elapsed := now.Sub(gs.LastMoveTime)
	gs.LastMoveTime = now

	if gs.Turn == "white" {
		gs.WhiteTimeRemaining -= elapsed
		if gs.WhiteTimeRemaining <= 0 {
			gs.WhiteTimeRemaining = 0
			gs.IsCheckmate = true
			notifyFunc(EventTypeGameOver, EventGameOver{Reason: "timeout"})
			gs.StopTimer()
			return
		}
	} else {
		gs.BlackTimeRemaining -= elapsed
		if gs.BlackTimeRemaining <= 0 {
			gs.BlackTimeRemaining = 0
			gs.IsCheckmate = true
			notifyFunc(EventTypeGameOver, EventGameOver{Reason: "timeout"})
			gs.StopTimer()
			return
		}
	}

	if time.Since(gs.LastTimeUpdateSent) > (5 * time.Second) {
		notifyFunc(EventTypeTimeUpdate, EventTimeUpdate{
			WhiteTime: gs.WhiteTimeRemaining.Seconds(),
			BlackTime: gs.BlackTimeRemaining.Seconds(),
		})
		gs.LastTimeUpdateSent = time.Now()
	}

	gs.Timer = time.AfterFunc(1*time.Second, func() {
		gs.onTimerTick(notifyFunc)
	})
}

func (gs *GameState) StopTimer() {
	if gs.Timer != nil {
		gs.Timer.Stop()
		gs.Timer = nil
	}
}
