package main

import (
	"database/sql"
	"encoding/json"
	"net/http"

	"golang.org/x/crypto/bcrypt"
)

type LoginData struct {
	Id           int    `json:"user_id"`
	RefreshToken string `json:"refresh_token"`
	AccessToken  string `json:"access_token"`
}

func loginHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	var user struct {
		Username string `json:"username"`
		Password string `json:"password"`
	}

	// Parse the request body into the struct
	if err := json.NewDecoder(r.Body).Decode(&user); err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusBadRequest)
		return
	}

	if len(user.Username) < 3 || len(user.Username) > 20 || len(user.Password) < 8 || len(user.Password) > 72 {
		sendErrorResponse(w, "Niepoprawne dane logowania", http.StatusUnauthorized)
		return
	}

	var storedPassword string
	var userID int
	err := db.QueryRow("SELECT id, password FROM users WHERE username = ?", user.Username).Scan(&userID, &storedPassword)
	if err != nil {
		sendErrorResponse(w, "Niepoprawne dane logowania", http.StatusUnauthorized)
		return
	}

	if err := bcrypt.CompareHashAndPassword([]byte(storedPassword), []byte(user.Password)); err != nil {
		sendErrorResponse(w, "Niepoprawne dane logowania", http.StatusUnauthorized)
		return
	}

	accessToken, err := createAccessToken(userID)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	refreshToken, err := createRefreshToken(db, userID, GetIP(r), 0)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(LoginData{Id: userID, RefreshToken: refreshToken, AccessToken: accessToken})
}

func logoutHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	var data struct {
		Token string `json:"refresh_token"`
	}

	if err := json.NewDecoder(r.Body).Decode(&data); err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusBadRequest)
		return
	}

	_, err := db.Exec("UPDATE refresh_tokens SET deactivated = 1 WHERE token = ?", data.Token)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	w.WriteHeader(http.StatusOK)
}
