package main

import (
	"database/sql"
	"embed"
	"encoding/json"
	"fmt"
	"log"
	"net/http"
	"os"
	"sync"
	"text/template"

	_ "modernc.org/sqlite"
)

//go:embed templates/*
var templatesFS embed.FS

const MAX_REQ_SIZE = 1000000

type TemplateData struct {
	ExeSize string
	ZipSize string
}

type FileSizes struct {
	ExeSize string
	ZipSize string
	mu      sync.RWMutex
}

var fileSizes FileSizes

type ErrorResponse struct {
	Message string `json:"message"`
}

func bytesToHumanReadable(size int64) string {
	const (
		KB = 1024
		MB = KB * 1024
		GB = MB * 1024
	)

	switch {
	case size >= GB:
		return fmt.Sprintf("%.2f GB", float64(size)/float64(GB))
	case size >= MB:
		return fmt.Sprintf("%.2f MB", float64(size)/float64(MB))
	case size >= KB:
		return fmt.Sprintf("%.2f KB", float64(size)/float64(KB))
	default:
		return fmt.Sprintf("%d B", size)
	}
}

func loadFileSizes() error {
	exeInfo, err := os.Stat("./downloads/latest.exe")
	if err != nil {
		return err
	}

	zipInfo, err := os.Stat("./downloads/latest.zip")
	if err != nil {
		return err
	}

	fileSizes.mu.Lock()
	fileSizes.ExeSize = bytesToHumanReadable(exeInfo.Size())
	fileSizes.ZipSize = bytesToHumanReadable(zipInfo.Size())
	fileSizes.mu.Unlock()

	return nil
}

func sendErrorResponse(w http.ResponseWriter, message string, statusCode int) {
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(statusCode)
	json.NewEncoder(w).Encode(ErrorResponse{Message: message})
}

func main() {

	logFile, err := os.OpenFile("szachy.log", os.O_APPEND|os.O_CREATE|os.O_WRONLY, 0644)
	if err != nil {
		log.Fatal(err)
	}
	defer logFile.Close()

	log.SetOutput(logFile)

	if err := loadFileSizes(); err != nil {
		log.Fatal("Failed to load file sizes:", err)
	}

	db, err := sql.Open("sqlite", "chess.db")
	if err != nil {
		log.Fatal("Failed to connect to database:", err)
	}
	defer db.Close()

	createTables(db)

	jwtSecret = []byte(os.Getenv("CHESS_JWT_SECRET"))

	if len(jwtSecret) == 0 {
		log.Fatal("JWT SECRET ENV VAR NOT SET\n")
		return
	}

	mux := http.NewServeMux()

	indexTmpl, err := template.ParseFS(templatesFS, "templates/index.html")
	if err != nil {
		log.Fatal("Error parsing template:", err)
	}

	docsTmpl, err := template.ParseFS(templatesFS, "templates/docs.html")
	if err != nil {
		log.Fatal("Error parsing template:", err)
	}

	mux.Handle("GET /", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		if r.URL.Path != "/" {
			http.Error(w, "Not found", http.StatusNotFound)
			return
		}

		fileSizes.mu.RLock()
		data := TemplateData{
			ExeSize: fileSizes.ExeSize,
			ZipSize: fileSizes.ZipSize,
		}
		fileSizes.mu.RUnlock()

		w.Header().Set("Content-Type", "text/html")
		if err := indexTmpl.Execute(w, data); err != nil {
			log.Printf("Error executing template: %v", err)
			http.Error(w, "Internal server error", http.StatusInternalServerError)
		}
	})))

	mux.Handle("POST /login", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { loginHandler(db, w, r) })))
	mux.Handle("POST /refresh", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { refreshTokenHandler(db, w, r) })))
	mux.Handle("POST /logout", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { logoutHandler(db, w, r) })))
	mux.Handle("POST /register", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { registerHandler(db, w, r) })))

	// for authorized only

	mux.Handle("GET /user/{id...}", AuthMiddleware(LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { userHandler(db, w, r) }))))
	mux.Handle("GET /ip", AuthMiddleware(LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.WriteHeader(200)
		w.Write([]byte(GetIP(r)))
	}))))

	// online game

	mux.Handle("GET /rooms", AuthMiddleware(LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { roomsHandler(db, w, r, false) }))))
	mux.Handle("GET /rooms/all", AuthMiddleware(LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { roomsHandler(db, w, r, true) }))))
	mux.Handle("GET /rooms/{id}", AuthMiddleware(WSMiddleWare(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { gameHandler(db, w, r) }))))
	mux.Handle("POST /rooms", AuthMiddleware(LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { createRoomHandler(db, w, r) }))))

	mux.Handle("GET /top/{limit}", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) { rankingHandler(db, w, r) })))

	mux.Handle("GET /download/executable", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.Header().Set("Content-Disposition", `attachment; filename="Szachy Azbestest.exe"`)
		http.ServeFile(w, r, "./downloads/latest.exe")
	})))

	mux.Handle("GET /download/source", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.Header().Set("Content-Disposition", `attachment; filename="Szachy Azbestest.zip"`)
		http.ServeFile(w, r, "./downloads/latest.zip")
	})))

	mux.Handle("GET /docs", LoggingMiddleware(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		w.WriteHeader(http.StatusOK)
		if err := docsTmpl.Execute(w, nil); err != nil {
			log.Printf("Error executing template: %v", err)
			http.Error(w, "Internal server error", http.StatusInternalServerError)
		}
	})))

	addr := ":8080"
	if len(os.Args) > 1 {
		addr = os.Args[1]
	}
	server := &http.Server{
		Addr:    addr,
		Handler: mux,
	}

	log.Println("Server started on :8080")
	log.Fatal(server.ListenAndServeTLS("cert.pem", "key.pem"))

	//log.Printf("Server started on %s\n", addr)
	log.Fatal(server.ListenAndServeTLS("cert.pem", "key.pem"))
	log.Fatal(server.ListenAndServe())
}

func GetIP(r *http.Request) string {
	ip := r.Header.Get("CF-Connecting-IP")
	if ip == "" {
		ip = r.RemoteAddr
	}
	return ip
}

type responseWriter struct {
	http.ResponseWriter
	statusCode int
}

func (rw *responseWriter) WriteHeader(code int) {
	rw.statusCode = code
	rw.ResponseWriter.WriteHeader(code)
}

func createTables(db *sql.DB) {
	_, err := db.Exec(`
		CREATE TABLE IF NOT EXISTS users (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			username VARCHAR(20) UNIQUE NOT NULL,
			display_name VARCHAR(20),
			password VARCHAR(60) NOT NULL,
			created_at DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
			elo INTEGER NOT NULL DEFAULT 1500
		);

		CREATE TABLE IF NOT EXISTS refresh_tokens (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			user_id INTEGER NOT NULL,
			token TEXT UNIQUE NOT NULL,
			created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
			ip_address TEXT,
			expires_at DATETIME DEFAULT (DATETIME('now', '+30 days')),
			used_at DATETIME DEFAULT NULL,
			parent_token_id INTEGER DEFAULT NULL,
			deactivated BOOLEAN DEFAULT FALSE,
			FOREIGN KEY (user_id) REFERENCES users(id)
		);

		CREATE TABLE IF NOT EXISTS rooms (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			code VARCHAR(6) UNIQUE NOT NULL, -- 6-digit code, e.g., "123456"
			name VARCHAR(20) NOT NULL,
			is_public BOOLEAN NOT NULL DEFAULT FALSE,
			created_at DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
			creator_id INTEGER NOT NULL,
			white_player_id INTEGER,
			black_player_id INTEGER,
			status VARCHAR(20) NOT NULL DEFAULT 'open', -- 'open', 'in_progress', 'finished', 'abandoned'
			winner_id INTEGER,
			result VARCHAR(20), -- 'white_won', 'black_won', 'draw', 'abandoned', NULL if ongoing
			started_at DATETIME,
			ended_at DATETIME,
			white_time_remaining INTEGER DEFAULT 0, -- Time left at game end (ms)
			black_time_remaining INTEGER DEFAULT 0, -- Time left at game end (ms)
			initial_time INTEGER NOT NULL DEFAULT 300000, -- Initial time per player (ms)
			FOREIGN KEY (creator_id) REFERENCES users(id),
			FOREIGN KEY (white_player_id) REFERENCES users(id),
			FOREIGN KEY (black_player_id) REFERENCES users(id),
			FOREIGN KEY (winner_id) REFERENCES users(id)
		);

		CREATE TABLE IF NOT EXISTS moves (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			room_id INTEGER NOT NULL, -- Now references rooms(id)
			move_number INTEGER NOT NULL,
			player_id INTEGER NOT NULL,
			move_notation VARCHAR(10) NOT NULL,
			timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
			time_remaining INTEGER NOT NULL, -- Time left after this move (ms)
			is_check BOOLEAN NOT NULL DEFAULT FALSE,
			is_checkmate BOOLEAN NOT NULL DEFAULT FALSE,
			is_capture BOOLEAN NOT NULL DEFAULT FALSE,
			FOREIGN KEY (room_id) REFERENCES rooms(id),
			FOREIGN KEY (player_id) REFERENCES users(id)
		);

		CREATE TABLE IF NOT EXISTS game_events (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			room_id INTEGER NOT NULL, -- Now references rooms(id)
			event_type VARCHAR(20) NOT NULL,
			player_id INTEGER,
			payload TEXT,
			time_remaining INTEGER NOT NULL DEFAULT 0, -- Time left for this player at event time (ms)
			timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
			FOREIGN KEY (room_id) REFERENCES rooms(id),
			FOREIGN KEY (player_id) REFERENCES users(id)
		);

		CREATE TABLE IF NOT EXISTS game_states (
			id INTEGER PRIMARY KEY AUTOINCREMENT,
			room_id INTEGER NOT NULL, -- Now references rooms(id)
			move_number INTEGER NOT NULL,
			board_state TEXT NOT NULL,
			turn VARCHAR(6) NOT NULL,
			white_castle_kingside BOOLEAN NOT NULL,
			white_castle_queenside BOOLEAN NOT NULL,
			black_castle_kingside BOOLEAN NOT NULL,
			black_castle_queenside BOOLEAN NOT NULL,
			en_passant_square VARCHAR(2) NOT NULL,
			half_move_clock INTEGER NOT NULL,
			full_move_number INTEGER NOT NULL,
			timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
			FOREIGN KEY (room_id) REFERENCES rooms(id)
		);
	`)
	if err != nil {
		log.Fatal("Failed to create tables:", err)
	}
}
