package main

import (
	"context"
	"log"
	"net/http"
	"strings"
	"time"
)

type contextKey string

const userIDKey contextKey = "userID"
const roomKey contextKey = "roomKey"
const connKey contextKey = "connKey"
const roleKey contextKey = "roleKey" // is white == is creator ///

func WSMiddleWare(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		roomID := strings.Trim(r.PathValue("id"), " ")
		if len(roomID) != 6 {
			sendErrorResponse(w, "Nieznaleziono takiego pokoju", http.StatusNotFound)
			return
		}

		roomsMu.RLock()
		room, exists := rooms[roomID]
		roomsMu.RUnlock()
		if !exists {
			sendErrorResponse(w, "Nieznaleziono takiego pokoju", http.StatusNotFound)
			return
		}

		userID := getUserIDFromContext(r)
		// _, isPlayerInRoom := rooms.isPlayerInRoom(userID)
		// if isPlayerInRoom {
		// 	sendErrorResponse(w, "Już jesteś połączony z innym pokojem", http.StatusNotFound)
		// 	return
		// }

		ctx := context.WithValue(r.Context(), roomKey, room)
		roomsMu.Lock()
		if room.isFull() && !room.isPlayerInvolved(userID) {
			roomsMu.Unlock()
			sendErrorResponse(w, "Pokój jest już pełen", http.StatusBadRequest)
			return
		}
		if (room.WhitePlayer.UserID == userID && room.isWhiteConnected()) ||
			(room.BlackPlayer.UserID == userID && room.isBlackConnected()) {
			roomsMu.Unlock()
			sendErrorResponse(w, "Już jesteś połączony do tego pokoju", http.StatusBadRequest)
			return
		}

		if room.BlackPlayer.UserID == 0 && room.WhitePlayer.UserID != userID {
			ctx = context.WithValue(ctx, connKey, &room.BlackPlayer.Conn)
			room.BlackPlayer.UserID = userID
		} else {
			ctx = context.WithValue(ctx, connKey, &room.WhitePlayer.Conn)
		}
		if room.WhitePlayer.UserID == userID {
			ctx = context.WithValue(ctx, roleKey, "white")
		} else {
			ctx = context.WithValue(ctx, roleKey, "black")
		}

		// if room.WhitePlayer.UserID == userID {
		// 	if room.isWhiteConnected() {
		// 		sendErrorResponse(w, "Już jesteś połączony do tego pokoju", http.StatusNotFound)
		// 		return
		// 	}
		// 	ctx = context.WithValue(ctx, connKey, &room.WhitePlayer.Conn)
		// 	ctx = context.WithValue(ctx, roleKey, true)
		// } else if !room.isBlackConnected() && (room.BlackPlayer.UserID == 0 || room.BlackPlayer.UserID == userID) {
		// 	room.BlackPlayer.UserID = userID
		// 	ctx = context.WithValue(ctx, connKey, &room.BlackPlayer.Conn)
		// 	ctx = context.WithValue(ctx, roleKey, false)
		// } else {
		// 	roomsMu.Unlock()
		// 	sendErrorResponse(w, "Pokój jest już pełen", http.StatusNotFound)
		// 	return
		// }
		roomsMu.Unlock()
		next.ServeHTTP(w, r.WithContext(ctx))
	})
}

func LoggingMiddleware(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		start := time.Now()

		lrw := &responseWriter{ResponseWriter: w, statusCode: http.StatusOK}
		next.ServeHTTP(lrw, r)

		duration := time.Since(start)
		log.Printf("%s %s | Status: %d | IP: %s | Took: %v | UserID: %d",
			r.Method, r.URL.Path, lrw.statusCode, GetIP(r), duration, getUserIDFromContext(r))
	})
}

func AuthMiddleware(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		authHeader := r.Header.Get("Authorization")
		if authHeader == "" {
			log.Printf("Unauthorized request: No Authorization header | %s %s | IP: %s", r.Method, r.URL.Path, GetIP(r))
			w.WriteHeader(http.StatusUnauthorized)
			return
		}

		// "Bearer <token>"
		parts := strings.Split(authHeader, " ")
		if len(parts) != 2 || parts[0] != "Bearer" {
			log.Printf("Unauthorized request: Invalid token format | %s %s | IP: %s", r.Method, r.URL.Path, GetIP(r))
			w.WriteHeader(http.StatusUnauthorized)
			return
		}

		tokenString := parts[1]
		userID := getUserIDFromToken(tokenString)
		if userID == 0 {
			log.Printf("Unauthorized request: Invalid token | %s %s | IP: %s", r.Method, r.URL.Path, GetIP(r))
			w.WriteHeader(http.StatusUnauthorized)
			return
		}
		ctx := context.WithValue(r.Context(), userIDKey, userID)
		next.ServeHTTP(w, r.WithContext(ctx))
	})
}
