package main

import (
	"database/sql"
	"encoding/json"
	"log"
	"net/http"
	"strings"

	"golang.org/x/crypto/bcrypt"
)

func HashPassword(password string) (string, error) {
	hashedBytes, err := bcrypt.GenerateFromPassword([]byte(password), bcrypt.DefaultCost)
	if err != nil {
		return "", err
	}
	return string(hashedBytes), nil
}

func registerHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	// Read JSON input
	var user struct {
		Username    string `json:"username"`
		DisplayName string `json:"display_name"`
		Password    string `json:"password"`
	}

	// Parse the request body into the struct
	if err := json.NewDecoder(r.Body).Decode(&user); err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusBadRequest)
		return
	}

	// Validate username length (3-20 characters)
	if len(user.Username) < 3 || len(user.Username) > 20 {
		sendErrorResponse(w, "Nazwa użytkownika musi mieć od 3 do 20 znaków", http.StatusBadRequest)
		return
	}

	// Validate password length (minimum 8 characters, max to fit bcrypt hash)
	if len(user.Password) < 8 || len(user.Password) > 72 { // bcrypt max length is 72
		sendErrorResponse(w, "Hasło musi mieć co najmniej 8 znaków", http.StatusBadRequest)
		return
	}

	// Hash the password
	hashedPassword, err := HashPassword(user.Password)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	// Prepare the SQL statement
	stmt, err := db.Prepare(`
		INSERT INTO users (username, display_name, password, created_at)
		VALUES (?, ?, ?, CURRENT_TIMESTAMP)
	`)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}
	defer stmt.Close()

	// Execute the SQL statement and check for unique constraint violation
	result, err := stmt.Exec(user.Username, user.DisplayName, hashedPassword)
	if err != nil {
		// Check if the error is related to unique constraint violation (username already taken)
		if strings.Contains(err.Error(), "UNIQUE constraint failed") {
			sendErrorResponse(w, "Nazwa użytkownika jest już zajęta", http.StatusBadRequest)
			return
		}
		log.Printf("Error executing statement: %v", err) // Log the detailed error
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	// Get the last inserted ID
	id, err := result.LastInsertId()
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	accessToken, err := createAccessToken(int(id))
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	refreshToken, err := createRefreshToken(db, int(id), GetIP(r), 0)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	// Respond with the created user ID
	response := LoginData{
		Id:           int(id),
		RefreshToken: refreshToken,
		AccessToken:  accessToken,
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(response)
}
