package main

import (
	"database/sql"
	"fmt"
	"log"
	"math/rand"
	"strings"
	"sync"
	"time"

	"github.com/coder/websocket"
)

const MAX_ROOM_NAME_LEN = 10 // +#123456
const ROOM_DEFAULT_NAME = "Pokój"

type WSMessage struct {
	Type    string `json:"type"`
	Payload any    `json:"payload"`
}

type Player struct {
	UserID int
	Conn   *websocket.Conn
}

type PlayerSerializable struct {
	IsConnected bool `json:"is_connected"`
	SimpleUserProfile
}

type RoomSerializable struct {
	ID          string              `json:"id"`
	Name        string              `json:"name"`
	WhitePlayer *PlayerSerializable `json:"white_player"`
	BlackPlayer *PlayerSerializable `json:"black_player"`
}

type MoveHistoryEntry struct {
	State GameState
	Move  string
}

type MoveHistory []MoveHistoryEntry

type Room struct {
	ID              string
	Name            string
	IsPublic        bool
	CreatedAt       time.Time
	WhitePlayer     Player
	BlackPlayer     Player
	GameState       GameState
	WinnerID        int
	DrawRequestedBy string
	UndoRequestedBy string
	UID             int
	EloUpdated      bool
	mu              sync.Mutex
}

type Rooms map[string]*Room

var (
	rooms   = make(Rooms)
	roomsMu sync.RWMutex
)

func (rooms *Rooms) isPlayerInRoom(playerID int) (string, bool) {
	for _, room := range *rooms {
		if room.isPlayerInvolved(playerID) {
			return room.ID, true
		}
	}
	return "", false
}

func (room *Room) isWhiteConnected() bool {
	return room.WhitePlayer.Conn != nil
}

func (room *Room) isBlackConnected() bool {
	return room.BlackPlayer.Conn != nil
}

func (room *Room) arePlayersConnected() bool {
	return room.isWhiteConnected() && room.isBlackConnected()
}

func (room *Room) isFull() bool {
	return room.WhitePlayer.UserID != 0 && room.BlackPlayer.UserID != 0
}

func (room *Room) isPlayerInvolved(playerID int) bool {
	return room.WhitePlayer.UserID == playerID || room.BlackPlayer.UserID == playerID
}

func (room *Room) disconnectPlayers() {
	if room.WhitePlayer.Conn != nil {
		room.WhitePlayer.Conn.CloseNow()
		room.WhitePlayer.Conn = nil
	}
	if room.BlackPlayer.Conn != nil {
		room.BlackPlayer.Conn.CloseNow()
		room.BlackPlayer.Conn = nil
	}
}

func (room *Room) close(db *sql.DB) {
	result := "abandoned"
	var winnerID sql.NullInt64
	if room.WinnerID != 0 {
		winnerID = sql.NullInt64{Int64: int64(room.WinnerID), Valid: true}
		if room.WinnerID == room.WhitePlayer.UserID {
			result = "white_won"
		} else {
			result = "black_won"
		}
	} else if room.GameState.IsDraw {
		result = "draw"
	}

	_, err := db.Exec(`
		UPDATE rooms
		SET status = 'finished', winner_id = ?, result = ?, ended_at = CURRENT_TIMESTAMP,
			white_time_remaining = ?, black_time_remaining = ?
		WHERE id = ?`,
		winnerID, result,
		room.GameState.WhiteTimeRemaining.Milliseconds(),
		room.GameState.BlackTimeRemaining.Milliseconds(),
		room.UID)
	if err != nil {
		log.Printf("Failed to update room status for UID %d: %v", room.UID, err)
	}

	room.stopTimer()
	room.disconnectPlayers()
	delete(rooms, room.ID)
}

func (room *Room) makeWinner(winnerID int) {
	if room.WinnerID != 0 || room.GameState.IsDraw {
		return
	}
	room.WinnerID = winnerID
	fmt.Printf("User with ID %d won room#%s\n", winnerID, room.ID)
}

func (room *Room) toSerializable(db *sql.DB) *RoomSerializable {
	rs := RoomSerializable{
		ID:          room.ID,
		Name:        room.Name,
		BlackPlayer: nil,
		WhitePlayer: nil,
	}
	var user UserProfile
	var err error
	if room.BlackPlayer.UserID != 0 {
		rs.BlackPlayer = &PlayerSerializable{
			IsConnected: room.BlackPlayer.Conn != nil,
		}
		user, err = getUserProfileByID(db, room.BlackPlayer.UserID)
		if err == nil {
			rs.BlackPlayer.SimpleUserProfile = user.Simplify()
		}
	}
	if room.WhitePlayer.UserID != 0 {
		rs.WhitePlayer = &PlayerSerializable{
			IsConnected: room.WhitePlayer.Conn != nil,
		}
		user, err = getUserProfileByID(db, room.WhitePlayer.UserID)
		if err == nil {
			rs.WhitePlayer.SimpleUserProfile = user.Simplify()
		}
	}
	return &rs
}

func newRoom(db *sql.DB, name string, isPublic bool, creatorID int) *Room {
	var code string
	attempts := 0
	maxAttempts := 10

	roomsMu.Lock()
	defer roomsMu.Unlock()

	for attempts < maxAttempts {
		code = fmt.Sprintf("%06d", rand.Intn(1000000))
		var count int
		err := db.QueryRow("SELECT COUNT(*) FROM rooms WHERE code = ?", code).Scan(&count)
		if err == nil && count == 0 {
			break
		}
		attempts++
		if attempts >= maxAttempts {
			return nil
		}
	}

	finalName := name
	if finalName == "" {
		finalName = fmt.Sprintf("%s#%s", ROOM_DEFAULT_NAME, code)
	} else if len(finalName) > MAX_ROOM_NAME_LEN {
		finalName = finalName[:MAX_ROOM_NAME_LEN]
	}

	// Insert into rooms table
	result, err := db.Exec(`
		INSERT INTO rooms (code, name, is_public, creator_id, white_player_id, status, initial_time)
		VALUES (?, ?, ?, ?, ?, 'open', 300000)`,
		code, finalName, isPublic, creatorID, creatorID)
	if err != nil {
		log.Printf("Failed to create room in DB: %v", err)
		return nil
	}

	roomUID, _ := result.LastInsertId() // Get the auto-incremented id

	room := &Room{
		ID:          code,
		Name:        finalName,
		IsPublic:    isPublic,
		CreatedAt:   time.Now(),
		WhitePlayer: Player{UserID: creatorID},
		GameState:   NewGameState(),
		UID:         int(roomUID),
	}
	rooms[code] = room
	return room
}

func (room *Room) GetOpponent(playerID int) *Player {
	if room.WhitePlayer.UserID == playerID {
		return &room.BlackPlayer
	} else if room.BlackPlayer.UserID == playerID {
		return &room.WhitePlayer
	}
	return nil
}

func (room *Room) undoLastMove() error {
	if len(room.GameState.MoveHistory) == 0 {
		return fmt.Errorf("no moves to undo")
	}
	lastEntry := room.GameState.MoveHistory[len(room.GameState.MoveHistory)-1]
	room.GameState = lastEntry.State
	return nil
}

func (room *Room) makeMove(moveStr string) error { // Removed db param since logging moved to gameHandler
	parts := strings.Split(moveStr, "-")
	if len(parts) != 2 {
		return fmt.Errorf("invalid move format")
	}

	move := Move{
		From: parts[0],
		To:   parts[1],
	}

	if strings.Contains(move.To, "=") {
		promotionParts := strings.Split(move.To, "=")
		if len(promotionParts) != 2 {
			return fmt.Errorf("invalid promotion format")
		}
		move.To = promotionParts[0]
		move.Promotion = rune(promotionParts[1][0])
	}

	currentState := room.GameState

	if err := room.GameState.makeMove(move); err != nil {
		return err
	}

	room.GameState.MoveHistory = append(room.GameState.MoveHistory, MoveHistoryEntry{
		State: currentState,
		Move:  moveStr,
	})
	fmt.Printf("added %s to move history with %d moves in history\n", moveStr, len(currentState.MoveHistory))

	room.stopTimer()
	room.startTimer()
	return nil
}

func (room *Room) startTimer() {
	room.GameState.StartTimer(func(eventType string, payload any) {
		notifyPlayers(room, eventType, payload)
		if eventType == EventTypeGameOver {
			if payload.(EventGameOver).Reason == "timeout" {
				roomsMu.Lock()
				defer roomsMu.Unlock()
				if room.GameState.Turn == "white" {
					room.makeWinner(room.BlackPlayer.UserID)
				} else {
					room.makeWinner(room.WhitePlayer.UserID)
				}
			}
		}
	})
}

func (room *Room) stopTimer() {
	room.GameState.StopTimer()
}
